<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

if (file_exists(__DIR__ . '/vendor/autoload.php')) {
    require_once __DIR__ . '/vendor/autoload.php';
}

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Database\Exception\CouldNotInstallModule;
use JustIdea\ExpertSender\Infrastructure\Database\Exception\CouldNotUninstallModule;
use JustIdea\ExpertSender\Infrastructure\Database\Install\Installer;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Uninstaller;
use JustIdea\ExpertSender\Infrastructure\Hook\HookFactoryInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\LoggerInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\ReportLoggerInterface;
use JustIdea\ExpertSender\Infrastructure\Translator\ExceptionTranslatorInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\DirUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\VersionUtility;
use PrestaShop\ModuleLibServiceContainer\DependencyInjection\ServiceContainer;

class ExpertSenderOfficial extends Module
{
    /**
     * @var bool
     */
    public $bootstrap = true;

    /**
     * @var string
     */
    public $confirmUninstall;

    /**
     * @var ServiceContainer
     */
    private $serviceContainer;

    /**
     * @var HookFactoryInterface
     */
    private $hookService;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var ReportLoggerInterface
     */
    private $reportLogger;

    /**
     * @var ExceptionTranslatorInterface
     */
    private $exceptionTranslator;

    /**
     * @var string
     */
    public $template_admin_dir;

    public function __construct()
    {
        $this->name = 'expertsenderofficial';
        $this->tab = 'front_office_features';
        $this->version = '1.0.3';
        $this->displayName = $this->l('Expert Sender Official');
        $this->author = 'JustIdea Agency';
        $this->description = $this->l('This module integrates Prestashop with Expert Sender service.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->template_admin_dir = '../../../../modules/' . $this->name . '/views/templates/admin/';

        parent::__construct();

        if (!VersionUtility::isPhpVersionLessThan('7.2.0')) {
            return;
        }

        $this->autoload();
    }

    /**
     * @param string $serviceName
     *
     * @return mixed
     */
    public function getService($serviceName)
    {
        if (null === $this->serviceContainer) {
            $this->serviceContainer = new ServiceContainer($this->name, $this->getLocalPath());
        }

        return $this->serviceContainer->getService($serviceName);
    }

    /**
     * @return HookFactoryInterface
     */
    public function getHookService()
    {
        if (null === $this->hookService) {
            $this->hookService = $this->getService(HookFactoryInterface::class);
        }

        return $this->hookService;
    }

    /**
     * @return LoggerInterface
     */
    public function getLogger()
    {
        if (null === $this->logger) {
            $this->logger = $this->getService(LoggerInterface::class);
        }

        return $this->logger;
    }

    /**
     * @return ReportLoggerInterface
     */
    public function getReportLogger()
    {
        if (null === $this->reportLogger) {
            $this->reportLogger = $this->getService(ReportLoggerInterface::class);
        }

        return $this->reportLogger;
    }

    /**
     * @return ExceptionTranslatorInterface
     */
    public function getExceptionTranslator()
    {
        if (null === $this->exceptionTranslator) {
            $this->exceptionTranslator = $this->getService(ExceptionTranslatorInterface::class);
        }

        return $this->exceptionTranslator;
    }

    public function install(): bool
    {
        if (!parent::install()) {
            return false;
        }

        if (!DirUtility::copyOverrideFolder($this)) {
            return false;
        }

        /** @var Installer $installer */
        $installer = $this->getService(Installer::class);

        try {
            $installer->init();
        } catch (CouldNotInstallModule $exception) {
            $this->getLogger()->error('Failed to install module.', [
                'context' => [],
                'exceptions' => ExceptionUtility::getExceptions($exception),
            ]);

            $this->_errors[] = $this->getExceptionTranslator()->translate($exception);

            return false;
        }

        return true;
    }

    public function uninstall(): bool
    {
        if (!parent::uninstall()) {
            return false;
        }

        if (!DirUtility::copyOverrideFolder($this)) {
            return false;
        }

        /** @var Uninstaller $uninstaller */
        $uninstaller = $this->getService(Uninstaller::class);

        try {
            $uninstaller->init();

            return true;
        } catch (CouldNotUninstallModule $exception) {
            $this->getLogger()->error('Failed to uninstall module.', [
                'context' => [],
                'exceptions' => ExceptionUtility::getExceptions($exception),
            ]);

            $this->_errors[] = $this->getExceptionTranslator()->translate($exception);

            return false;
        }
    }

    public function enable($force_all = false): bool
    {
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            if (!DirUtility::copyOverrideFolder($this)) {
                return false;
            }
        }

        return (bool) parent::enable($force_all);
    }

    public function disable($force_all = false): bool
    {
        return (bool) parent::disable($force_all);
    }

    public function getContent(): void
    {
        $redirectLink = $this->context->link->getAdminLink(Config::SETTINGS_CONTROLLER);
        Tools::redirectAdmin($redirectLink);
    }

    /**
     * Handled in the actionObjectCustomerAddAfter hook (Customer object).
     */
    public function hookActionCustomerAccountAdd(&$params)
    {
    }

    /**
     * Handled in the actionObjectCustomerUpdateAfter hook (Customer object).
     */
    public function hookActionCustomerAccountUpdate(&$params)
    {
    }

    public function hookActionOrderReturn(&$params)
    {
        /*
         * @TODO Hook unhandled.
         */
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectAddressAddAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectAddressDeleteAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectAddressUpdateAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectCustomerAddressAddAfter(&$params)
    {
        return $this->hookActionObjectAddressAddAfter($params);
    }

    public function hookActionObjectCustomerAddressDeleteAfter(&$params)
    {
        return $this->hookActionObjectAddressDeleteAfter($params);
    }

    public function hookActionObjectCustomerAddressUpdateAfter(&$params)
    {
        return $this->hookActionObjectAddressUpdateAfter($params);
    }

    public function hookActionObjectCustomerAddAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectCustomerDeleteAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectCustomerUpdateAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectCustomerUpdateBefore(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectExpertSenderOrderStateMappingAddAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectExpertSenderOrderStateMappingUpdateAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectOrderAddAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectOrderDeleteAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectOrderUpdateAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    public function hookActionObjectOrderDetailUpdateAfter(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    /**
     * Handled in the actionObjectOrderUpdateAfter hook (Order object).
     */
    public function hookActionOrderHistoryAddAfter(&$params)
    {
    }

    public function hookActionProductCancel(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    /**
     * Handled in the actionObjectAddressUpdateAfter hook (Address object).
     */
    public function hookActionSubmitCustomerAddressForm(&$params)
    {
    }

    public function hookActionFrontControllerSetMedia(&$params)
    {
        return $this->getHookService()->getHook(__FUNCTION__)->exec($params);
    }

    private function autoload(): void
    {
        require_once $this->getLocalPath() . '/vendor/autoload.php';
    }
}
