<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Api\Client;

use JustIdea\ExpertSender\Api\Exception\ApiException;
use JustIdea\ExpertSender\Api\Exception\CurlConnectTimeoutException;
use JustIdea\ExpertSender\Api\Response\ApiResponse;
use JustIdea\ExpertSender\Infrastructure\Exception\ExpertSenderException;

if (!defined('_PS_VERSION_')) {
    exit;
}

class CurlClient implements HttpClientInterface
{
    /**
     * Default response timeout (in seconds).
     */
    private const DEFAULT_TIMEOUT = 8;

    /**
     * Default connect timeout (in seconds).
     */
    private const DEFAULT_CONNECT_TIMEOUT = 2;

    /**
     * The maximum number of retries.
     */
    private const MAX_RETRIES = 5;

    /**
     * The amount of milliseconds the delay is being increased with on each retry.
     */
    private const DELAY_INCREASE_MS = 4000;

    /**
     * @var int
     */
    private $startTime;

    /**
     * @param string $httpMethod
     * @param string $url
     * @param array|string $headers
     * @param string $httpBody
     *
     * @return stdClass|\JsonSerializable|null
     *
     * @throws ApiException
     * @throws CurlConnectTimeoutException
     */
    public function send($httpMethod, $url, $headers = [], $httpBody = [])
    {
        for ($i = 0; $i <= self::MAX_RETRIES; ++$i) {
            $this->startTime = microtime(true);

            usleep($i * self::DELAY_INCREASE_MS);

            try {
                $response = $this->attemptRequest($httpMethod, $url, $headers, $httpBody);
                $statusCode = curl_getinfo($response['curl'], CURLINFO_RESPONSE_CODE);
                curl_close($response['curl']);

                return $this->parseResponseBody($response['content'], $statusCode, $httpBody);
            } catch (CurlConnectTimeoutException $exception) {
                if (self::MAX_RETRIES === $i) {
                    throw CurlConnectTimeoutException::failedToPerformAction($exception, ['max_retires' => self::MAX_RETRIES]);
                }
            }
        }
    }

    /**
     * @param string $httpMethod
     * @param string $url
     * @param array $headers
     * @param array $httpBody
     *
     * @return stdClass|void|\JsonSerializable|null
     *
     * @throws ApiException
     * @throws CurlConnectTimeoutException
     */
    protected function attemptRequest($httpMethod, $url, $headers, $httpBody)
    {
        $curl = curl_init($url);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $this->parseHeaders($headers));
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, self::DEFAULT_CONNECT_TIMEOUT);
        curl_setopt($curl, CURLOPT_TIMEOUT, self::DEFAULT_TIMEOUT);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);

        $this->configureCurlByMethod($curl, $httpMethod, $httpBody);

        $content = curl_exec($curl);

        if (false === $content) {
            $this->handleCurlError($curl);
        }

        return ['curl' => $curl, 'content' => $content];
    }

    protected function configureCurlByMethod($curl, $httpMethod, $httpBody)
    {
        switch ($httpMethod) {
            case HttpClientInterface::HTTP_POST:
                curl_setopt($curl, CURLOPT_POST, true);
                curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($httpBody));
                // curl_setopt($curl, CURLOPT_POSTFIELDS, $httpBody);
                break;
            case HttpClientInterface::HTTP_GET:
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, HttpClientInterface::HTTP_GET);
                curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($httpBody));
                // curl_setopt($curl, CURLOPT_POSTFIELDS, $httpBody); // causes curl_exec freeze

                break;
            case HttpClientInterface::HTTP_PATCH:
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, HttpClientInterface::HTTP_PATCH);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $httpBody);

                break;
            case HttpClientInterface::HTTP_DELETE:
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, HttpClientInterface::HTTP_DELETE);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $httpBody);

                break;
            default:
                throw ExpertSenderException::invalidArgument($httpMethod);
        }
    }

    protected function parseResponseBody($response, $statusCode, $httpBody)
    {
        if (empty($response)) {
            return new ApiResponse($statusCode);
        }

        $body = json_decode($response, true);
        if (JSON_ERROR_NONE !== json_last_error()) {
            return new ApiResponse($statusCode, [], "Unable to decode response: '{$response}'.");
        }

        // Jeśli odpowiedź zawiera kod błędu
        if (isset($body['error_code'])) {
            return new ApiResponse($statusCode, [], $body['error_message'] ?? '');
        }

        // W zależności od statusu kodu, zwracaj odpowiedni model odpowiedzi
        return new ApiResponse($statusCode, $body);
    }

    protected function parseHeaders($headers)
    {
        $result = [];

        foreach ($headers as $key => $value) {
            $result[] = $key . ': ' . $value;
        }

        return $result;
    }

    protected function handleCurlError($curl)
    {
        $errorNumber = curl_errno($curl);
        $errorMessage = curl_error($curl);
        $executionTime = microtime(true) - $this->startTime;

        curl_close($curl);

        if ($this->isConnectTimeoutError($errorNumber, $executionTime)) {
            throw CurlConnectTimeoutException::failedDueToTimeout(null, ['error_message' => $errorMessage, 'error_number' => $errorNumber, 'exec_time' => $executionTime]);
        }

        switch ($errorNumber) {
            case CURLE_OPERATION_TIMEOUTED:
                throw new ApiException("Operation timeout error: $errorMessage");
            default:
                throw new ApiException("Curl error: $errorMessage");
        }
    }

    protected function isConnectTimeoutError($curlErrorNumber, $executionTime)
    {
        $connectErrors = [
            CURLE_COULDNT_RESOLVE_HOST,
            CURLE_COULDNT_CONNECT,
            CURLE_SSL_CONNECT_ERROR,
            CURLE_GOT_NOTHING,
        ];

        if (in_array($curlErrorNumber, $connectErrors)) {
            return true;
        }

        if (CURLE_OPERATION_TIMEOUTED === $curlErrorNumber && $executionTime < self::DEFAULT_TIMEOUT) {
            return true;
        }

        return false;
    }
}
