<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Customer\Action;

use JustIdea\ExpertSender\Api\Enum\HttpStatusCode;
use JustIdea\ExpertSender\Api\Response\AbstractApiResponse;
use JustIdea\ExpertSender\Core\Customer\Api\Request\GetCustomerApiRequestInterface;
use JustIdea\ExpertSender\Core\Customer\DTO\GetCustomerRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Customer\Exception\BadRequestException;
use JustIdea\ExpertSender\Core\Customer\Exception\CustomerProcessException;
use JustIdea\ExpertSender\Core\Customer\Exception\UnauthorizedException;
use JustIdea\ExpertSender\Core\Customer\Model\CustomerData;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Cache\CacheInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\ArrayUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class GetCustomerAction implements ActionInterface
{
    public const FILE_NAME = 'GetCustomerAction';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var CacheInterface
     */
    private $cache;

    /**
     * @var GetCustomerApiRequestInterface
     */
    private $request;

    public function __construct(
        ModuleFactory $moduleFactory,
        CacheInterface $cache,
        GetCustomerApiRequestInterface $request
    ) {
        $this->module = $moduleFactory->getModule();
        $this->cache = $cache;
        $this->request = $request;
    }

    /**
     * @param RequestModel $requestModel
     */
    public function run(\JsonSerializable $requestModel): CustomerData
    {
        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function called', self::FILE_NAME), __METHOD__));

        try {
            /** @var AbstractApiResponse $responseModel */
            $responseModel = $this->request->call($requestModel);
        } catch (\Exception $exception) {
            throw CustomerProcessException::unknownError($exception, ['context' => '', 'request' => $requestModel->jsonSerialize()]);
        }

        switch ($responseModel->getStatusCode()) {
            case HttpStatusCode::HTTP_OK:
            case HttpStatusCode::HTTP_CREATED:
                $this->module->getLogger()->debug(sprintf($this->module->l('%s - Response returned', self::FILE_NAME), __METHOD__), ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);

                break;
            case HttpStatusCode::HTTP_BAD_REQUEST:
                throw new BadRequestException($responseModel->getError(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
            case HttpStatusCode::HTTP_UNAUTHORIZED:
                throw new UnauthorizedException($responseModel->getError(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
            case HttpStatusCode::HTTP_NO_CONTENT:
            case HttpStatusCode::HTTP_NOT_FOUND:
            case HttpStatusCode::HTTP_INTERNAL_SERVER_ERROR:
            default:
                throw new CustomerProcessException($responseModel->getError(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
        }

        $customer = $responseModel->getData()['data'];

        $result = CustomerData::create(
            0,
            (string) $customer['email'],
            isset($customer['phone']) ? (string) $customer['phone'] : null,
            (string) 0,
            (string) 0,
            (string) $customer['firstName'],
            (string) $customer['lastName'],
            isset($customer['dateOfBirth']) ? (string) $customer['dateOfBirth'] : null,
            (string) $customer['gender'],
            CustomerData::createCustomAttributeValueCollection(isset($customer['customAttributes']) ? ArrayUtility::getArray($customer['customAttributes']) : []),
            CustomerData::createConsentsData(isset($customer['consentsData']) ? ArrayUtility::getArray($customer['consentsData']) : [])
        );

        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function ended', self::FILE_NAME), __METHOD__));

        return $result;
    }
}
