<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Customer\Action;

use JustIdea\ExpertSender\Core\Customer\DTO\SaveCustomerConsentSyncedFlagRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Customer\Exception\CustomerProcessException;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderCustomerConsentRepositoryInterface;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\EntityManager\EntityManagerInterface;
use JustIdea\ExpertSender\Infrastructure\EntityManager\ObjectModelUnitOfWork;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SaveCustomerConsentSyncedFlagAction implements ActionInterface
{
    public const FILE_NAME = 'SaveCustomerConsentSyncedFlagAction';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var ExpertSenderCustomerConsentRepositoryInterface
     */
    private $expertSenderCustomerConsentRepository;

    /**
     * @var EntityManagerInterface
     */
    private $entityManager;

    public function __construct(
        ModuleFactory $moduleFactory,
        ExpertSenderCustomerConsentRepositoryInterface $expertSenderCustomerConsentRepository,
        EntityManagerInterface $entityManager
    ) {
        $this->module = $moduleFactory->getModule();
        $this->expertSenderCustomerConsentRepository = $expertSenderCustomerConsentRepository;
        $this->entityManager = $entityManager;
    }

    /**
     * @param RequestModel $requestModel
     */
    public function run(\JsonSerializable $requestModel)
    {
        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function called', self::FILE_NAME), __METHOD__));

        try {
            /** @var \ExpertSenderCustomerConsent */
            $customerConsentEntity = $this->expertSenderCustomerConsentRepository->findOneBy([
                'id_customer' => $requestModel->getCustomerId(),
                'id_field' => $requestModel->getFieldId(),
            ]);

            if (null === $customerConsentEntity) {
                $customerConsentEntity = new \ExpertSenderCustomerConsent();
                $customerConsentEntity->id_customer = $requestModel->getCustomerId();
                $customerConsentEntity->id_field = $requestModel->getFieldId();
            }

            if ($customerConsentEntity->synced !== null && (int) $customerConsentEntity->synced === $requestModel->getSynced()) {
                goto skipAction;
            }

            $customerConsentEntity->synced = $requestModel->getSynced();
            $customerConsentEntity->date_sync = RequestModel::SYNCED === $requestModel->getSynced() ? date('Y-m-d H:i:s') : null;

            $this->entityManager->persist($customerConsentEntity, ObjectModelUnitOfWork::UNIT_OF_WORK_SAVE);
            $this->entityManager->flush();
        } catch (\Exception $exception) {
            throw CustomerProcessException::unknownError($exception, ['context' => '', 'request' => $requestModel->jsonSerialize()]);
        }

        skipAction:
        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function ended', self::FILE_NAME), __METHOD__));
    }
}
