<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Customer\Model;

use JustIdea\ExpertSender\Core\Model\AbstractModel;
use JustIdea\ExpertSender\Infrastructure\Utility\CustomerInfoUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\TimeZoneUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

/** Object model for post /orders method */
class CustomerData extends AbstractModel
{
    /**
     * @var int
     */
    private $id;

    /**
     * @var string
     */
    private $email;

    /**
     * @var ?string
     */
    private $phone;

    /**
     * @var string
     */
    private $crmId;

    /**
     * @var string
     */
    private $visitorId;

    /**
     * @var string
     */
    private $firstName;

    /**
     * @var string
     */
    private $lastName;

    /**
     * @var ?string
     */
    private $dateOfBirth;

    /**
     * @var ?string
     */
    private $gender;

    /**
     * @var CustomAttributeValueCollection
     */
    private $customAttributeValueCollection;

    /**
     * @var ConsentsData
     */
    private $consentsData;

    public function __construct(
        int $id,
        string $email,
        ?string $phone,
        string $crmId,
        string $visitorId,
        string $firstName,
        string $lastName,
        ?string $dateOfBirth,
        string $gender,
        CustomAttributeValueCollection $customAttributeValueCollection,
        ConsentsData $consentsData
    ) {
        $this->id = $id;
        $this->email = $email;
        $this->phone = null !== $phone ? CustomerInfoUtility::convertPhoneToValidFormat($phone) : null;
        $this->crmId = $crmId;
        $this->visitorId = $visitorId;
        $this->firstName = $firstName;
        $this->lastName = $lastName;
        $this->dateOfBirth = null !== $dateOfBirth ? TimeZoneUtility::convertDateTimeToISO8601Format($dateOfBirth) : null;
        $this->gender = $gender;
        $this->customAttributeValueCollection = $customAttributeValueCollection;
        $this->consentsData = $consentsData;
    }

    public function getId(): int
    {
        return $this->id;
    }

    public function getEmail(): string
    {
        return $this->email;
    }

    public function getPhone(): ?string
    {
        return $this->phone;
    }

    public function getCrmId(): string
    {
        return $this->crmId;
    }

    public function getVisitorId(): string
    {
        return $this->visitorId;
    }

    public function getFirstName(): string
    {
        return $this->firstName;
    }

    public function getLastName(): string
    {
        return $this->lastName;
    }

    public function getDateOfBirth(): ?string
    {
        return $this->dateOfBirth;
    }

    public function getGender(): ?string
    {
        return $this->gender;
    }

    public function getCustomAttributeValueCollection(): CustomAttributeValueCollection
    {
        return $this->customAttributeValueCollection;
    }

    public function getConsentsData(): ConsentsData
    {
        return $this->consentsData;
    }

    public function jsonSerialize(): array
    {
        $consentsExist = !empty($this->getConsentsData()->getConsents());

        $return = [
            'email' => $this->getEmail(),
            'phone' => $this->getPhone(),
            'crmId' => $this->getCrmId(),
            // 'visitorId' => $this->getVisitorId(),
            'firstName' => $this->getFirstName(),
            'lastName' => $this->getLastName(),
            'dateOfBirth' => $this->getDateOfBirth(),
            'gender' => $this->getGender(),
            'customAttributes' => $this->getCustomAttributeValueCollection()->jsonSerialize(),
        ];

        if ($consentsExist) {
            $return['consentsData'] = $this->getConsentsData()->jsonSerialize();
        }

        return $return;
    }

    public static function create(
        int $customerId,
        string $email,
        ?string $phone,
        string $crmId,
        string $visitorId,
        string $firstName,
        string $lastName,
        ?string $dateOfBirth,
        ?string $gender,
        CustomAttributeValueCollection $customAttributeValueCollection,
        ConsentsData $consentsData
    ): self {
        return new self(
            $customerId,
            $email,
            $phone,
            $crmId,
            $visitorId,
            $firstName,
            $lastName,
            $dateOfBirth,
            $gender,
            $customAttributeValueCollection,
            $consentsData
        );
    }

    public static function createCustomAttributeValueCollection(array $rawCustomAttributeValueCollection): CustomAttributeValueCollection
    {
        $customAttributeValueCollection = new CustomAttributeValueCollection();
        foreach ($rawCustomAttributeValueCollection as $key => $rawCustomAttributeValue) {
            $customAttributeValue = new CustomAttributeValue(
                (string) $rawCustomAttributeValue['name'],
                (string) $rawCustomAttributeValue['value']
            );
            $customAttributeValueCollection->add($customAttributeValue);
        }

        return $customAttributeValueCollection;
    }

    public static function createConsentsData(array $rawConsentsData): ConsentsData
    {
        $consentsData = ConsentsData::create(
            isset($rawConsentsData['force']) ? (bool) $rawConsentsData['force'] : false,
            isset($rawConsentsData['confirmationMessageId']) ? (int) $rawConsentsData['confirmationMessageId'] : 0
        );

        if (isset($rawConsentsData['consents']) && is_array($rawConsentsData['consents'])) {
            foreach ($rawConsentsData['consents'] as $rawConsentValue) {
                $consentValue = ConsentValue::create(
                    (int) $rawConsentValue['id'],
                    $rawConsentValue['value'],
                    (int) $rawConsentValue['prestaFieldId']
                );
                $consentsData->add($consentValue);
            }
        }

        return $consentsData;
    }
}
