<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Customer\Processor;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Customer\Action\CleanupConsentsForCustomerIdAction;
use JustIdea\ExpertSender\Core\Customer\Action\PostCustomerAction;
use JustIdea\ExpertSender\Core\Customer\Action\SaveCustomerConsentSyncedFlagAction;
use JustIdea\ExpertSender\Core\Customer\Action\UpdateCustomerSyncedFlagAction;
use JustIdea\ExpertSender\Core\Customer\Action\UpdateLastTimeCustomerSyncWasRunAction;
use JustIdea\ExpertSender\Core\Customer\Action\UpdateLastTimeSyncedCustomerIdAction;
use JustIdea\ExpertSender\Core\Customer\DTO\CleanupConsentsForCustomerIdRequestData;
use JustIdea\ExpertSender\Core\Customer\DTO\PostCustomerRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Customer\DTO\SaveCustomerConsentSyncedFlagRequestData;
use JustIdea\ExpertSender\Core\Customer\DTO\UpdateCustomerSyncedFlagRequestData;
use JustIdea\ExpertSender\Core\Customer\DTO\UpdateLastTimeCustomerSyncWasRunRequestData;
use JustIdea\ExpertSender\Core\Customer\DTO\UpdateLastTimeSyncedCustomerIdRequestData;
use JustIdea\ExpertSender\Core\Customer\Exception\BadRequestException;
use JustIdea\ExpertSender\Core\Customer\Exception\CustomerProcessException;
use JustIdea\ExpertSender\Core\Customer\Provider\CustomerProvider as DatabaseCustomerProvider;
use JustIdea\ExpertSender\Core\Enum\CustomerAddingMode;
use JustIdea\ExpertSender\Core\Enum\MatchingMode;
use JustIdea\ExpertSender\Core\Enum\ProcessState;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Logger\LoggerInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\ReportLoggerInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SyncCustomersProcessor implements CustomerProcessorInterface
{
    public const FILE_NAME = 'SyncCustomersProcessor';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var ReportLoggerInterface
     */
    private $reportLogger;

    /**
     * @var DatabaseCustomerProvider
     */
    private $customerProvider;

    /**
     * @var PostCustomerAction
     */
    private $postCustomerAction;

    /**
     * @var UpdateCustomerSyncedFlagAction
     */
    private $updateCustomerSyncedFlagAction;

    /**
     * @var UpdateLastTimeSyncedCustomerIdAction
     */
    private $updateLastTimeSyncedCustomerIdAction;

    /**
     * @var UpdateLastTimeCustomerSyncWasRunAction
     */
    private $updateLastTimeCustomerSyncWasRunAction;

    /**
     * @var SaveCustomerConsentSyncedFlagAction
     */
    private $saveCustomerConsentSyncedFlagAction;

    /**
     * @var CleanupConsentsForCustomerIdAction
     */
    private $cleanupConsentsForCustomerIdAction;

    /**
     * @var int
     */
    private $successCount;

    /**
     * @var array
     */
    private $errorMessages;

    public function __construct(
        ModuleFactory $moduleFactory,
        LoggerInterface $logger,
        ReportLoggerInterface $reportLogger,
        DatabaseCustomerProvider $customerProvider,
        PostCustomerAction $postCustomerAction,
        UpdateCustomerSyncedFlagAction $updateCustomerSyncedFlagAction,
        UpdateLastTimeSyncedCustomerIdAction $updateLastTimeSyncedCustomerIdAction,
        UpdateLastTimeCustomerSyncWasRunAction $updateLastTimeCustomerSyncWasRunAction,
        SaveCustomerConsentSyncedFlagAction $saveCustomerConsentSyncedFlagAction,
        CleanupConsentsForCustomerIdAction $cleanupConsentsForCustomerIdAction
    ) {
        $this->module = $moduleFactory->getModule();
        $this->logger = $logger;
        $this->reportLogger = $reportLogger;
        $this->customerProvider = $customerProvider;
        $this->postCustomerAction = $postCustomerAction;
        $this->updateCustomerSyncedFlagAction = $updateCustomerSyncedFlagAction;
        $this->updateLastTimeSyncedCustomerIdAction = $updateLastTimeSyncedCustomerIdAction;
        $this->updateLastTimeCustomerSyncWasRunAction = $updateLastTimeCustomerSyncWasRunAction;
        $this->saveCustomerConsentSyncedFlagAction = $saveCustomerConsentSyncedFlagAction;
        $this->cleanupConsentsForCustomerIdAction = $cleanupConsentsForCustomerIdAction;

        $this->successCount = 0;
        $this->errorMessages = [];
    }

    /**
     * @param array $settings
     *
     * @return string $result
     */
    public function processAction($settings = [], $isRetry = false): string
    {
        try {
            $rawCustomers = $this->customerProvider->getMany();
        } catch (\Exception $exception) {
            throw CustomerProcessException::unknownError($exception);
        }

        $this->updateLastTimeCustomerSyncWasRunAction->run(UpdateLastTimeCustomerSyncWasRunRequestData::create());

        if (!iterator_count($rawCustomers)) {
            if (\Tools::getValue('action') == 'handleSyncSubmit') {
                $wasRun = (bool) \Configuration::get(Config::SYNC_PROCESS_CUSTOMER_IN_PROGRESS, true);

                if (!$wasRun) {
                    if (!$isRetry && \Configuration::get(Config::LAST_SYNCED_CUSTOMER_ID, 0) !== 0) {
                        $this->updateLastTimeSyncedCustomerIdAction->run(UpdateLastTimeSyncedCustomerIdRequestData::create(0));

                        return $this->processAction($settings, true);
                    } else {
                        \Configuration::updateValue(Config::SYNC_PROCESS_CUSTOMER_IN_PROGRESS, true);
                        \Configuration::updateValue(Config::SYNC_PROCESS_CUSTOMER_FINISHED, true);
                        $this->reportLogger->info($this->module->l('No customer to be synced.', self::FILE_NAME));
                        $this->reportLogger->info($this->module->l('Customer sync process has been completed.', self::FILE_NAME));

                        return ProcessState::NO_AFFECT;
                    }
                } else {
                    if ((bool) \Configuration::get(Config::SYNC_PROCESS_CUSTOMER_FINISHED, true)) {
                        return ProcessState::NO_AFFECT;
                    } else {
                        $this->reportLogger->info($this->module->l('Customer sync process has been completed.', self::FILE_NAME));
                        \Configuration::updateValue(Config::SYNC_PROCESS_CUSTOMER_FINISHED, true);

                        return ProcessState::NO_AFFECT;
                    }
                }
            } else {
                if (!$isRetry && \Configuration::get(Config::LAST_SYNCED_CUSTOMER_ID, 0) !== 0) {
                    $this->updateLastTimeSyncedCustomerIdAction->run(UpdateLastTimeSyncedCustomerIdRequestData::create(0));

                    return $this->processAction($settings, true);
                } else {
                    $this->reportLogger->info($this->module->l('No customer has been synced.', self::FILE_NAME));
                    $this->reportLogger->info($this->module->l('Customer sync process has been completed.', self::FILE_NAME));

                    return ProcessState::NO_AFFECT;
                }
            }
        }
        try {
            foreach ($rawCustomers as $rawCustomer) {
                try {
                    $this->postCustomerAction->run(RequestModel::create(
                        CustomerAddingMode::ADD_OR_UPDATE,
                        MatchingMode::EMAIL,
                        $rawCustomer
                    ));

                    ++$this->successCount;

                    foreach ($rawCustomer->getConsentsData()->getConsents() as $value) {
                        $this->storeCustomerConsentIntoDatabase((int) $rawCustomer->getId(), (int) $value['prestaFieldId'], SaveCustomerConsentSyncedFlagRequestData::SYNCED);
                    }
                    $this->cleanupCustomerConsentsInDatabase((int) $rawCustomer->getId());

                    $this->updateCustomerSyncedFlagAction->run(UpdateCustomerSyncedFlagRequestData::create(
                        (int) $rawCustomer->getId(),
                        UpdateCustomerSyncedFlagRequestData::SYNCED
                    ));
                } catch (BadRequestException $exception) {
                    $this->errorMessages[] = sprintf($this->module->l('Customer ID #%s: %s', self::FILE_NAME), (int) $rawCustomer->getId(), $exception->getMessage());
                    $this->logger->error(sprintf($this->module->l('Failed to sync the Customer with ID #%s', self::FILE_NAME), (int) $rawCustomer->getId()), [
                        'context' => [],
                        'request' => ExceptionUtility::getLastRequest($exception),
                        'response' => ExceptionUtility::getLastResponse($exception),
                        'exceptions' => ExceptionUtility::getExceptions($exception),
                    ]);
                }

                $this->updateLastTimeSyncedCustomerIdAction->run(UpdateLastTimeSyncedCustomerIdRequestData::create(
                    (int) $rawCustomer->getId()
                ));
            }
        } finally {
            if ($this->successCount) {
                $this->reportLogger->success(sprintf($this->module->l('There were %s customer(s) successfully synced to the ECDP.', self::FILE_NAME), (int) $this->successCount));
            } else {
                $this->reportLogger->info($this->module->l('No customer has been successfully synced.', self::FILE_NAME));
            }

            if (!empty($this->errorMessages)) {
                foreach ($this->errorMessages as $error) {
                    $this->reportLogger->error($error);
                }
            }
            \Configuration::updateValue(Config::SYNC_PROCESS_CUSTOMER_IN_PROGRESS, true);
        }

        return ProcessState::SUCCESS;
    }

    /**
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function storeCustomerConsentIntoDatabase(int $customerId, int $fieldId, int $syncFlag)
    {
        $this->saveCustomerConsentSyncedFlagAction->run(SaveCustomerConsentSyncedFlagRequestData::create(
            $customerId,
            $fieldId,
            $syncFlag
        ));
    }

    /**
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function cleanupCustomerConsentsInDatabase(int $customerId)
    {
        $this->cleanupConsentsForCustomerIdAction->run(CleanupConsentsForCustomerIdRequestData::create(
            $customerId
        ));
    }
}
