<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Model;

if (!defined('_PS_VERSION_')) {
    exit;
}

abstract class Collection extends AbstractModel implements CollectionInterface
{
    protected $items = [];

    public function add($item): void
    {
        $this->validateType($item);
        $this->items[] = $item;
    }

    public function getAll(): array
    {
        return $this->items;
    }

    public function get(int $index)
    {
        return isset($this->items[$index]) ? $this->items[$index] : null;
    }

    public function getBy(CollectionCriteria $criteria, bool $parseArray = false): array
    {
        $conditions = $criteria->getConditions();

        $filteredItems = array_filter($this->items, function ($item) use ($conditions) {
            foreach ($conditions as $property => $value) {
                $getter = 'get' . ucfirst($property);
                if (!method_exists($item, $getter) || $item->$getter() !== $value) {
                    return false;
                }
            }

            return true;
        });

        if ($parseArray) {
            return array_map(function ($item) {
                return method_exists($item, 'jsonSerialize') ? $item->jsonSerialize() : (array) $item;
            }, array_values($filteredItems));
        }

        return array_values($filteredItems);
    }

    public function getOneBy(CollectionCriteria $criteria, bool $parseArray = false)
    {
        $conditions = $criteria->getConditions();

        foreach ($this->items as $item) {
            $matches = true;
            foreach ($conditions as $property => $value) {
                $getter = 'get' . ucfirst($property);
                if (!method_exists($item, $getter) || $item->$getter() !== $value) {
                    $matches = false;
                    break;
                }
            }

            if ($matches) {
                return $parseArray ? $item->jsonSerialize() : $item;
            }
        }

        return null;
    }

    public function exists(CollectionCriteria $criteria): bool
    {
        $conditions = $criteria->getConditions();

        foreach ($this->items as $item) {
            $matches = true;
            foreach ($conditions as $property => $value) {
                $getter = 'get' . ucfirst($property);
                if (!method_exists($item, $getter) || $item->$getter() !== $value) {
                    $matches = false;
                    break;
                }
            }

            if ($matches) {
                return true;
            }
        }

        return false;
    }

    public function isEmpty(): bool
    {
        return empty($this->items);
    }

    public function count(): int
    {
        return count($this->items);
    }

    public function jsonSerialize(): array
    {
        $items = $this->getAll();

        $returnArray = [];

        foreach ($items as $key => $item) {
            $returnArray[] = $item->jsonSerialize();
        }

        return $returnArray;
    }

    abstract protected function validateType($item): void;
}
