<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Order\Action;

use JustIdea\ExpertSender\Api\Enum\HttpStatusCode;
use JustIdea\ExpertSender\Api\Exception\ExpertSenderApiException;
use JustIdea\ExpertSender\Api\Response\AbstractApiResponse;
use JustIdea\ExpertSender\Core\Order\Api\Request\GetProductAttributesApiRequestInterface;
use JustIdea\ExpertSender\Core\Order\DTO\GetProductAttributesRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Order\Exception\BadRequestException;
use JustIdea\ExpertSender\Core\Order\Exception\OrderProcessException;
use JustIdea\ExpertSender\Core\Order\Exception\UnauthorizedException;
use JustIdea\ExpertSender\Core\Order\Model\ProductAttributes;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Cache\CacheInterface;
use JustIdea\ExpertSender\Infrastructure\Cache\CacheKey;
use JustIdea\ExpertSender\Infrastructure\Utility\ArrayUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class GetProductAttributesAction implements ActionInterface
{
    public const FILE_NAME = 'GetProductAttributesAction';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var CacheInterface
     */
    private $cache;

    /**
     * @var GetProductAttributesApiRequestInterface
     */
    private $request;

    public function __construct(
        ModuleFactory $moduleFactory,
        CacheInterface $cache,
        GetProductAttributesApiRequestInterface $request
    ) {
        $this->module = $moduleFactory->getModule();
        $this->cache = $cache;
        $this->request = $request;
    }

    /**
     * @param RequestModel $requestModel
     */
    public function run(\JsonSerializable $requestModel): ProductAttributes
    {
        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function called', self::FILE_NAME), __METHOD__));

        $cacheKey = CacheKey::productAttributes();

        if ($this->cache->has($cacheKey)) {
            $result = $this->cache->get($cacheKey);

            return $result;
        }

        try {
            /** @var AbstractApiResponse $responseModel */
            $responseModel = $this->request->call($requestModel);
        } catch (ExpertSenderApiException $exception) {
            throw $exception;
        } catch (\Throwable $exception) {
            throw OrderProcessException::unknownError($exception, ['context' => '', 'request' => $requestModel->jsonSerialize()]);
        }

        switch ($responseModel->getStatusCode()) {
            case HttpStatusCode::HTTP_OK:
            case HttpStatusCode::HTTP_CREATED:
                $this->module->getLogger()->debug(sprintf($this->module->l('%s - Response returned', self::FILE_NAME), __METHOD__), ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);

                break;
            case HttpStatusCode::HTTP_BAD_REQUEST:
                throw new BadRequestException($responseModel->getError(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
            case HttpStatusCode::HTTP_UNAUTHORIZED:
                throw new UnauthorizedException($responseModel->getError(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
            case HttpStatusCode::HTTP_NO_CONTENT:
            case HttpStatusCode::HTTP_NOT_FOUND:
            case HttpStatusCode::HTTP_INTERNAL_SERVER_ERROR:
            default:
                throw new OrderProcessException($responseModel->getError(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
        }

        $consents = $responseModel->getData()['data'];

        $result = ProductAttributes::create(
            ProductAttributes::createProductAttributeDataCollection(is_array($consents) ? ArrayUtility::getArray($consents) : [])
        );

        $this->cache->set($cacheKey, $result, self::CACHE_LIFETIME);

        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function ended', self::FILE_NAME), __METHOD__));

        return $result;
    }
}
