<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Order\Action;

use JustIdea\ExpertSender\Api\Enum\HttpStatusCode;
use JustIdea\ExpertSender\Api\Response\AbstractApiResponse;
use JustIdea\ExpertSender\Core\Order\Api\Request\PostOrderApiRequestInterface;
use JustIdea\ExpertSender\Core\Order\DTO\PostOrderApiRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Order\Exception\BadRequestException;
use JustIdea\ExpertSender\Core\Order\Exception\MissingWebsiteIdException;
use JustIdea\ExpertSender\Core\Order\Exception\OrderProcessException;
use JustIdea\ExpertSender\Core\Order\Exception\UnauthorizedException;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;

if (!defined('_PS_VERSION_')) {
    exit;
}

class PostOrderAction implements ActionInterface
{
    public const FILE_NAME = 'PostOrderAction';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var PostOrderApiRequestInterface
     */
    private $request;

    public function __construct(
        ModuleFactory $moduleFactory,
        PostOrderApiRequestInterface $request
    ) {
        $this->module = $moduleFactory->getModule();
        $this->request = $request;
    }

    /**
     * @param RequestModel $requestModel
     *
     * @throws OrderProcessException
     */
    public function run(\JsonSerializable $requestModel)
    {
        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function called', self::FILE_NAME), __METHOD__));

        try {
            /** @var AbstractApiResponse $responseModel */
            $responseModel = $this->request->call($requestModel);
        } catch (MissingWebsiteIdException $exception) {
            throw $exception;
        } catch (\Exception $exception) {
            throw OrderProcessException::unknownError($exception, ['context' => '', 'request' => $requestModel->jsonSerialize()]);
        }

        switch ($responseModel->getStatusCode()) {
            case HttpStatusCode::HTTP_OK:
            case HttpStatusCode::HTTP_CREATED:
                $this->module->getLogger()->debug(sprintf($this->module->l('%s - Response returned', self::FILE_NAME), __METHOD__), ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);

                break;
            case HttpStatusCode::HTTP_BAD_REQUEST:
                throw new BadRequestException($responseModel->getErrorDetails(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
            case HttpStatusCode::HTTP_UNAUTHORIZED:
                throw new UnauthorizedException($responseModel->getErrorDetails(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
            case HttpStatusCode::HTTP_NO_CONTENT:
            case HttpStatusCode::HTTP_NOT_FOUND:
            case HttpStatusCode::HTTP_INTERNAL_SERVER_ERROR:
            default:
                throw new OrderProcessException($responseModel->getErrorDetails(), $responseModel->getStatusCode(), null, ['context' => '', 'request' => $requestModel->jsonSerialize(), 'response' => $responseModel->jsonSerialize()]);
        }

        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function ended', self::FILE_NAME), __METHOD__));
    }
}
