<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Order\Action;

use JustIdea\ExpertSender\Core\Order\DTO\SaveOrderStateRelationRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Order\Exception\OrderProcessException;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderOrderStateMappingRelationRepositoryInterface;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\EntityManager\EntityManagerInterface;
use JustIdea\ExpertSender\Infrastructure\EntityManager\ObjectModelUnitOfWork;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SaveOrderStateRelationAction implements ActionInterface
{
    public const FILE_NAME = 'SaveOrderStateRelationAction';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var ExpertSenderOrderStateMappingRelationRepositoryInterface
     */
    private $orderStateMappingRelationRepository;

    /**
     * @var EntityManagerInterface
     */
    private $entityManager;

    public function __construct(
        ModuleFactory $moduleFactory,
        ExpertSenderOrderStateMappingRelationRepositoryInterface $orderStateMappingRelationRepository,
        EntityManagerInterface $entityManager
    ) {
        $this->module = $moduleFactory->getModule();
        $this->orderStateMappingRelationRepository = $orderStateMappingRelationRepository;
        $this->entityManager = $entityManager;
    }

    /**
     * @param RequestModel $requestModel
     */
    public function run(\JsonSerializable $requestModel)
    {
        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function called', self::FILE_NAME), __METHOD__));

        try {
            /** @var \ExpertSenderOrderStateMappingRelation[] */
            $allCurrentRelations = $this->orderStateMappingRelationRepository->findBy(['id_order_state_mapping' => $requestModel->getOrderStateMappingId()]);

            $currentOrderStates = [];
            foreach ($allCurrentRelations as $relation) {
                $currentOrderStates[$relation->id_order_state] = $relation;
            }

            foreach ($allCurrentRelations as $relation) {
                if (!in_array($relation->id_order_state, $requestModel->getOrderStateIds())) {
                    $this->entityManager->persist($relation, ObjectModelUnitOfWork::UNIT_OF_WORK_DELETE);
                }
            }

            foreach ($requestModel->getOrderStateIds() as $orderStateId) {
                if (!isset($currentOrderStates[$orderStateId])) {
                    /** @var \ExpertSenderOrderStateMappingRelation */
                    $entity = $this->orderStateMappingRelationRepository->findOneBy(['id_order_state' => $orderStateId]);

                    if (null === $entity) {
                        $entity = new \ExpertSenderOrderStateMappingRelation();
                        $entity->id_order_state = $orderStateId;
                    }

                    if ($entity->id_order_state_mapping !== null && (int) $entity->id_order_state_mapping === $requestModel->getOrderStateMappingId()) {
                        continue;
                    }

                    $entity->id_order_state_mapping = $requestModel->getOrderStateMappingId();
                    $this->entityManager->persist($entity, ObjectModelUnitOfWork::UNIT_OF_WORK_SAVE);
                }
            }

            $this->entityManager->flush();
        } catch (\Exception $exception) {
            throw OrderProcessException::unknownError($exception, ['context' => '', 'request' => $requestModel->jsonSerialize()]);
        }

        $this->module->getLogger()->debug(sprintf($this->module->l('%s - Function ended', self::FILE_NAME), __METHOD__));
    }
}
