<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Order\Model;

use JustIdea\ExpertSender\Core\Model\AbstractModel;
use JustIdea\ExpertSender\Infrastructure\Utility\ArrayUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\TimeZoneUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

/** Object model for post /orders method */
class OrderData extends AbstractModel
{
    /**
     * @var string
     */
    private $id;

    /**
     * @var string
     */
    private $date;

    /**
     * @var string|null
     */
    private $status;

    /**
     * @var float
     */
    private $totalValue;

    /**
     * @var OrderCustomerData
     */
    private $orderCustomerData;

    /**
     * @var OrderProductDataCollection
     */
    private $orderProductDataCollection;

    /**
     * @var string
     */
    private $timeZone;

    /**
     * @var int
     */
    private $websiteId;

    /**
     * @var string
     */
    private $currency;

    /**
     * @var float
     */
    private $returnsValue;

    /**
     * @var OrderAttributeValueCollection
     */
    private $orderAttributeValueCollection;

    public function __construct(
        string $id,
        string $date,
        ?string $status,
        float $totalValue,
        OrderCustomerData $orderCustomerData,
        OrderProductDataCollection $orderProductDataCollection,
        string $timeZone,
        int $websiteId,
        string $currency,
        float $returnsValue,
        OrderAttributeValueCollection $orderAttributeValueCollection
    ) {
        $this->id = $id;
        $this->date = TimeZoneUtility::convertDateTimeToISO8601Format($date);
        $this->status = $status;
        $this->totalValue = $totalValue;
        $this->orderCustomerData = $orderCustomerData;
        $this->orderProductDataCollection = $orderProductDataCollection;
        $this->timeZone = $timeZone;
        $this->websiteId = $websiteId;
        $this->currency = $currency;
        $this->returnsValue = $returnsValue;
        $this->orderAttributeValueCollection = $orderAttributeValueCollection;
    }

    public function getId(): string
    {
        return $this->id;
    }

    public function getDate(): string
    {
        return $this->date;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function getTotalValue(): float
    {
        return $this->totalValue;
    }

    public function getOrderCustomerData(): OrderCustomerData
    {
        return $this->orderCustomerData;
    }

    public function getProducts(): OrderProductDataCollection
    {
        return $this->orderProductDataCollection;
    }

    public function getTimeZone(): string
    {
        return $this->timeZone;
    }

    public function getWebsiteId(): int
    {
        return $this->websiteId;
    }

    public function getCurrency(): string
    {
        return $this->currency;
    }

    public function getReturnsValue(): float
    {
        return $this->returnsValue;
    }

    public function getOrderAttributeValueCollection(): OrderAttributeValueCollection
    {
        return $this->orderAttributeValueCollection;
    }

    public function jsonSerialize(): array
    {
        return [
            'id' => $this->getId(),
            'date' => $this->getDate(),
            'status' => $this->getStatus(),
            'totalValue' => $this->getTotalValue(),
            'customer' => $this->getOrderCustomerData()->jsonSerialize(),
            'products' => $this->getProducts()->jsonSerialize(),
            'timeZone' => $this->getTimeZone(),
            'websiteId' => $this->getWebsiteId(),
            'currency' => $this->getCurrency(),
            'returnsValue' => $this->getReturnsValue(),
            'orderAttributes' => $this->getOrderAttributeValueCollection()->jsonSerialize(),
        ];
    }

    public static function create(
        string $id,
        string $date,
        ?string $status,
        float $totalValue,
        OrderCustomerData $orderCustomerData,
        OrderProductDataCollection $orderProductDataCollection,
        string $timeZone,
        int $websiteId,
        string $currency,
        float $returnsValue,
        OrderAttributeValueCollection $orderAttributeValueCollection
    ): self {
        return new self(
            $id,
            $date,
            $status,
            $totalValue,
            $orderCustomerData,
            $orderProductDataCollection,
            $timeZone,
            $websiteId,
            $currency,
            $returnsValue,
            $orderAttributeValueCollection
        );
    }

    public static function createOrderCustomerData(array $orderOrderCustomerData): OrderCustomerData
    {
        $customer = new OrderCustomerData(
            (string) $orderOrderCustomerData['email'],
            (string) $orderOrderCustomerData['phone'],
            (string) $orderOrderCustomerData['crmId']
        );

        return $customer;
    }

    public static function createOrderProductDataCollection(array $orderOrderProductDataCollection): OrderProductDataCollection
    {
        $products = new OrderProductDataCollection();
        foreach ($orderOrderProductDataCollection as $key => $orderOrderProductData) {
            $product = new OrderProductData(
                (string) $orderOrderProductData['id'],
                (string) $orderOrderProductData['name'],
                (float) $orderOrderProductData['price'],
                (int) $orderOrderProductData['quantity'],
                (int) $orderOrderProductData['returned'],
                (string) $orderOrderProductData['url'],
                (string) $orderOrderProductData['imageUrl'],
                (string) $orderOrderProductData['category'],
                self::createProductAttributeValueCollection(isset($orderOrderProductData['productAttributes']) ? ArrayUtility::getArray($orderOrderProductData['productAttributes']) : [])
            );
            $products->add($product);
        }

        return $products;
    }

    public static function createProductAttributeValueCollection(array $orderProductAttributeValueCollection): ProductAttributeValueCollection
    {
        $productAttributeValueCollection = new ProductAttributeValueCollection();

        // @TODO Implement product assosiations validation. Sending incorrect attribute name causes api errors.
        foreach ($orderProductAttributeValueCollection as $key => $orderProductAttributeValue) {
            $productAttributeValue = new ProductAttributeValue(
                (string) $orderProductAttributeValue['name'],
                (string) $orderProductAttributeValue['value']
            );
            $productAttributeValueCollection->add($productAttributeValue);
        }

        return $productAttributeValueCollection;
    }

    public static function createOrderAttributeValueCollection(array $orderOrderAttributeValueCollection): OrderAttributeValueCollection
    {
        $orderAttributeValueCollection = new OrderAttributeValueCollection();

        // @TODO Implement order assosiations validation. Sending incorrect attribute name causes api errors.
        foreach ($orderOrderAttributeValueCollection as $key => $orderOrderAttributeValue) {
            $orderAttributeValue = new OrderAttributeValue(
                (string) $orderOrderAttributeValue['name'],
                (string) $orderOrderAttributeValue['value']
            );
            $orderAttributeValueCollection->add($orderAttributeValue);
        }

        return $orderAttributeValueCollection;
    }
}
