<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Order\Processor;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Enum\OrderAddingMode;
use JustIdea\ExpertSender\Core\Enum\ProcessState;
use JustIdea\ExpertSender\Core\Order\Action\PostOrderAction;
use JustIdea\ExpertSender\Core\Order\Action\UpdateLastTimeOrderSyncWasRunAction;
use JustIdea\ExpertSender\Core\Order\Action\UpdateLastTimeSyncedOrderIdAction;
use JustIdea\ExpertSender\Core\Order\Action\UpdateOrderSyncedFlagAction;
use JustIdea\ExpertSender\Core\Order\DTO\PostOrderApiRequestData as RequestModel;
use JustIdea\ExpertSender\Core\Order\DTO\UpdateLastTimeOrderSyncWasRunRequestData;
use JustIdea\ExpertSender\Core\Order\DTO\UpdateLastTimeSyncedOrderIdRequestData;
use JustIdea\ExpertSender\Core\Order\DTO\UpdateOrderSyncedFlagRequestData;
use JustIdea\ExpertSender\Core\Order\Exception\BadRequestException;
use JustIdea\ExpertSender\Core\Order\Exception\OrderProcessException;
use JustIdea\ExpertSender\Core\Order\Model\OrderData;
use JustIdea\ExpertSender\Core\Order\Provider\OrderProvider as DatabaseOrderProvider;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Logger\LoggerInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\ReportLoggerInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SyncOrdersProcessor implements OrderProcessorInterface
{
    public const FILE_NAME = 'SyncOrdersProcessor';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var ReportLoggerInterface
     */
    private $reportLogger;

    /**
     * @var DatabaseOrderProvider
     */
    private $orderProvider;

    /**
     * @var PostOrderAction
     */
    private $postOrderAction;

    /**
     * @var UpdateOrderSyncedFlagAction
     */
    private $updateOrderSyncedFlagAction;

    /**
     * @var UpdateLastTimeSyncedOrderIdAction
     */
    private $updateLastTimeSyncedOrderIdAction;

    /**
     * @var UpdateLastTimeOrderSyncWasRunAction
     */
    private $updateLastTimeOrderSyncWasRunAction;

    /**
     * @var int
     */
    private $successCount;

    /**
     * @var array
     */
    private $errorMessages;

    public function __construct(
        ModuleFactory $moduleFactory,
        LoggerInterface $logger,
        ReportLoggerInterface $reportLogger,
        DatabaseOrderProvider $orderProvider,
        PostOrderAction $postOrderAction,
        UpdateOrderSyncedFlagAction $updateOrderSyncedFlagAction,
        UpdateLastTimeSyncedOrderIdAction $updateLastTimeSyncedOrderIdAction,
        UpdateLastTimeOrderSyncWasRunAction $updateLastTimeOrderSyncWasRunAction
    ) {
        $this->module = $moduleFactory->getModule();
        $this->logger = $logger;
        $this->reportLogger = $reportLogger;
        $this->orderProvider = $orderProvider;
        $this->postOrderAction = $postOrderAction;
        $this->updateOrderSyncedFlagAction = $updateOrderSyncedFlagAction;
        $this->updateLastTimeSyncedOrderIdAction = $updateLastTimeSyncedOrderIdAction;
        $this->updateLastTimeOrderSyncWasRunAction = $updateLastTimeOrderSyncWasRunAction;
        $this->successCount = 0;
        $this->errorMessages = [];
    }

    /**
     * @param array $settings
     *
     * @return string $result
     */
    public function processAction($settings = [], $isRetry = false): string
    {
        try {
            /** @var array<OrderData> */
            $rawOrders = $this->orderProvider->getMany();
        } catch (\Exception $exception) {
            throw OrderProcessException::unknownError($exception);
        }

        $this->updateLastTimeOrderSyncWasRunAction->run(UpdateLastTimeOrderSyncWasRunRequestData::create());

        if (!iterator_count($rawOrders)) {
            if (\Tools::getValue('action') == 'handleSyncSubmit') {
                $wasRun = (bool) \Configuration::get(Config::SYNC_PROCESS_ORDER_IN_PROGRESS, true);

                if (!$wasRun) {
                    if (!$isRetry && \Configuration::get(Config::LAST_SYNCED_ORDER_ID, 0) !== 0) {
                        $this->updateLastTimeSyncedOrderIdAction->run(UpdateLastTimeSyncedOrderIdRequestData::create(0));

                        return $this->processAction($settings, true);
                    } else {
                        \Configuration::updateValue(Config::SYNC_PROCESS_ORDER_IN_PROGRESS, true);
                        \Configuration::updateValue(Config::SYNC_PROCESS_ORDER_FINISHED, true);
                        $this->reportLogger->info($this->module->l('No order to be synced.', self::FILE_NAME));
                        $this->reportLogger->info($this->module->l('Order sync process has been completed.', self::FILE_NAME));

                        return ProcessState::NO_AFFECT;
                    }
                } else {
                    if ((bool) \Configuration::get(Config::SYNC_PROCESS_ORDER_FINISHED, true)) {
                        return ProcessState::NO_AFFECT;
                    } else {
                        $this->reportLogger->info($this->module->l('Order sync process has been completed.', self::FILE_NAME));
                        \Configuration::updateValue(Config::SYNC_PROCESS_ORDER_FINISHED, true);

                        return ProcessState::NO_AFFECT;
                    }
                }
            } else {
                if (!$isRetry && \Configuration::get(Config::LAST_SYNCED_ORDER_ID, 0) !== 0) {
                    $this->updateLastTimeSyncedOrderIdAction->run(UpdateLastTimeSyncedOrderIdRequestData::create(0));

                    return $this->processAction($settings, true);
                } else {
                    $this->reportLogger->info($this->module->l('No order has been synced.', self::FILE_NAME));
                    $this->reportLogger->info($this->module->l('Order sync process has been completed.', self::FILE_NAME));

                    return ProcessState::NO_AFFECT;
                }
            }
        }

        try {
            foreach ($rawOrders as $rawOrder) {
                try {
                    $this->postOrderAction->run(RequestModel::create(
                        OrderAddingMode::ADD_OR_REPLACE,
                        $rawOrder
                    ));

                    ++$this->successCount;

                    $this->updateOrderSyncedFlagAction->run(UpdateOrderSyncedFlagRequestData::create(
                        (int) $rawOrder->getId(),
                        UpdateOrderSyncedFlagRequestData::SYNCED
                    ));
                } catch (BadRequestException $exception) {
                    $this->errorMessages[] = sprintf($this->module->l('Order ID #%s: %s', self::FILE_NAME), (int) $rawOrder->getId(), $exception->getMessage());
                    $this->logger->error(sprintf($this->module->l('Failed to sync the Order with ID #%s', self::FILE_NAME), (int) $rawOrder->getId()), [
                        'context' => [],
                        'request' => ExceptionUtility::getLastRequest($exception),
                        'response' => ExceptionUtility::getLastResponse($exception),
                        'exceptions' => ExceptionUtility::getExceptions($exception),
                    ]);
                }

                $this->updateLastTimeSyncedOrderIdAction->run(UpdateLastTimeSyncedOrderIdRequestData::create(
                    (int) $rawOrder->getId()
                ));
            }
        } finally {
            if ($this->successCount) {
                $this->reportLogger->success(sprintf($this->module->l('There were %s order(s) successfully synced to the ECDP.', self::FILE_NAME), (int) $this->successCount));
            } else {
                $this->reportLogger->info($this->module->l('No order has been successfully synced.', self::FILE_NAME));
            }

            if (!empty($this->errorMessages)) {
                foreach ($this->errorMessages as $error) {
                    $this->reportLogger->error($error);
                }
            }
            \Configuration::updateValue(Config::SYNC_PROCESS_ORDER_IN_PROGRESS, true);
        }

        return ProcessState::SUCCESS;
    }
}
