<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Order\Provider\Queryable;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Order\Model\OrderData;
use JustIdea\ExpertSender\Core\Queryable\CommonQueryable;
use JustIdea\ExpertSender\Core\Repository\DatabaseCriteria;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderOrderRepositoryInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\ArrayUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\TimeZoneUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class OrderQueryable extends CommonQueryable
{
    public const FILE_NAME = 'OrderQueryable';

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var ExpertSenderOrderRepositoryInterface
     */
    private $expertSenderOrderRepository;

    public function __construct(
        \ExpertSenderOfficial $module,
        ExpertSenderOrderRepositoryInterface $expertSenderOrderRepository
    ) {
        $this->module = $module;
        $this->expertSenderOrderRepository = $expertSenderOrderRepository;
    }

    protected function fetchData(): array
    {
        $lastSyncedOrderId = (int) \Configuration::get(Config::LAST_SYNCED_ORDER_ID, 0);

        return $this->expertSenderOrderRepository->getFullOrderDetailsWithCriteria((new DatabaseCriteria())
                ->where('eso.id_order', $lastSyncedOrderId, '>')
                ->where('eso.synced', 0, '='));
    }

    protected function processData(array $data): \Traversable
    {
        $orders = isset($data) ? $data : [];

        if (!count($orders)) {
            return [];
        }

        $this->module->getLogger()->debug(sprintf($this->module->l('There are %s orders to be synced.', self::FILE_NAME), count($orders)));

        foreach ($orders as $order) {
            yield OrderData::create(
                (string) $order['id'],
                (string) $order['date'],
                isset($order['status']) ? $order['status'] : null,
                (float) $order['totalValue'],
                OrderData::createOrderCustomerData(ArrayUtility::getArray($order['customer'])),
                OrderData::createOrderProductDataCollection(ArrayUtility::getArray($order['products'])),
                (string) ($order['timeZone'] ?? TimeZoneUtility::DEFAULT_TIME_ZONE),
                (int) \Configuration::get(Config::WEBSITE_ID),
                (string) $order['currency'],
                (float) $order['returnsValue'],
                OrderData::createOrderAttributeValueCollection(isset($order['orderAttributes']) ? ArrayUtility::getArray($order['orderAttributes']) : [])
            );
        }
    }
}
