<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Repository;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Customer\Model\ConsentsData;
use JustIdea\ExpertSender\Core\Shared\Mappings\ConsentAttribute\ConsentAttributeFactoryInterface;
use JustIdea\ExpertSender\Core\Shared\Mappings\CustomerAttribute\CustomerAttributeFactoryInterface;
use JustIdea\ExpertSender\Infrastructure\Repository\CollectionRepository;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ExpertSenderCustomerRepository extends CollectionRepository implements ExpertSenderCustomerRepositoryInterface
{
    /**
     * @var ConsentAttributeFactoryInterface
     */
    private $consentService;

    /**
     * @var CustomerAttributeFactoryInterface
     */
    private $customerService;

    /**
     * @var ExpertSenderConsentMappingRepositoryInterface
     */
    private $consentMappingRepository;

    /**
     * @var ExpertSenderCustomerMappingRepositoryInterface
     */
    private $customerAttributeMappingRepository;

    /**
     * @var Link
     */
    protected $link;

    /**
     * @var int
     */
    protected $langId;

    /**
     * @var bool
     */
    protected $includePhone;

    public function __construct(
        ConsentAttributeFactoryInterface $consentService,
        CustomerAttributeFactoryInterface $customerService,
        ExpertSenderConsentMappingRepositoryInterface $consentMappingRepository,
        ExpertSenderCustomerMappingRepositoryInterface $customerAttributeMappingRepository,
        \Link $link,
        int $langId
    ) {
        $this->consentService = $consentService;
        $this->customerService = $customerService;
        $this->consentMappingRepository = $consentMappingRepository;
        $this->customerAttributeMappingRepository = $customerAttributeMappingRepository;
        $this->link = $link;
        $this->langId = $langId;

        $this->includePhone = (bool) \Configuration::get(Config::SYNC_INCLUDE_PHONE);

        parent::__construct(\ExpertSenderCustomer::class);
    }

    public function getFullCustomerDetailsWithCriteria(DatabaseCriteria $criteria)
    {
        return $this->getFullCustomers($criteria);
    }

    private function getFullCustomers(DatabaseCriteria $criteria)
    {
        $customersData = $this->fetchCustomersData($criteria);
        if (empty($customersData)) {
            return [];
        }

        $this->fetchCustomerAttribute($customersData);
        $this->fetchConsentsData($customersData);

        return $customersData;
    }

    private function fetchConsentsData(array &$customersData)
    {
        foreach ($customersData as &$customerData) {
            $customerData['consentsData'] = [];
            $customerData['consentsData']['consents'] = [];
            $customerData['consentsData']['force'] = false;
            $customerData['consentsData']['confirmationMessageId'] = null;
        }

        /** @var \ExpertSenderConsentMapping */
        $mappings = $this->consentMappingRepository->findBy(['active' => 1]);

        foreach ($mappings as $mapping) {
            foreach ($customersData as &$customerData) {
                /** @var ConsentsData $result */
                $result = $this->consentService->getAttribute((int) $mapping->id_field)->fetch([
                    'object' => $mapping,
                    'id_customer' => $customerData['id'],
                ]);

                if ($result instanceof ConsentsData) {
                    $customerData['consentsData']['consents'] = array_merge($customerData['consentsData']['consents'], $result->getConsents());
                    if ($result->getForce()) {
                        $customerData['consentsData']['force'] = $result->getForce();
                    }
                    if ($result->getConfirmationMessageId()) {
                        $customerData['consentsData']['confirmationMessageId'] = $result->getConfirmationMessageId();
                    }
                }
            }
        }

        foreach ($customersData as &$customerData) {
            if (empty($customerData['consentsData']['consents'])) {
                unset($customerData['consentsData']);
            }
        }
    }

    private function fetchCustomerAttribute(array &$customersData)
    {
        /** @var \ExpertSenderCustomerMapping */
        $mappings = $this->customerAttributeMappingRepository->findBy(['active' => 1]);

        foreach ($mappings as $mapping) {
            foreach ($customersData as &$customerData) {
                $result = $this->customerService->getAttribute((int) $mapping->id_field)->fetch([
                    'object' => $mapping,
                    'id_customer' => $customerData['id'],
                ]);

                if ($result instanceof \JsonSerializable) {
                    $customerData['customerAttributes'][] = $result->jsonSerialize();
                }
            }
        }
    }

    private function fetchCustomersData(DatabaseCriteria $criteria)
    {
        $query = new \DbQuery();
        $query->select('
                c.id_customer AS id,
                c.email AS email,
                c.id_customer AS crmId,
                c.id_customer AS visitorId,
                c.firstname AS firstName,
                c.lastname AS lastName,
                c.birthday AS dateOfBirth,
                c.id_gender AS genderId');
        if ($this->includePhone) {
            $query->select('
                (SELECT addr.phone FROM ' . _DB_PREFIX_ . 'address addr
                    WHERE addr.id_customer = esc.id_customer
                    AND addr.phone != ""
                    ORDER BY addr.date_add DESC
                    LIMIT 1
                ) AS customerPhone');
        }
        $query->from(\ExpertSenderCustomer::$definition['table'], 'esc')
            ->innerJoin('customer', 'c', 'esc.id_customer = c.id_customer')
            ->where($criteria->toSqlWhereClause())
            ->groupBy('esc.id_customer')
            ->limit($criteria->getLimit() ?? $this->batchSize, $criteria->getStart() ?? 0);

        return \Db::getInstance()->executeS($query);
    }

    /**
     * @return int customers in queue
     */
    public function getTotalRemainingCustomersInQueue(): int
    {
        $query = new \DbQuery();

        $query->select('COUNT(*)')
              ->from(\ExpertSenderCustomer::$definition['table'], 'esc')
              ->where('esc.synced = 0');

        $result = \Db::getInstance()->getValue($query);

        return (int) $result;
    }

    /**
     * @param DatabaseCriteria $criteria
     *
     * @return array customerIds
     */
    public function importCustomersByDateRange(DatabaseCriteria $criteria): array
    {
        $query = new \DbQuery();
        $query->select('c.id_customer')
              ->from('customer', 'c')
              ->where($criteria->toSqlWhereClause());

        $results = \Db::getInstance()->executeS($query);

        if (empty($results)) {
            return 0;
        }

        $customerIds = array_column($results, 'id_customer');

        $this->batchInsertCustomersWithDateRange($customerIds);

        return $customerIds;
    }

    private function batchInsertCustomersWithDateRange(array $customerIds): int
    {
        $customersToInsert = array_map(function ($customerId) {
            return [
                'id_customer' => $customerId,
                'synced' => 0,
            ];
        }, $customerIds);

        $existingCustomerIds = $this->getExistingCustomerIds();
        $customersToInsert = array_filter($customersToInsert, function ($customer) use ($existingCustomerIds) {
            return !in_array($customer['id_customer'], $existingCustomerIds);
        });

        $batches = array_chunk($customersToInsert, $this->batchSize);

        $affectedRows = 0;
        foreach ($batches as $batch) {
            if (\Db::getInstance()->insert(\ExpertSenderCustomer::$definition['table'], $batch)) {
                $affectedRows += \Db::getInstance()->Affected_Rows();
            }
        }

        return $affectedRows;
    }

    private function getExistingCustomerIds()
    {
        $query = new \DbQuery();
        $query->select('id_customer')
            ->from(\ExpertSenderCustomer::$definition['table']);

        $results = \Db::getInstance()->executeS($query);

        return array_column($results, 'id_customer');
    }
}
