<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Repository;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Shared\Mappings\OrderAttribute\OrderAttributeFactoryInterface;
use JustIdea\ExpertSender\Core\Shared\Mappings\OrderState\OrderStateFactoryInterface;
use JustIdea\ExpertSender\Core\Shared\Mappings\ProductAttribute\ProductAttributeFactoryInterface;
use JustIdea\ExpertSender\Infrastructure\Repository\CollectionRepository;

if (!defined('_PS_VERSION_')) {
    exit;
}
class ExpertSenderOrderRepository extends CollectionRepository implements ExpertSenderOrderRepositoryInterface
{
    /**
     * @var ProductAttributeFactoryInterface
     */
    private $productAttributeService;

    /**
     * @var OrderAttributeFactoryInterface
     */
    private $orderAttributeService;

    /**
     * @var OrderStateFactoryInterface
     */
    private $orderStateService;

    /**
     * @var ExpertSenderProductMappingRepositoryInterface
     */
    private $productMappingRepository;

    /**
     * @var ExpertSenderOrderMappingRepositoryInterface
     */
    private $orderAttributeMappingRepository;

    /**
     * @var ExpertSenderOrderStateMappingRepositoryInterface
     */
    private $orderStateMappingRepository;

    /**
     * @var ExpertSenderOrderStateMappingRelationRepositoryInterface
     */
    private $orderStateMappingRelationRepository;

    /**
     * @var Link
     */
    protected $link;

    /**
     * @var int
     */
    protected $langId;

    /**
     * @var \ExpertSenderProductMapping
     */
    private $productMappings;

    /**
     * @var bool
     */
    protected $includePhone;

    public function __construct(
        ProductAttributeFactoryInterface $productAttributeService,
        OrderAttributeFactoryInterface $orderAttributeService,
        OrderStateFactoryInterface $orderStateService,
        ExpertSenderProductMappingRepositoryInterface $productMappingRepository,
        ExpertSenderOrderMappingRepositoryInterface $orderAttributeMappingRepository,
        ExpertSenderOrderStateMappingRepositoryInterface $orderStateMappingRepository,
        ExpertSenderOrderStateMappingRelationRepositoryInterface $orderStateMappingRelationRepository,
        \Link $link,
        int $langId
    ) {
        $this->productAttributeService = $productAttributeService;
        $this->orderAttributeService = $orderAttributeService;
        $this->orderStateService = $orderStateService;
        $this->productMappingRepository = $productMappingRepository;
        $this->orderAttributeMappingRepository = $orderAttributeMappingRepository;
        $this->orderStateMappingRepository = $orderStateMappingRepository;
        $this->orderStateMappingRelationRepository = $orderStateMappingRelationRepository;
        $this->link = $link;
        $this->langId = $langId;

        $this->includePhone = (bool) \Configuration::get(Config::SYNC_INCLUDE_PHONE);

        $this->productMappings = $this->productMappingRepository->findBy(['active' => 1]);

        parent::__construct(\ExpertSenderOrder::class);
    }

    public function getFullOrderDetailsWithCriteria(DatabaseCriteria $criteria)
    {
        return $this->getFullOrders($criteria);
    }

    private function getFullOrders(DatabaseCriteria $criteria)
    {
        $ordersData = $this->fetchOrdersData($criteria);

        if (empty($ordersData)) {
            return [];
        }

        $this->fetchOrderStatus($ordersData);
        $this->fetchOrderAttribute($ordersData);
        $this->enrichOrdersWithCustomerInfo($ordersData);
        $this->enrichOrdersWithProductsData($ordersData);

        return $ordersData;
    }

    private function fetchOrdersData(DatabaseCriteria $criteria)
    {
        $query = new \DbQuery();

        /*
         * To include the partial refund:
         *
         * IFNULL(SUM(osl.amount + osl.shipping_cost_amount), 0) +
         * (SELECT IFNULL(SUM(od.product_quantity_return * od.product_price), 0)
         * FROM ' . _DB_PREFIX_ . 'order_detail od
         * WHERE od.id_order = o.id_order) AS returnsValue
         */
        $query->select('
                o.id_order AS id,
                o.date_add AS date,
                o.current_state AS orderState,
                o.total_paid AS totalValue,
                c.email AS customerEmail,
                c.id_customer AS customerCrmId,
                cur.iso_code AS currency,
                (SELECT IFNULL(SUM(od.product_quantity_return * od.product_price), 0)
                 FROM ' . _DB_PREFIX_ . 'order_detail od
                 WHERE od.id_order = o.id_order) AS returnsValue');

        if ($this->includePhone) {
            $query->select('
                (SELECT addr.phone FROM ' . _DB_PREFIX_ . 'address addr
                    WHERE addr.id_customer = o.id_customer
                    AND addr.phone != ""
                    ORDER BY addr.date_add DESC
                    LIMIT 1
                ) AS customerPhone');
        }

        $query->from(\ExpertSenderOrder::$definition['table'], 'eso')
            ->innerJoin('orders', 'o', 'eso.id_order = o.id_order')
            ->innerJoin('customer', 'c', 'o.id_customer = c.id_customer')
            ->innerJoin('currency', 'cur', 'o.id_currency = cur.id_currency')
            ->leftJoin('order_slip', 'osl', 'eso.id_order = osl.id_order')
            ->where($criteria->toSqlWhereClause())
            ->groupBy('eso.id_order')
            ->limit($criteria->getLimit() ?? $this->batchSize, $criteria->getStart() ?? 0);

        return \Db::getInstance()->executeS($query);
    }

    private function fetchOrderAttribute(array &$ordersData)
    {
        /** @var \ExpertSenderCustomerMapping */
        $mappings = $this->orderAttributeMappingRepository->findBy(['active' => 1]);

        foreach ($mappings as $mapping) {
            foreach ($ordersData as &$orderData) {
                $result = $this->orderAttributeService->getAttribute((int) $mapping->id_field)->fetch([
                    'object' => $mapping,
                    'id_order' => $orderData['id'],
                ]);

                if ($result instanceof \JsonSerializable) {
                    $orderData['orderAttributes'][] = $result->jsonSerialize();
                }
            }
        }
    }

    private function fetchOrderStatus(array &$ordersData)
    {
        foreach ($ordersData as &$orderData) {
            /** @var \ExpertSenderOrderStateMappingRelation */
            $relation = $this->orderStateMappingRelationRepository->findOneBy(['id_order_state' => (int) $orderData['orderState']]);

            if ($relation === null) {
                goto skipOrderStatusMatch;
            }

            /** @var \ExpertSenderOrderStateMapping */
            $mapping = $this->orderStateMappingRepository->findOneBy(['id_order_state_mapping' => $relation->id_order_state_mapping, 'active' => 1]);

            if ($mapping === null) {
                goto skipOrderStatusMatch;
            }

            $result = $this->orderStateService->getAttribute((int) $mapping->id_field)->fetch([
                'object' => $mapping,
                'id_order' => $orderData['id'],
                'id_order_state' => $orderData['orderState'],
            ]);

            if ($result instanceof \JsonSerializable) {
                $status = $result->jsonSerialize();

                if (isset($status['name'])) {
                    $orderData['status'] = $status['name'];
                }
            }

            skipOrderStatusMatch:
            unset($orderData['orderState']);
        }
    }

    private function enrichOrdersWithCustomerInfo(array &$ordersData)
    {
        foreach ($ordersData as &$order) {
            $order['customer'] = [
                'email' => $order['customerEmail'],
                'phone' => isset($order['customerPhone']) ? $order['customerPhone'] : null,
                'crmId' => $order['customerCrmId'],
            ];

            unset($order['customerEmail'], $order['customerPhone'], $order['customerCrmId']);
        }

        return $ordersData;
    }

    private function enrichOrdersWithProductsData(array &$ordersData)
    {
        $orderIds = array_column($ordersData, 'id');

        if (empty($orderIds)) {
            return [];
        }

        foreach ($ordersData as &$order) {
            $order['products'] = $this->fetchProductsDataForOrders((int) $order['id']);
        }

        return $ordersData;
    }

    private function fetchProductsDataForOrders(int $orderId)
    {
        $query = new \DbQuery();

        /*
         * To include the partial refund:
         *
         * IFNULL(SUM(od.product_quantity_refunded + od.product_quantity_return), 0) AS returned
         */
        $query->select('
                od.id_order AS orderId,
                od.product_id AS id,
                od.product_name AS name,
                od.product_price AS price,
                IFNULL(SUM(od.product_quantity_return), 0) AS returned,
                od.product_attribute_id AS productAttributeId,
                od.product_quantity AS quantity,
                pi.id_image AS imageId,
                (SELECT it2.name FROM ' . _DB_PREFIX_ . 'image_type it2 WHERE it2.products = 1 ORDER BY it2.width DESC,
                it2.height DESC LIMIT 1) AS imageTypeName,
                pl.link_rewrite AS productLinkRewrite,
                cl.name AS categoryName')
            ->from('order_detail', 'od')
            ->innerJoin('product', 'p', 'od.product_id = p.id_product')
            ->leftJoin('product_lang', 'pl', 'p.id_product = pl.id_product AND pl.id_lang = ' . (int) $this->langId)
            ->leftJoin('image', 'pi', 'p.id_product = pi.id_product AND pi.cover=1')
            ->leftJoin('category_lang', 'cl', 'p.id_category_default = cl.id_category AND cl.id_lang = ' . (int) $this->langId)
            ->where('od.id_order = ' . $orderId)
            ->groupBy('od.id_order_detail');

        $productsData = \Db::getInstance()->executeS($query);
        $groupedData = [];

        foreach ($productsData as $product) {
            $product['url'] = $this->link->getProductLink($product['id'], null, null, null, null, null, $product['productAttributeId']);
            if ($product['imageId'] && $product['imageTypeName']) {
                $product['imageUrl'] = $this->link->getImageLink($product['productLinkRewrite'], $product['imageId'], $product['imageTypeName']);
            } else {
                $product['imageUrl'] = '';
            }
            $product['category'] = $product['categoryName'];

            $this->fetchProductAttributesData($product);

            unset($product['categoryName'], $product['imageId'], $product['imageTypeName'], $product['productLinkRewrite'], $product['productAttributeId']);
            $groupedData[] = $product;
        }

        return $groupedData;
    }

    private function fetchProductAttributesData(array &$product)
    {
        $product['productAttributes'] = [];

        foreach ($this->productMappings as $mapping) {
            $result = $this->productAttributeService->getAttribute((int) $mapping->id_field)->fetch([
                'object' => $mapping,
                'id_product' => $product['id'],
                'id_product_attribute' => $product['productAttributeId'],
            ]);

            if ($result instanceof \JsonSerializable) {
                $product['productAttributes'][] = $result->jsonSerialize();
            }
        }
    }

    /**
     * @return int orders in queue
     */
    public function getTotalRemainingOrdersInQueue(): int
    {
        $query = new \DbQuery();

        $query->select('COUNT(*)')
            ->from(\ExpertSenderOrder::$definition['table'], 'eso')
            ->where('eso.synced = 0');

        $result = \Db::getInstance()->getValue($query);

        return (int) $result;
    }

    /**
     * @param DatabaseCriteria $criteria
     *
     * @return int affectedRows
     */
    public function importOrdersByDateRange(DatabaseCriteria $criteria): int
    {
        $query = new \DbQuery();
        $query->select('o.id_order')
            ->from('orders', 'o')
            ->where($criteria->toSqlWhereClause());

        $results = \Db::getInstance()->executeS($query);

        if (empty($results)) {
            return 0;
        }

        $orderIds = array_column($results, 'id_order');

        return $this->batchInsertOrdersWithDateRange($orderIds);
    }

    private function batchInsertOrdersWithDateRange(array $orderIds): int
    {
        $ordersToInsert = array_map(function ($orderId) {
            return [
                'id_order' => $orderId,
                'synced' => 0,
            ];
        }, $orderIds);

        $existingOrderIds = $this->getExistingOrderIds();
        $ordersToInsert = array_filter($ordersToInsert, function ($order) use ($existingOrderIds) {
            return !in_array($order['id_order'], $existingOrderIds);
        });

        $affectedRows = 0;
        $batches = array_chunk($ordersToInsert, $this->batchSize);
        foreach ($batches as $batch) {
            if (\Db::getInstance()->insert(\ExpertSenderOrder::$definition['table'], $batch)) {
                $affectedRows += \Db::getInstance()->Affected_Rows();
            }
        }

        return $affectedRows;
    }

    private function getExistingOrderIds()
    {
        $query = new \DbQuery();
        $query->select('id_order')
            ->from(\ExpertSenderOrder::$definition['table']);

        $results = \Db::getInstance()->executeS($query);

        return array_column($results, 'id_order');
    }
}
