<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Shared\Mappings;

use JustIdea\ExpertSender\Core\Customer\Model\CustomAttributeValue;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\AttributeFieldLine;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\PrestashopAttributeFields;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\PrestashopAttributeFieldValues;
use JustIdea\ExpertSender\Core\Shared\Mappings\Translator\AttributesTranslatorInterface;
use JustIdea\ExpertSender\Infrastructure\Adapter\Validate;
use JustIdea\ExpertSender\Infrastructure\Enum\SelectValue;
use JustIdea\ExpertSender\Infrastructure\Utility\ArrayUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\FieldsMappingUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

abstract class AbstractAttribute implements AttributeInterface
{
    public const FILE_NAME = 'AbstractAttribute';

    /**
     * @var array
     */
    protected $params = [];

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    protected $module;

    /**
     * @var Context
     */
    protected $context;

    /**
     * @var AttributesTranslatorInterface
     */
    protected $translator;

    /**
     * @param \ExpertSenderOfficial $module
     */
    public function __construct(
        \ExpertSenderOfficial $module,
        AttributesTranslatorInterface $translator
    ) {
        $this->module = $module;
        $this->context = \Context::getContext();
        $this->translator = $translator;
    }

    /**
     * @param array $params
     *
     * @return CustomAttributeValue|OrderAttributeValue|ConsentValue|ProductAttributeValue|null
     */
    final public function fetch(array $params = [])
    {
        try {
            $result = $this->fetchAttribute($params);

            return $result;
        } catch (\Exception $exception) {
            $this->module->getLogger()->error($this->module->l('Attribute fetch failed', self::FILE_NAME), [
                'order_attribute_name' => static::class,
                'params' => $params,
                'exception' => $exception->getMessage(),
            ]);

            return null;
        }
    }

    /**
     * @param array $params
     *
     * @return CustomAttributeValue|OrderAttributeValue|ConsentValue|ProductAttributeValue|null
     */
    public function fetchAttribute(array $params = [])
    {
        $this->params = $params;

        if ($this->validateFetchAttributeRequest()
            && $this->isValidFetchAttributeRequest()) {
            return $this->fetchAttributeRequest();
        }

        return null;
    }

    /**
     * @return bool
     */
    public function validateFetchAttributeRequest(): bool
    {
        return is_array($this->params)
        && isset($this->params['object'])
        && property_exists($this->params['object'], 'id')
        && $this->params['object']->id
        && property_exists($this->params['object'], 'id_field')
        && $this->params['object']->id_field
        && property_exists($this->params['object'], 'id_expertsender_field')
        && $this->params['object']->id_expertsender_field
        && Validate::isLoadedObject($this->params['object']);
    }

    /**
     * @return bool
     */
    abstract protected function isValidFetchAttributeRequest(): bool;

    /**
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    public function fetchAttributeRequest()
    {
        $fieldsMapping = FieldsMappingUtility::deserializeFieldsMapping((string) $this->params['object']->{'fields_mapping'});

        if ((int) $fieldsMapping['id_mapping_field_value'] > 0) {
            return $this->fetchMappingMatchValue((int) $fieldsMapping['id_mapping_field'], (int) $fieldsMapping['id_mapping_field_value']);
        } else {
            return $this->fetchMappingValue((int) $fieldsMapping['id_mapping_field']);
        }
    }

    /**
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    abstract protected function fetchMappingValue(int $fieldId);

    /**
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    abstract protected function fetchMappingMatchValue(int $fieldId, int $fieldValueId);

    final public function fetchDefinition(array $params = []): ?PrestashopAttributeFields
    {
        try {
            $attributeFields = $this->fetchAttributeDefinition($params);

            $result = new PrestashopAttributeFields(
                is_array($attributeFields) ? ArrayUtility::getArray($attributeFields) : []
            );

            return $result;
        } catch (\Exception $exception) {
            $this->module->getLogger()->error($this->module->l('Attribute field name fetch failed', self::FILE_NAME), [
                'attribute_name' => static::class,
                'params' => $params,
                'exception' => $exception->getMessage(),
            ]);

            return null;
        }
    }

    /**
     * @param array $params
     *
     * @return array<AttributeFieldLine>
     */
    public function fetchAttributeDefinition(array $params = [])
    {
        $this->params = $params;

        if ($this->isValidFetchAttributeDefinitionRequest()) {
            $results = [];

            /**
             * @var array<AttributeFieldLine> $definitions
             */
            $definitions = $this->fetchAttributeDefinitionRequest();

            foreach ($definitions as $value) {
                $results[] = AttributeFieldLine::create(
                    (int) $value['id'],
                    (string) ($this->translator->translate($value['id']) ? $this->translator->translate($value['id']) : $value['name']),
                    (bool) (isset($value['disabled']) ? $value['disabled'] : false)
                )->jsonSerialize();
            }

            if (!empty($results)) {
                array_unshift($results, AttributeFieldLine::create(
                    SelectValue::CHOOSE,
                    $this->module->l('--- Choose ---', self::FILE_NAME)
                )->jsonSerialize());
            }

            return $results;
        }

        return null;
    }

    /**
     * @return bool
     */
    abstract protected function isValidFetchAttributeDefinitionRequest(): bool;

    /**
     * @return array<AttributeFieldLine>
     */
    abstract protected function fetchAttributeDefinitionRequest(): array;

    final public function fetchValues(array $params = []): ?PrestashopAttributeFieldValues
    {
        try {
            $attributeFields = $this->fetchAttributeValues($params);

            $result = new PrestashopAttributeFieldValues(
                is_array($attributeFields) ? ArrayUtility::getArray($attributeFields) : []
            );

            return $result;
        } catch (\Exception $exception) {
            $this->module->getLogger()->error($this->module->l('Attribute field name fetch failed', self::FILE_NAME), [
                'attribute_name' => static::class,
                'params' => $params,
                'exception' => $exception->getMessage(),
            ]);

            return null;
        }
    }

    /**
     * @param array $params
     *
     * @return array<AttributeFieldLine>
     */
    public function fetchAttributeValues(array $params = [])
    {
        $this->params = $params;

        if ($this->isValidFetchAttributeValuesRequest()) {
            $results = [];
            $definitions = $this->fetchAttributeValuesRequest();

            foreach ($definitions as $value) {
                $results[] = AttributeFieldLine::create(
                    (int) $value['id'],
                    (string) $this->translator->translate($value['id']) ? $this->translator->translate($value['id']) : $value['name']
                )->jsonSerialize();
            }

            if (!empty($results)) {
                array_unshift($results, AttributeFieldLine::create(
                    SelectValue::CHOOSE,
                    $this->module->l('--- Choose ---', self::FILE_NAME)
                )->jsonSerialize());
            }

            return $results;
        }

        return null;
    }

    /**
     * @return bool
     */
    abstract protected function isValidFetchAttributeValuesRequest(): bool;

    /**
     * @return array
     */
    abstract protected function fetchAttributeValuesRequest(): array;
}
