<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Shared\Mappings;

use JustIdea\ExpertSender\Core\Model\CollectionCriteria;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\PrestashopAttributes;
use JustIdea\ExpertSender\Core\Shared\Mappings\NullableAttribute\NullableAttribute;
use JustIdea\ExpertSender\Core\Shared\Mappings\Translator\AttributesTranslatorInterface;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Utility\ArrayUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

abstract class AbstractAttributeFactory implements AttributeFactoryInterface
{
    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    protected $module;

    /**
     * @var AttributesTranslatorInterface
     */
    protected $translator;

    public function __construct(
        ModuleFactory $moduleFactory,
        AttributesTranslatorInterface $translator
    ) {
        $this->module = $moduleFactory->getModule();
        $this->translator = $translator;
    }

    /**
     * @param int $attributeId
     *
     * @return AttributeInterface
     */
    public function getAttribute(int $attributeId): AttributeInterface
    {
        $create_method_name = $this->getAttributeMethodName($attributeId);

        return method_exists($this, $create_method_name) ?
        $this->{$create_method_name}() :
        $this->getNullableAttribute();
    }

    /**
     * @param string $attribute
     *
     * @return string
     */
    private function getAttributeMethodName(int $attributeId): string
    {
        $definedAttributeLines = $this->getAttributeDefinition()->getAttributesLines();

        if ($definedAttributeLine = $definedAttributeLines->getOneBy((new CollectionCriteria())->where('id', (int) $attributeId))) {
            return 'get' . ucfirst($definedAttributeLine->getMethodName()) . 'Attribute';
        }

        return 'getNullableAttribute';
    }

    /**
     * @return NullableAttribute
     */
    protected function getNullableAttribute(): NullableAttribute
    {
        return new NullableAttribute($this->module, $this->translator);
    }

    /**
     * @return PrestashopAttributes
     */
    public function getAttributeDefinition(): PrestashopAttributes
    {
        /** @var array $data */
        $attributes = $this->getAttributeDefinitionArray();

        $result = new PrestashopAttributes(
            $this->translator,
            is_array($attributes) ? ArrayUtility::getArray($attributes) : []
        );

        return $result;
    }

    abstract protected function getAttributeDefinitionArray(): array;
}
