<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Shared\Mappings\CustomerAttribute\TableCustomerAttribute;

use JustIdea\ExpertSender\Core\Customer\Model\CustomAttributeValue;
use JustIdea\ExpertSender\Core\Enum\CustomAttributeDefinition;
use JustIdea\ExpertSender\Core\Shared\Mappings\AbstractAttributeExtended;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\AttributeFieldValueLine;
use JustIdea\ExpertSender\Infrastructure\Enum\SelectValue;
use JustIdea\ExpertSender\Infrastructure\Utility\TimeZoneUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class TableCustomerAttribute extends AbstractAttributeExtended
{
    public const FILE_NAME = 'TableCustomerAttribute';

    /**
     * @return array
     */
    protected function getFieldDefinitionArray(): array
    {
        return [
            CustomAttributeDefinition::PS_CUSTOMER_TABLE_GENDER => [
                'id' => CustomAttributeDefinition::PS_CUSTOMER_TABLE_GENDER,
                'name' => 'Gender',
                'methodName' => 'customerGender',
            ],
            CustomAttributeDefinition::PS_CUSTOMER_TABLE_WEBSITE => [
                'id' => CustomAttributeDefinition::PS_CUSTOMER_TABLE_WEBSITE,
                'name' => 'Website',
                'methodName' => 'customerWebsite',
            ],
            CustomAttributeDefinition::PS_CUSTOMER_TABLE_BIRTHDAY_DATE => [
                'id' => CustomAttributeDefinition::PS_CUSTOMER_TABLE_BIRTHDAY_DATE,
                'name' => 'Birthday date',
                'methodName' => 'customerBirthdayDate',
            ],
            CustomAttributeDefinition::PS_CUSTOMER_TABLE_REGISTER_DATE => [
                'id' => CustomAttributeDefinition::PS_CUSTOMER_TABLE_REGISTER_DATE,
                'name' => 'Register date',
                'methodName' => 'customerRegisterDate',
            ],
            CustomAttributeDefinition::PS_CUSTOMER_TABLE_LANGUAGE => [
                'id' => CustomAttributeDefinition::PS_CUSTOMER_TABLE_LANGUAGE,
                'name' => 'Language',
                'methodName' => 'customerLanguage',
            ],
        ];
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeRequest(): bool
    {
        return isset($this->params['id_customer'])
        && $this->params['object'] instanceof \ExpertSenderCustomerMapping;
    }

    /**
     * @return CustomAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCustomerGenderMappingValue(): ?CustomAttributeValue
    {
        $langId = (int) \Configuration::get('PS_LANG_DEFAULT');
        $customerId = $this->params['id_customer'];

        $query = new \DbQuery();
        $query->select('gl.name AS value')
            ->from('customer', 'c')
            ->leftJoin('gender_lang', 'gl', 'c.id_gender = gl.id_gender AND gl.id_lang = ' . $langId)
            ->where('c.id_customer = ' . $customerId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return CustomAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) $result['value']
            );
        }

        return null;
    }

    /**
     * @param int $fieldValueId
     *
     * @return CustomAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCustomerGenderMappingMatchValue(int $fieldValueId): ?CustomAttributeValue
    {
        $langId = (int) \Configuration::get('PS_LANG_DEFAULT');
        $customerId = $this->params['id_customer'];

        $query = new \DbQuery();
        $query->select('gl.name AS value')
            ->from('customer', 'c')
            ->leftJoin('gender_lang', 'gl', 'c.id_gender = gl.id_gender AND gl.id_lang = ' . $langId)
            ->where('c.id_customer = ' . $customerId)
            ->where('c.id_gender = ' . $fieldValueId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return CustomAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) CustomAttributeValue::VALUE_TRUE
            );
        }

        return null;
    }

    /**
     * @return CustomAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCustomerWebsiteMappingValue(): ?CustomAttributeValue
    {
        $customerId = $this->params['id_customer'];

        $query = new \DbQuery();
        $query->select('c.website AS value')
            ->from('customer', 'c')
            ->where('c.id_customer = ' . $customerId);

        $result = \Db::getInstance()->getRow($query);

        if (!$result || $result['value'] === null || !$result['value']) {
            return null;
        }

        return CustomAttributeValue::create(
            (string) $this->params['object']->expertsender_field_name,
            (string) $result['value']
        );
    }

    /**
     * @return CustomAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCustomerBirthdayDateMappingValue(): ?CustomAttributeValue
    {
        $customerId = $this->params['id_customer'];

        $query = new \DbQuery();
        $query->select('c.birthday AS value')
            ->from('customer', 'c')
            ->where('c.id_customer = ' . $customerId);

        $result = \Db::getInstance()->getRow($query);

        if (!$result) {
            return null;
        }

        $formattedDate = TimeZoneUtility::convertDateTimeToISO8601Format($result['value']);
        if ($formattedDate === null) {
            return null;
        }

        return CustomAttributeValue::create(
            (string) $this->params['object']->expertsender_field_name,
            (string) $formattedDate
        );
    }

    /**
     * @return CustomAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCustomerRegisterDateMappingValue(): ?CustomAttributeValue
    {
        $customerId = $this->params['id_customer'];

        $query = new \DbQuery();
        $query->select('c.date_add AS value')
            ->from('customer', 'c')
            ->where('c.id_customer = ' . $customerId);

        $result = \Db::getInstance()->getRow($query);

        if (!$result) {
            return null;
        }

        $formattedDate = TimeZoneUtility::convertDateTimeToISO8601Format($result['value']);
        if ($formattedDate === null) {
            return null;
        }

        return CustomAttributeValue::create(
            (string) $this->params['object']->expertsender_field_name,
            (string) $formattedDate
        );
    }

    /**
     * @return CustomAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCustomerLanguageMappingValue(): ?CustomAttributeValue
    {
        $customerId = $this->params['id_customer'];

        $query = new \DbQuery();

        $query->select('l.iso_code AS value')
            ->from('customer', 'c')
            ->leftJoin('lang', 'l', 'c.id_lang = l.id_lang')
            ->where('c.id_customer = ' . $customerId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return CustomAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) \Tools::strtoupper($result['value'])
            );
        }

        return null;
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeDefinitionRequest(): bool
    {
        return is_array($this->params)
        && empty($this->params);
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeValuesRequest(): bool
    {
        return is_array($this->params)
        && isset($this->params['fieldId']);
    }

    /**
     * @return int
     */
    protected function getFieldDefinitionLineId(): int
    {
        return (int) $this->params['fieldId'];
    }

    /**
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function getCustomerGenderAttributeFieldValue(): array
    {
        $results = [];
        $langId = (int) \Configuration::get('PS_LANG_DEFAULT');

        $query = new \DbQuery();
        $query->select('
            g.id_gender AS id,
            gl.name'
        )
            ->from('gender', 'g')
            ->leftJoin('gender_lang', 'gl', 'g.id_gender = gl.id_gender AND gl.id_lang = ' . $langId);

        $values = \Db::getInstance()->executeS($query);

        foreach ($values as $value) {
            $results[] = AttributeFieldValueLine::create(
                (int) $value['id'],
                (string) $value['name']
            )->jsonSerialize();
        }

        if (!empty($results)) {
            array_unshift($results,
                AttributeFieldValueLine::create(
                    SelectValue::ANY,
                    $this->module->l('Pass the value', self::FILE_NAME)
                )->jsonSerialize()
            );
        }

        return $results;
    }
}
