<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Shared\Mappings\OrderAttribute\TableOrderAttribute;

use JustIdea\ExpertSender\Core\Customer\Model\CustomAttributeValue;
use JustIdea\ExpertSender\Core\Order\Model\OrderAttributeValue;
use JustIdea\ExpertSender\Core\Shared\Mappings\AbstractAttributeExtended;
use JustIdea\ExpertSender\Infrastructure\Enum\OrderAttributes;

if (!defined('_PS_VERSION_')) {
    exit;
}

class TableOrderAttribute extends AbstractAttributeExtended
{
    public const FILE_NAME = 'TableOrderAttribute';

    /**
     * @return array
     */
    protected function getFieldDefinitionArray(): array
    {
        return [
            OrderAttributes::PS_ORDER_TABLE_REFERENCE => [
                'id' => OrderAttributes::PS_ORDER_TABLE_REFERENCE,
                'name' => 'Reference',
                'methodName' => 'reference',
            ],
            OrderAttributes::PS_ORDER_TABLE_CARRIER_NAME => [
                'id' => OrderAttributes::PS_ORDER_TABLE_CARRIER_NAME,
                'name' => 'Carrier name',
                'methodName' => 'carrierName',
            ],
            OrderAttributes::PS_ORDER_TABLE_STATE => [
                'id' => OrderAttributes::PS_ORDER_TABLE_STATE,
                'name' => 'Current state',
                'methodName' => 'currentState',
            ],
            OrderAttributes::PS_ORDER_TABLE_PAYMENT_NAME => [
                'id' => OrderAttributes::PS_ORDER_TABLE_PAYMENT_NAME,
                'name' => 'Payment Name',
                'methodName' => 'paymentName',
            ],
            OrderAttributes::PS_ORDER_TABLE_LANGUAGE => [
                'id' => OrderAttributes::PS_ORDER_TABLE_LANGUAGE,
                'name' => 'Language',
                'methodName' => 'language',
            ],
            OrderAttributes::PS_ORDER_TABLE_VALID => [
                'id' => OrderAttributes::PS_ORDER_TABLE_VALID,
                'name' => 'Is order valid',
                'methodName' => 'validOrder',
            ],
        ];
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeRequest(): bool
    {
        return isset($this->params['id_order'])
        && $this->params['object'] instanceof \ExpertSenderOrderMapping;
    }

    /**
     * @return OrderAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchReferenceMappingValue(): ?OrderAttributeValue
    {
        $orderId = $this->params['id_order'];

        $query = new \DbQuery();
        $query->select('o.reference AS value')
            ->from('orders', 'o')
            ->where('o.id_order = ' . $orderId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return OrderAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) $result['value']
            );
        }

        return null;
    }

    /**
     * @return OrderAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCarrierNameMappingValue(): ?OrderAttributeValue
    {
        $orderId = $this->params['id_order'];

        $query = new \DbQuery();
        $query->select('c.name AS value')
            ->from('orders', 'o')
            ->leftJoin('carrier', 'c', 'o.id_carrier = c.id_carrier')
            ->where('o.id_order = ' . $orderId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return OrderAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) ($result['value'] ? $result['value'] : str_replace(['#', ';'], '', \Configuration::get('PS_SHOP_NAME')))
            );
        }

        return null;
    }

    /**
     * @return OrderAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchPaymentNameMappingValue(): ?OrderAttributeValue
    {
        $orderId = $this->params['id_order'];

        $query = new \DbQuery();
        $query->select('o.payment AS value')
            ->from('orders', 'o')
            ->where('o.id_order = ' . $orderId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return OrderAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) $result['value']
            );
        }

        return null;
    }

    /**
     * @return OrderAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchCurrentStateMappingValue(): ?OrderAttributeValue
    {
        $orderId = $this->params['id_order'];
        $langId = (int) \Configuration::get('PS_LANG_DEFAULT');

        $query = new \DbQuery();
        $query->select('osl.name AS value')
            ->from('orders', 'o')
            ->leftJoin('order_state_lang', 'osl', 'o.current_state = osl.id_order_state AND osl.id_lang = ' . $langId)
            ->where('o.id_order = ' . $orderId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return OrderAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) $result['value']
            );
        }

        return null;
    }

    /**
     * @return OrderAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchLanguageMappingValue(): ?OrderAttributeValue
    {
        $orderId = $this->params['id_order'];

        $query = new \DbQuery();
        $query->select('l.iso_code AS value')
            ->from('orders', 'o')
            ->leftJoin('lang', 'l', 'o.id_lang = l.id_lang')
            ->where('o.id_order = ' . $orderId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return OrderAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) \Tools::strtoupper($result['value'])
            );
        }

        return null;
    }

    /**
     * @return OrderAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchValidOrderMappingValue(): ?OrderAttributeValue
    {
        $orderId = $this->params['id_order'];

        $query = new \DbQuery();
        $query->select('o.valid AS value')
            ->from('orders', 'o')
            ->where('o.id_order = ' . $orderId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return OrderAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) ((int) $result['value'] ? CustomAttributeValue::VALUE_TRUE : CustomAttributeValue::VALUE_FALSE)
            );
        }

        return null;
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeDefinitionRequest(): bool
    {
        return is_array($this->params)
        && empty($this->params);
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeValuesRequest(): bool
    {
        return is_array($this->params)
        && isset($this->params['fieldId']);
    }

    /**
     * @return int
     */
    protected function getFieldDefinitionLineId(): int
    {
        return (int) $this->params['fieldId'];
    }
}
