<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Core\Shared\Mappings\ProductAttribute\FeatureProductAttribute;

use JustIdea\ExpertSender\Core\Order\Model\ProductAttributeValue;
use JustIdea\ExpertSender\Core\Shared\Mappings\AbstractAttribute;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\AttributeFieldLine;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\AttributeFieldValueLine;
use JustIdea\ExpertSender\Infrastructure\Enum\SelectValue;

if (!defined('_PS_VERSION_')) {
    exit;
}

class FeatureProductAttribute extends AbstractAttribute
{
    public const FILE_NAME = 'FeatureProductAttribute';

    /**
     * @return bool
     */
    protected function isValidFetchAttributeRequest(): bool
    {
        return isset($this->params['id_product'])
        && $this->params['object'] instanceof \ExpertSenderProductMapping;
    }

    /**
     * @param int $fieldId
     *
     * @return ProductAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    public function fetchMappingValue(int $fieldId): ?ProductAttributeValue
    {
        $productId = (int) $this->params['id_product'];
        $langId = (int) \Configuration::get('PS_LANG_DEFAULT');

        $query = new \DbQuery();
        $query->select('
                fvl.value AS value'
        )
            ->from('feature_product', 'fp')
            ->innerJoin('feature_value', 'fv', 'fp.id_feature = fv.id_feature AND fp.id_feature_value = fv.id_feature_value')
            ->innerJoin('feature_value_lang', 'fvl', 'fp.id_feature_value = fvl.id_feature_value AND fvl.id_lang = ' . $langId)
            ->where('fp.id_product = ' . $productId)
            ->where('fp.id_feature = ' . $fieldId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return ProductAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) $result['value']
            );
        }

        return null;
    }

    /**
     * @param int $fieldId
     * @param int $fieldValueId
     *
     * @return ProductAttributeValue|null
     *
     * @throws PrestaShopException
     * @throws PrestaShopDatabaseException
     * @throws Exception
     */
    protected function fetchMappingMatchValue(int $fieldId, int $fieldValueId): ?ProductAttributeValue
    {
        $productId = (int) $this->params['id_product'];
        $langId = (int) \Configuration::get('PS_LANG_DEFAULT');

        $query = new \DbQuery();
        $query->select('
                fvl.value AS value'
        )
            ->from('feature_product', 'fp')
            ->innerJoin('feature_value', 'fv', 'fp.id_feature = fv.id_feature AND fp.id_feature_value = fv.id_feature_value')
            ->innerJoin('feature_value_lang', 'fvl', 'fp.id_feature_value = fvl.id_feature_value AND fvl.id_lang = ' . $langId)
            ->where('fp.id_product = ' . $productId)
            ->where('fp.id_feature = ' . $fieldId)
            ->where('fp.id_feature_value = ' . $fieldValueId);

        $result = \Db::getInstance()->getRow($query);

        if ($result) {
            return ProductAttributeValue::create(
                (string) $this->params['object']->expertsender_field_name,
                (string) ProductAttributeValue::VALUE_TRUE
            );
        }

        return null;
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeDefinitionRequest(): bool
    {
        return is_array($this->params)
        && empty($this->params);
    }

    /**
     * @return array<AttributeFieldLine>
     */
    protected function fetchAttributeDefinitionRequest(): array
    {
        $results = [];

        $query = new \DbQuery();
        $query->select('
                f.id_feature AS id,
                fl.name'
        )
            ->from('feature', 'f')
            ->leftJoin('feature_lang', 'fl', 'f.id_feature = fl.id_feature AND fl.id_lang = ' . (int) \Configuration::get('PS_LANG_DEFAULT'));

        $definitions = \Db::getInstance()->executeS($query);

        foreach ($definitions as $value) {
            $results[] = AttributeFieldLine::create(
                (int) $value['id'],
                (string) $value['name']
            )->jsonSerialize();
        }

        return $results;
    }

    /**
     * @return bool
     */
    protected function isValidFetchAttributeValuesRequest(): bool
    {
        return is_array($this->params)
        && isset($this->params['fieldId']);
    }

    /**
     * @return array
     */
    public function fetchAttributeValuesRequest(): array
    {
        $fieldId = $this->params['fieldId'];
        $results = [];

        $query = new \DbQuery();
        $query->select('
                fv.id_feature_value AS id,
                fvl.value AS name'
        )
            ->from('feature_value', 'fv')
            ->innerJoin('feature_value_lang', 'fvl', 'fv.id_feature_value = fvl.id_feature_value')
            ->where('fv.id_feature = ' . (int) $fieldId)
            ->where('fvl.id_lang = ' . (int) \Configuration::get('PS_LANG_DEFAULT'));

        $values = \Db::getInstance()->executeS($query);

        foreach ($values as $value) {
            $results[] = AttributeFieldValueLine::create(
                (int) $value['id'],
                (string) $value['name']
            )->jsonSerialize();
        }

        if (!empty($results)) {
            array_unshift($results,
                AttributeFieldValueLine::create(
                    SelectValue::ANY,
                    $this->module->l('Pass the value', self::FILE_NAME)
                )->jsonSerialize()
            );
        }

        return $results;
    }
}
