<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Cache;

use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use Symfony\Component\Cache\Adapter\FilesystemAdapter;

if (!defined('_PS_VERSION_')) {
    exit;
}

class FilesystemCache implements CacheInterface
{
    private $cache;

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    public function __construct(ModuleFactory $module)
    {
        $this->module = $module->getModule();
        $this->cache = new FilesystemAdapter('', 0, $this->getCacheDirectory());
    }

    /** {@inheritDoc} */
    public function get($key, $default = null)
    {
        return $this->cache->getItem($key)->get();
    }

    /** {@inheritDoc} */
    public function has($key)
    {
        return $this->cache->hasItem($key);
    }

    /** {@inheritDoc} */
    public function set($key, $value, $ttl = 3600)
    {
        $item = $this->cache->getItem($key)
            ->set($value)
            ->expiresAfter($ttl);

        $this->cache->save($item);

        return $item;
    }

    /** {@inheritDoc} */
    public function delete($key)
    {
        return $this->cache->deleteItem($key);
    }

    /** {@inheritDoc} */
    public function clear()
    {
        return $this->cache->clear();
    }

    /** {@inheritDoc} */
    public function getMultiple($keys, $default = null)
    {
        $result = [];

        foreach ($keys as $key) {
            $result[$key] = $this->get($key);
        }

        return $result;
    }

    /** {@inheritDoc} */
    public function setMultiple($values, $ttl = null)
    {
        foreach ($values as $key => $value) {
            $this->set($key, $value, $ttl);
        }

        return true;
    }

    /** {@inheritDoc} */
    public function deleteMultiple($keys)
    {
        foreach ($keys as $key) {
            $this->delete($key);
        }

        return true;
    }

    private function getCacheDirectory()
    {
        $dir = _PS_CACHE_DIR_ . $this->module->name;

        if (!is_dir($dir)) {
            mkdir($dir, 0775, true);
        }

        return $dir;
    }
}
