<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Enum\SelectValue;
use JustIdea\ExpertSender\Infrastructure\Exception\ExpertSenderException;
use JustIdea\ExpertSender\Infrastructure\Logger\Logger;
use JustIdea\ExpertSender\Infrastructure\Response\JsonResponse;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\VersionUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

abstract class AbstractAdminController extends \ModuleAdminController
{
    public const FILE_NAME = 'AbstractAdminController';

    /** @var \ModuleCore&\Module&\ExpertSenderOfficial */
    public $module;

    public $override_folder = '';

    public $show_sql_query = false;

    public $back_url;

    protected $parametres = [];

    public function __construct()
    {
        parent::__construct();
        $this->bootstrap = true;

        $this->override_folder = '_override/';
        $this->tpl_folder = '_override/';
    }

    public function initProcess()
    {
        if (!isset($this->list_id)) {
            $this->list_id = $this->table;
        }

        if ('0' === $this->context->cookie->{'submitFilter' . $this->list_id} || '' === $this->context->cookie->{'submitFilter' . $this->list_id}) {
            $this->context->cookie->{'submitFilter' . $this->list_id} = false;
        }

        if (\Tools::isSubmit($this->controller_name . '_config')) {
            if ($this->saveForm()) {
                $this->confirmations[] = $this->module->l('Successful update.', self::FILE_NAME);
            }
        }

        if (false == empty($this->errors)) {
            $this->errors = array_unique($this->errors);
            foreach ($this->errors as $error) {
                $this->module->getLogger()->error($error);
            }
        }

        return parent::initProcess();
    }

    public function postProcess()
    {
        return parent::postProcess();
    }

    public function initContent()
    {
        // $this->displayMenuTabs();

        parent::initContent();
    }

    public function saveForm()
    {
        $result = true;

        foreach (\Tools::getAllValues() as $fieldName => $fieldValue) {
            if (in_array($fieldName, $this->parametres)) {
                $result &= \Configuration::updateValue(\Tools::strtoupper($fieldName), pSQL($fieldValue));
            }
        }

        return $result;
    }

    public function clearFieldsForm()
    {
        $this->fields_form = [];
        $this->tpl_form_vars = [];
    }

    public function clearFieldsList()
    {
        $this->actions = $this->tpl_form_vars = $this->toolbar_btn = [];
        $this->filter = $this->list_no_link = $this->multiple_fieldsets = $this->show_sql_query = false;

        $this->fields_list = $this->className = $this->bulk_actions = $this->display = $this->_select = $this->_join = $this->_where = $this->_orderBy = $this->_orderWay = $this->_group = $this->_filterHaving = $this->_filter = $this->_listsql = null;

        $this->toolbar_title = array_unique($this->breadcrumbs);
        $this->table = 'configuration';
        $this->list_id = 'configuration';
        $this->identifier = 'id_configuration';
        $this->identifier_name = 'name';
        $this->actions_available = ['view', 'edit', 'duplicate', 'delete'];
        $this->_defaultOrderBy = 'id_configuration';
        $this->_defaultOrderWay = 'ASC';
    }

    public function renderForm($fields_form = null)
    {
        if (null === $fields_form) {
            $fields_form = $this->fields_form;
        }

        if (null === $fields_form || !is_array($fields_form)) {
            return;
        }

        $helper = new \HelperForm();
        $this->setHelperDisplay($helper);

        // $helper->token = $this->token;
        $helper->module = $this->module;
        $helper->override_folder = $this->override_folder;
        $helper->default_form_language = (int) \Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) \Configuration::get('PS_LANG_DEFAULT');
        $helper->tpl_vars = [
            'fields_value' => $this->tpl_form_vars,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        $fields_value = $this->getFieldsValue($this->object);

        \Hook::exec('action' . $this->controller_name . 'FormModifier', [
            'object' => &$this->object,
            'fields' => &$this->fields_form,
            'fields_value' => &$fields_value,
            'form_vars' => &$this->tpl_form_vars,
        ]);

        $helper->back_url = (isset($this->back_url)) ? $this->back_url : $this->context->link->getAdminLink($this->controller_name);
        $helper->show_cancel_button = (isset($this->show_form_cancel_button)) ? $this->show_form_cancel_button : ('add' == $this->display || 'edit' == $this->display);
        $helper->fields_value = $fields_value;
        $helper->submit_action = $this->submit_action;
        $back = urldecode(\Tools::getValue('back', ''));
        if (empty($back)) {
            $back = self::$currentIndex . '&token=' . $this->token;
        }
        if (!\Validate::isCleanHtml($back)) {
            exit(\Tools::displayError());
        }

        return $helper->generateForm($fields_form);
    }

    public function renderList($fields_list = null)
    {
        if (null === $fields_list) {
            $fields_list = $this->fields_list;
        }

        if (!($this->fields_list && is_array($this->fields_list))) {
            return false;
        }

        $this->getList($this->context->language->id);

        $helper = new \HelperList();
        $this->setHelperDisplay($helper);
        $helper->_default_pagination = $this->_default_pagination;
        $helper->_pagination = $this->_pagination;
        $helper->tpl_vars = $this->tpl_form_vars;
        $helper->tpl_delete_link_vars = $this->tpl_delete_link_vars;

        // For compatibility reasons, we have to check standard actions in class attributes
        foreach ($this->actions_available as $action) {
            if (!in_array($action, $this->actions) && isset($this->$action) && $this->$action) {
                $this->actions[] = $action;
            }
        }

        $helper->is_cms = $this->is_cms;
        $helper->sql = $this->_listsql;
        $list = $helper->generateList($this->_list, $this->fields_list);

        return $list;
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);

        \Media::addJsDef([
            'expertsender' => [
                'ajaxUrl' => $this->context->link->getAdminLink($this->controller_name),
                'cronUrl' => $this->context->link->getAdminLink(Config::CRON_CONTROLLER) . '&secure_key=' . \Configuration::getGlobalValue(Config::CRON_TOKEN),
                'ajaxToken' => $this->token,
                'messages' => [
                    'MAPPING_NOT_FOUND' => $this->module->getExceptionTranslator()->translate(ExpertSenderException::mappingNotFound()),
                    'SEARCH_NO_RESULTS' => $this->module->l('Oops, nothing found for:', self::FILE_NAME),
                    'DATE_PICKER_CURRENT_TEXT' => $this->module->l('Now', self::FILE_NAME),
                    'DATE_PICKER_CLOSE_TEXT' => $this->module->l('Done', self::FILE_NAME),
                    'DATE_PICKER_TIME_ONLY_TITLE' => $this->module->l('Choose Time', self::FILE_NAME),
                    'DATE_PICKER_TIME_TEXT' => $this->module->l('Time', self::FILE_NAME),
                    'DATE_PICKER_HOUR_TEXT' => $this->module->l('Hour', self::FILE_NAME),
                    'DATE_PICKER_MINUTE_TEXT' => $this->module->l('Minute', self::FILE_NAME),
                    'SELECT_VALUES_GROUP_NAME' => $this->module->l('Exact value', self::FILE_NAME),
                ],
                'enums' => [
                    'SELECT_CHOOSE' => SelectValue::CHOOSE,
                    'SELECT_ANY' => SelectValue::ANY,
                ],
                'selectors' => [
                    'SYNC_FORM_ID' => Config::SYNC_PROCESS_CONFIG_FORM_NAME,
                    'SYNC_LOGGER_TEXTAREA' => Config::LOGS_TEXTAREA,
                    'SYNC_BTN_START_ID' => Config::OPTIONS_START_SYNC_SUBMIT,
                    'SYNC_BTN_STOP_ID' => Config::OPTIONS_STOP_SYNC_SUBMIT,
                    'SYNC_ORDERS_SWITCH_NAME' => Config::SYNC_ORDERS,
                    'SYNC_CUSTOMERS_SWITCH_NAME' => Config::SYNC_CUSTOMERS,
                    'SYNC_RESET_SWITCH_NAME' => Config::SYNC_RESET,
                    'SYNC_LOGS_PROGRESS_BAR_ID' => Config::LOGS_PROGRESS_BAR,
                ],
            ],
        ]);

        $this->addJqueryPlugin(['chosen', 'fancybox']);
        $this->addJqueryUI('ui.datepicker');
        $this->addJS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/js/back.js');

        // $this->addJS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/js/form.js');

        $this->addCSS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/css/back.css');

        if (VersionUtility::isPsVersionLessThan('1.8.0')) {
            $this->addCSS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/css/back17.css');
        }
        $this->context->smarty->registerClass('ExpertSenderConfig', 'JustIdea\ExpertSender\Core\Config\Config');
    }

    public function displaySeverityColor($value, $row)
    {
        switch ($value) {
            case Logger::SEVERITY_INFO:
                return '<span class="badge badge-pill badge-success">' . sprintf($this->module->l('Informative only (%s)', self::FILE_NAME), Logger::SEVERITY_INFO) . '</span>';
            case Logger::SEVERITY_WARNING:
                return '<span class="badge badge-pill badge-warning">' . sprintf($this->module->l('Warning (%s)', self::FILE_NAME), Logger::SEVERITY_WARNING) . '</span>';
            case Logger::SEVERITY_ERROR:
                return '<span class="badge badge-pill badge-danger">' . sprintf($this->module->l('Error (%s)', self::FILE_NAME), Logger::SEVERITY_ERROR) . '</span>';
            case Logger::SEVERITY_CRITICAL:
                return '<span class="badge badge-pill badge-dark">' . sprintf($this->module->l('Major issue (crash)! (%s)', self::FILE_NAME), Logger::SEVERITY_CRITICAL) . '</span>';
            default:
                return $value;
        }
    }

    protected function ajaxResponse($value = null, $controller = null, $method = null): void
    {
        if ($value instanceof JsonResponse) {
            if (JsonResponse::HTTP_INTERNAL_SERVER_ERROR === $value->getStatusCode()) {
                $this->module->getLogger()->error($this->module->l('Failed to return valid response', self::FILE_NAME), [
                    'context' => [
                        'response' => $value->getContent(),
                    ],
                ]);
            }

            http_response_code($value->getStatusCode());

            $value = $value->getContent();
        }

        try {
            if (method_exists(\ControllerCore::class, 'ajaxRender')) {
                $this->ajaxRender($value, $controller, $method);

                exit;
            }

            $this->ajaxDie($value, $controller, $method);
        } catch (\Exception $exception) {
            $this->module->getLogger()->error($this->module->l('Could not return ajax response', self::FILE_NAME), [
                'context' => [
                    'response' => json_encode($value ?: []),
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
        }

        exit;
    }

    public function ensureHasPermissions(array $permissions, bool $ajax = false, bool $silentCheck = false): bool
    {
        foreach ($permissions as $permission) {
            if (!$this->access($permission)) {
                if ($ajax) {
                    $this->ajaxResponse(json_encode([
                        'error' => true,
                        'message' => $this->module->l('Unauthorized.', self::FILE_NAME),
                    ]), JsonResponse::HTTP_UNAUTHORIZED);
                } elseif (!$silentCheck) {
                    $this->errors[] = $this->module->l('You do not have permission to view this.', self::FILE_NAME);
                }

                return false;
            }
        }

        return true;
    }

    /**
     * assign default action in toolbar_btn smarty var, if they are not set.
     * uses override to specifically add, modify or remove items.
     */
    public function initToolbar()
    {
        if ('add' == $this->display || 'edit' == $this->display) {
            $this->toolbar_btn['save_and_stay'] = [
                'href' => self::$currentIndex . '&token=' . $this->token,
                'desc' => $this->module->l('Save and stay', self::FILE_NAME),
                'class' => 'process-icon-save-and-stay ',
            ];
        }

        switch ($this->display) {
            case 'add':
            case 'edit':
                $this->toolbar_btn['save'] = [
                    'href' => '#',
                    'desc' => $this->module->l('Save', self::FILE_NAME),
                ];
                $back = \Tools::safeOutput(\Tools::getValue('back', ''));
                if (empty($back)) {
                    $back = self::$currentIndex . '&token=' . $this->token;
                }
                if (!\Validate::isCleanHtml($back)) {
                    exit(\Tools::displayError());
                }
                if (!$this->lite_display) {
                    $this->toolbar_btn['cancel'] = [
                        'href' => $back,
                        'desc' => $this->module->l('Cancel', self::FILE_NAME),
                    ];
                }

                break;
            case 'view':
                $back = \Tools::safeOutput(\Tools::getValue('back', ''));
                if (empty($back)) {
                    $back = self::$currentIndex . '&token=' . $this->token;
                }
                if (!\Validate::isCleanHtml($back)) {
                    exit(\Tools::displayError());
                }
                if (!$this->lite_display) {
                    $this->toolbar_btn['back'] = [
                        'href' => $back,
                        'desc' => $this->module->l('Back to list', self::FILE_NAME),
                    ];
                }

                break;
            case 'options':
                $this->toolbar_btn['save'] = [
                    'href' => '#',
                    'desc' => $this->module->l('Save', self::FILE_NAME),
                ];

                break;
            case 'list':
                if (empty($this->toolbar_btn['new'])) {
                    $this->toolbar_btn['new'] = [
                        'href' => self::$currentIndex . '&add' . $this->table . '&token=' . $this->token,
                        'desc' => $this->module->l('Add new', self::FILE_NAME),
                    ];
                }
                if ($this->allow_export) {
                    $this->toolbar_btn['export'] = [
                        'href' => self::$currentIndex . '&export' . $this->table . '&token=' . $this->token,
                        'desc' => $this->module->l('Export', self::FILE_NAME),
                    ];
                }

                break;
            default:
                $this->toolbar_btn['modules-list'] = [
                    'href' => '#',
                    'desc' => $this->module->l('Back', self::FILE_NAME),
                ];
                if ($this->allow_export) {
                    $this->toolbar_btn['export'] = [
                        'href' => self::$currentIndex . '&export' . $this->table . '&token=' . $this->token,
                        'desc' => $this->module->l('Export', self::FILE_NAME),
                    ];
                }
        }
    }

    /**
     * @throws Exception
     * @throws SmartyException
     */
    public function display()
    {
        $cookieConfirmations = $this->context->cookie->__get('es_confirmations');
        if ($cookieConfirmations !== false && !empty(json_decode($cookieConfirmations))) {
            foreach (json_decode($cookieConfirmations) as $confirmation) {
                $this->confirmations[] = $confirmation;
            }
            $this->context->cookie->__unset('es_confirmations');
        }

        $cookieWarnings = $this->context->cookie->__get('es_warnings');
        if ($cookieWarnings !== false && !empty(json_decode($cookieWarnings))) {
            foreach (json_decode($cookieWarnings) as $warning) {
                $this->warnings[] = $warning;
            }
            $this->context->cookie->__unset('es_warnings');
        }

        $cookieErrors = $this->context->cookie->__get('es_errors');
        if ($cookieErrors !== false && !empty(json_decode($cookieErrors))) {
            foreach (json_decode($cookieErrors) as $error) {
                $this->errors[] = $error;
            }
            $this->context->cookie->__unset('es_errors');
        }

        parent::display();
    }

    public function saveErrorToCookie($error)
    {
        $errorsArray = [];

        if (is_array($error)) {
            foreach ($error as $msg) {
                $errorsArray[] = $msg;
            }
        } else {
            $errorsArray[] = $error;
        }

        $storedErrors = $this->context->cookie->__get('es_errors') !== false ? (array) json_decode($this->context->cookie->__get('es_errors')) : [];
        $this->context->cookie->__set('es_errors', \json_encode(array_merge($storedErrors, $errorsArray)));
    }

    public function saveWarningToCookie($warning)
    {
        $warningsArray = [];

        if (is_array($warning)) {
            foreach ($warning as $msg) {
                $warningsArray[] = $msg;
            }
        } else {
            $warningsArray[] = $warning;
        }

        $storedWarnings = $this->context->cookie->__get('es_warnings') !== false ? (array) json_decode($this->context->cookie->__get('es_warnings')) : [];
        $this->context->cookie->__set('es_warnings', \json_encode(array_merge($storedWarnings, $warningsArray)));
    }

    public function saveConfirmationToCookie($confirmation)
    {
        $confirmationsArray = [];

        if (is_array($confirmation)) {
            foreach ($confirmation as $msg) {
                $confirmationsArray[] = $msg;
            }
        } else {
            $confirmationsArray[] = $confirmation;
        }

        $storedConfirmations = $this->context->cookie->__get('es_confirmations') !== false ? (array) json_decode($this->context->cookie->__get('es_confirmations')) : [];
        $this->context->cookie->__set('es_confirmations', \json_encode(array_merge($storedConfirmations, $confirmationsArray)));
    }
}
