<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;

if (!defined('_PS_VERSION_')) {
    exit;
}

class AssociationController extends AbstractAssociationAdminController
{
    public const FILE_NAME = 'AssociationController';

    protected $statuses_array = [];

    /**
     * @var AssociationConsentsController
     */
    private $consentsController;

    /**
     * @var AssociationCustomersController
     */
    private $customersController;

    /**
     * @var AssociationOrdersController
     */
    private $ordersController;

    /**
     * @var AssociationOrderStatesController
     */
    private $orderStatesController;

    /**
     * @var AssociationProductsController
     */
    private $productsController;

    protected $spacer_size = '5';

    public function __construct()
    {
        parent::__construct();

        $orderStatesControllerClass = Config::ASSOCIATION_ORDER_STATES_CONTROLLER . 'Controller';
        $this->orderStatesController = new $orderStatesControllerClass();
        $this->orderStatesController->token = $this->token;

        $ordersControllerClass = Config::ASSOCIATION_ORDERS_CONTROLLER . 'Controller';
        $this->ordersController = new $ordersControllerClass();
        $this->ordersController->token = $this->token;

        $consentsControllerClass = Config::ASSOCIATION_CONSENTS_CONTROLLER . 'Controller';
        $this->consentsController = new $consentsControllerClass();
        $this->consentsController->token = $this->token;

        $customersControllerClass = Config::ASSOCIATION_CUSTOMERS_CONTROLLER . 'Controller';
        $this->customersController = new $customersControllerClass();
        $this->customersController->token = $this->token;

        $productsControllerClass = Config::ASSOCIATION_PRODUCTS_CONTROLLER . 'Controller';
        $this->productsController = new $productsControllerClass();
        $this->productsController->token = $this->token;

        $this->context->controller = $this;

        $this->override_folder = '_override/';
        $this->tpl_folder = '_override/';
    }

    public function initPageHeaderToolbar()
    {
        if (empty($this->display)) {
            $this->page_header_toolbar_btn['expertsender-association-order-states'] = [
                'href' => $this->context->link->getAdminLink(Config::ASSOCIATION_ORDER_STATES_CONTROLLER) . '&add' . $this->orderStatesController->table,
                'desc' => $this->module->l('State association', self::FILE_NAME),
                'icon' => 'process-icon-new',
            ];

            $this->page_header_toolbar_btn['expertsender-association-orders'] = [
                'href' => $this->context->link->getAdminLink(Config::ASSOCIATION_ORDERS_CONTROLLER) . '&add' . $this->ordersController->table,
                'desc' => $this->module->l('Order association', self::FILE_NAME),
                'icon' => 'process-icon-new',
            ];

            $this->page_header_toolbar_btn['expertsender-association-consent'] = [
                'href' => $this->context->link->getAdminLink(Config::ASSOCIATION_CONSENTS_CONTROLLER) . '&add' . $this->consentsController->table,
                'desc' => $this->module->l('Consent association', self::FILE_NAME),
                'icon' => 'process-icon-new',
            ];

            $this->page_header_toolbar_btn['expertsender-association-customer'] = [
                'href' => $this->context->link->getAdminLink(Config::ASSOCIATION_CUSTOMERS_CONTROLLER) . '&add' . $this->customersController->table,
                'desc' => $this->module->l('Customer association', self::FILE_NAME),
                'icon' => 'process-icon-new',
            ];

            $this->page_header_toolbar_btn['expertsender-association-products'] = [
                'href' => $this->context->link->getAdminLink(Config::ASSOCIATION_PRODUCTS_CONTROLLER) . '&add' . $this->productsController->table,
                'desc' => $this->module->l('Product association', self::FILE_NAME),
                'icon' => 'process-icon-new',
            ];
        }

        parent::initPageHeaderToolbar();
    }

    public function initContent()
    {
        parent::initContent();

        if (!$this->ajax) {
            $this->context->smarty->assign('associationLists', $this->renderAssociationLists());

            $this->content = $this->context->smarty->fetch($this->getTemplatePath() . 'tabs/tab-association.tpl');
            $this->context->smarty->assign('content', $this->content);
        }
    }

    public function postProcess()
    {
        foreach ($_GET as $get => $value) {
            // add, update, delete
            if (preg_match('/^((?!id_).*)expertsender_(.*)$/', $get, $match)) {
                if (strpos($match[2], 'consent_mapping') !== false) {
                    $controller = Config::ASSOCIATION_CONSENTS_CONTROLLER;
                    $controllerClass = $this->consentsController;
                    $identifier = $this->consentsController->identifier;
                } elseif (strpos($match[2], 'customer_mapping') !== false) {
                    $controller = Config::ASSOCIATION_CUSTOMERS_CONTROLLER;
                    $controllerClass = $this->customersController;
                    $identifier = $this->customersController->identifier;
                } elseif (strpos($match[2], 'order_mapping') !== false) {
                    $controller = Config::ASSOCIATION_ORDERS_CONTROLLER;
                    $controllerClass = $this->ordersController;
                    $identifier = $this->ordersController->identifier;
                } elseif (strpos($match[2], 'order_state_mapping') !== false) {
                    $controller = Config::ASSOCIATION_ORDER_STATES_CONTROLLER;
                    $controllerClass = $this->orderStatesController;
                    $identifier = $this->orderStatesController->identifier;
                } elseif (strpos($match[2], 'product_mapping') !== false) {
                    $controller = Config::ASSOCIATION_PRODUCTS_CONTROLLER;
                    $controllerClass = $this->productsController;
                    $identifier = $this->productsController->identifier;
                } else {
                    continue;
                }

                $controllerPath = $this->context->link->getAdminLink($controller);

                if ($this->ajax) {
                    return $controllerClass->postProcess();
                } else {
                    $url = $controllerPath . '&' . $get . '&' . $identifier . '=' . \Tools::getValue($identifier);
                    if (false !== strpos($match[1], 'submitBulk')) {
                        unset($_POST['token']);
                        $url .= '&' . http_build_query($_POST);
                    } elseif (false !== strpos($match[0], 'Orderby') || false !== strpos($match[0], 'Orderway')) {
                        unset($_GET['controller']);
                        unset($_GET['controllerUri']);
                        unset($_GET['token']);
                        $url .= '&' . http_build_query($_GET);
                    }

                    \Tools::redirectAdmin($url);
                }
            }

            unset($match);
        }

        return parent::postProcess();
    }

    private function renderAssociationLists()
    {
        $this->context->controller = $this->orderStatesController;
        $this->clearFieldsList();
        $associationLists = $this->orderStatesController->renderList();

        $this->context->controller = $this->ordersController;
        $this->clearFieldsList();
        $associationLists .= $this->ordersController->renderList();

        $this->context->controller = $this->consentsController;
        $this->clearFieldsList();
        $associationLists .= $this->consentsController->renderList();

        $this->context->controller = $this->customersController;
        $this->clearFieldsList();
        $associationLists .= $this->customersController->renderList();

        $this->context->controller = $this->productsController;
        $this->clearFieldsList();
        $associationLists .= $this->productsController->renderList();

        $this->context->controller = $this;

        return $associationLists;
    }
}
