<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Api\Exception\ExpertSenderApiException;
use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Model\CollectionCriteria;
use JustIdea\ExpertSender\Core\Order\Api\Repository\ProductAttributesApiRepositoryInterface;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderProductMappingRepositoryInterface;
use JustIdea\ExpertSender\Core\Shared\Mappings\Model\AttributesLine;
use JustIdea\ExpertSender\Core\Shared\Mappings\ProductAttribute\ProductAttributeFactoryInterface;
use JustIdea\ExpertSender\Infrastructure\Enum\SelectValue;
use JustIdea\ExpertSender\Infrastructure\Exception\ExpertSenderException;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\FieldsMappingUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class AssociationProductsController extends AbstractAssociationAdminController
{
    public const FILE_NAME = 'AssociationProductsController';

    /** @var ProductAttributesApiRepositoryInterface */
    public $mappingApiRepository;

    /** @var ProductAttributeFactoryInterface */
    public $mappingFactory;

    /** @var \ExpertSenderProductMapping */
    public $object;

    private $expertSenderFieldDoesNotExist = false;

    private $prestashopFieldDoesNotExist = false;

    public function __construct()
    {
        $this->table = \ExpertSenderProductMapping::$definition['table'];
        $this->identifier = \ExpertSenderProductMapping::$definition['primary'];
        $this->className = \ExpertSenderProductMapping::class;
        $this->multiple_fieldsets = true;

        parent::__construct();

        $this->override_folder = '_override/es_assoc_products/';
        $this->tpl_folder = '_override/es_assoc_products/';

        $this->tabAccess = \Profile::getProfileAccess($this->context->employee->id_profile, \Tab::getIdFromClassName(Config::ASSOCIATION_PRODUCTS_CONTROLLER));

        $this->mappingApiRepository = $this->module->getService(ProductAttributesApiRepositoryInterface::class);
        $this->mappingFactory = $this->module->getService(ProductAttributeFactoryInterface::class);
    }

    public function init()
    {
        parent::init();

        // @TODO Implement check if field is mapped and exists
        if (false) {
            $this->expertSenderFieldDoesNotExist = true;
        }
    }

    public function postProcess()
    {
        if (\Tools::isSubmit('submitAdd' . $this->table)
            || \Tools::isSubmit('submitAdd' . $this->table . 'AndStay')
        ) {
            $id_association = (int) \Tools::getValue($this->identifier);

            parent::validateRules();

            if (count($this->errors)) {
                return false;
            }

            if (!$id_association) {
                if ($this->access('add')) {
                    $association = new $this->className($id_association);
                    $this->copyFromPost($association, $this->table);
                    if (!$association->add()) {
                        $this->errors[] = $this->module->l('An error occurred while creating an object.', self::FILE_NAME) . ' <b>' . $this->table . ' (' . \Db::getInstance()->getMsgError() . ')</b>';
                    }

                    if (\Tools::isSubmit('submitAdd' . $this->table . 'AndStay')) {
                        \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_PRODUCTS_CONTROLLER) .
                            '&conf=3&update' . $this->table . '&' . $this->identifier . '=' . $association->id);
                    }

                    \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_CONTROLLER) . '&conf=3');
                } else {
                    $this->errors[] = $this->module->l('You do not have permission to add this.', self::FILE_NAME);
                }
            } else {
                try {
                    if ($this->access('edit')) {
                        $association = new $this->className($id_association);
                        $this->copyFromPost($association, $this->table);
                        if (!$association->update()) {
                            $this->errors[] = $this->module->l('An error occurred while updating an object.', self::FILE_NAME) . ' <b>' . $this->table . ' (' . \Db::getInstance()->getMsgError() . ')</b>';
                        }

                        if (\Tools::isSubmit('submitAdd' . $this->table . 'AndStay')) {
                            \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_PRODUCTS_CONTROLLER) .
                                '&conf=4&update' . $this->table . '&' . $this->identifier . '=' . $association->id);
                        }

                        \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_CONTROLLER) . '&conf=4');
                    } else {
                        $this->errors[] = $this->module->l('You do not have permission to edit this.', self::FILE_NAME);
                    }
                } catch (\PrestaShopException $e) {
                    $this->errors[] = $e->getMessage();
                }
            }
        } else {
            return parent::postProcess();
        }
    }

    public function renderList($fields_list = null)
    {
        if (Config::ASSOCIATION_PRODUCTS_CONTROLLER == \Tools::getValue('controller') && empty($this->errors) && empty($this->confirmations)) {
            $conf = \Tools::getValue('conf') ? '&conf=' . \Tools::getValue('conf') : '';
            \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_CONTROLLER) . $conf);
        }

        $this->toolbar_title = $this->module->l('Product fields associations', self::FILE_NAME);
        $this->table = \ExpertSenderProductMapping::$definition['table'];
        $this->list_id = \ExpertSenderProductMapping::$definition['table'];
        $this->identifier = \ExpertSenderProductMapping::$definition['primary'];
        $this->className = \ExpertSenderProductMapping::class;
        $this->_defaultOrderBy = \ExpertSenderProductMapping::$definition['primary'];

        $fields_list = [
            'id_product_mapping' => [
                'title' => $this->module->l('ID', self::FILE_NAME),
                'align' => 'text-center',
                'filter_key' => 'a!id_product_mapping',
            ],
            'mapping_name' => [
                'title' => $this->module->l('Association name', self::FILE_NAME),
                'align' => 'text-left',
                'filter_key' => 'a!mapping_name',
            ],
            'field_name' => [
                'title' => $this->module->l('Prestashop field name', self::FILE_NAME),
                'align' => 'text-left',
                'callback' => 'getFieldById',
                'search' => false,
                'orderby' => false,
            ],
            'id_expertsender_field' => [
                'title' => $this->module->l('ECDP field ID', self::FILE_NAME),
                'align' => 'text-center',
                'filter_key' => 'a!id_expertsender_field',
            ],
            'expertsender_field_name' => [
                'title' => $this->module->l('ECDP field name', self::FILE_NAME),
                'align' => 'text-left',
                'filter_key' => 'a!expertsender_field_name',
            ],
            'active' => [
                'title' => $this->module->l('Active', self::FILE_NAME),
                'type' => 'bool',
                'align' => 'center',
                'orderby' => false,
                'ajax' => true,
                'active' => 'status',
                'badge_success' => true,
                'badge_danger' => true,
            ],
        ];

        $this->fields_list = $fields_list;

        $this->toolbar_btn['new'] = [
            'href' => $this->context->link->getAdminLink(Config::ASSOCIATION_PRODUCTS_CONTROLLER) . '&add' . $this->table,
            'desc' => $this->module->l('Add new', self::FILE_NAME),
        ];

        return parent::renderList();
    }

    public function getFieldById($value, $row)
    {
        /** @var ProductAttributeDataCollection $prestaMappings */
        $prestaMappings = $this->mappingFactory->getAttributeDefinition()->getAttributesLines();

        /** @var ProductAttributeData $attribute */
        $attribute = $prestaMappings->getOneBy((new CollectionCriteria())->where('id', (int) $value));

        if ($attribute) {
            return $attribute->getName();
        }

        return $this->module->l('Undefined', self::FILE_NAME);
    }

    public function renderForm($fields_form = null)
    {
        if (!$this->loadObject(true)) {
            return;
        }

        $mappingId = $this->object->id_expertsender_field;
        $fieldId = (int) \Tools::getValue('id_field', $this->object->{'id_field'});

        $mappingsField = FieldsMappingUtility::deserializeFieldsMapping((string) $this->object->{'fields_mapping'});
        $mappingFieldId = $mappingsField['id_mapping_field'];
        $mappingFieldValueId = $mappingsField['id_mapping_field_value'];

        try {
            /** @var ProductAttributes $apiMappings */
            $apiMappings = $this->mappingApiRepository->getAttributes();

            /** @var PrestashopAttributes $prestaMappings */
            $prestaMappings = $this->mappingFactory->getAttributeDefinition();

            /** @var PrestashopAttributeFields $prestaMappingFields */
            $prestaMappingFields = $this->mappingFactory->getAttribute($fieldId)->fetchDefinition();

            /** @var PrestashopAttributeFieldValues $prestaMappingFieldValues */
            $prestaMappingFieldValues = $this->mappingFactory->getAttribute($fieldId)->fetchValues(['fieldId' => $mappingFieldId]);
        } catch (ExpertSenderApiException $exception) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the product attributes', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
            $this->saveErrorToCookie($this->module->getExceptionTranslator()->translate($exception));
            \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_CONTROLLER));
        } catch (ExpertSenderException $th) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the product attributes', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($th),
                ],
            ]);
            $this->saveErrorToCookie($this->module->getExceptionTranslator()->translate($th));
            \Tools::redirectAdmin($this->context->link->getAdminLink(Config::ASSOCIATION_CONTROLLER));
        }

        if ($mappingId !== null && !$apiMappings->getProductAttributeDataCollection()->exists((new CollectionCriteria())->where('id', (int) $mappingId))) {
            $this->expertSenderFieldDoesNotExist = true;
        }

        if (
            $this->object->id_field !== null
            && (
                (
                    $mappingFieldId !== SelectValue::CHOOSE
                    && !$prestaMappingFields->getAttributeFieldLines()->exists((new CollectionCriteria())->where('id', (int) $mappingFieldId))
                )
                || (
                    $mappingFieldId === SelectValue::CHOOSE
                    && $prestaMappingFields->getAttributeFieldLines()->exists((new CollectionCriteria())->where('id', (int) $mappingFieldId))
                )
            )
        ) {
            $this->prestashopFieldDoesNotExist = true;
        }

        if (
            $this->object->id_field !== null
            && (
                (
                    $mappingFieldValueId !== SelectValue::CHOOSE
                    && !$prestaMappingFieldValues->getAttributeFieldValueLines()->exists((new CollectionCriteria())->where('id', (int) $mappingFieldValueId))
                )
                || (
                    $mappingFieldValueId === SelectValue::CHOOSE
                    && $prestaMappingFieldValues->getAttributeFieldValueLines()->exists((new CollectionCriteria())->where('id', (int) $mappingFieldValueId))
                )
            )
        ) {
            $this->prestashopFieldDoesNotExist = true;
        }

        /** @var array $apiMappingArray */
        $apiMappingArray = array_merge(
            [AttributesLine::create(
                SelectValue::CHOOSE,
                $this->module->l('--- Choose ---', self::FILE_NAME),
                ''
            )->jsonSerialize()],
            $apiMappings->jsonSerialize());

        /** @var array $prestaMappingsArray */
        $prestaMappingsArray = array_merge(
            [AttributesLine::create(
                SelectValue::CHOOSE,
                $this->module->l('--- Choose ---', self::FILE_NAME),
                ''
            )->jsonSerialize()],
            $prestaMappings->jsonSerialize());

        /** @var array $prestaMappingFieldsArray */
        $prestaMappingFieldsArray = $prestaMappingFields->jsonSerialize();

        /** @var array $prestaMappingFieldValuesArray */
        $prestaMappingFieldValuesArray = $prestaMappingFieldValues->jsonSerialize();

        $formFields = [
            [
                'type' => 'switch',
                'name' => 'active',
                'label' => $this->module->l('Active', self::FILE_NAME),
                'desc' => $this->module->l('Check this option if you want to activate this association for API requests', self::FILE_NAME),
                'is_bool' => true,
                'default_value' => 1,
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
            [
                'type' => 'text',
                'name' => 'mapping_name',
                'label' => $this->module->l('Association auxiliary name', self::FILE_NAME),
                'desc' => $this->module->l('Enter an auxiliary name for the association, which will be visible only to the merchant.', self::FILE_NAME),
                'class' => 'fixed-width-xxl',
            ],
            [
                'type' => 'html',
                'name' => 'form_hr_1',
                'html_content' => '<hr>',
            ],
            [
                'type' => 'select',
                'name' => 'id_field',
                'label' => $this->module->l('Prestashop Field', self::FILE_NAME),
                'desc' => $this->module->l('Select one of the fields available in Prestashop.', self::FILE_NAME),
                'class' => 'fixed-width-xxl es-chosen es-internal-mapping',
                'options' => [
                    'query' => $prestaMappingsArray,
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'type' => 'select',
                'name' => 'id_mapping_field',
                'label' => $this->module->l('Choose field name', self::FILE_NAME),
                'desc' => $this->module->l('Select one of the field name available in Prestashop.', self::FILE_NAME),
                'class' => 'fixed-width-xxl es-chosen es-internal-mapping-field',
                'form_group_class' => (empty($prestaMappingFieldsArray) ? 'hidden' : ''),
                'options' => [
                    'query' => $prestaMappingFieldsArray,
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'type' => 'select',
                'name' => 'id_mapping_field_value',
                'label' => $this->module->l('Choose value', self::FILE_NAME),
                'desc' => $this->module->l('Select one of the field value available in Prestashop.', self::FILE_NAME),
                'class' => 'fixed-width-xxl es-chosen es-internal-mapping-field-value',
                'form_group_class' => (empty($prestaMappingFieldValuesArray) ? 'hidden' : ''),
                'options' => [
                    'query' => $prestaMappingFieldValuesArray,
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'type' => 'html',
                'name' => 'form_hr_1',
                'html_content' => '<hr>',
            ],
            [
                'type' => 'hidden',
                'name' => 'expertsender_field_name',
                'form_group_class' => 'es-api-mapping-hidden-name',
            ],
            [
                'type' => 'select',
                'name' => 'select_expertsender_field_name',
                'label' => $this->module->l('ECDP Field', self::FILE_NAME),
                'desc' => $this->module->l('Select one of the custom fields created in the Expert Sender panel.', self::FILE_NAME),
                'class' => 'fixed-width-xxl es-chosen es-api-mapping',
                'options' => [
                    'query' => $apiMappingArray,
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'type' => 'text',
                'name' => 'input_id_expertsender_field',
                'label' => $this->module->l('ECDP ID', self::FILE_NAME),
                'desc' => $this->module->l('Enter an id of the association', self::FILE_NAME),
                'class' => 'fixed-width-xxl es-api-mapping-id',
            ],
            [
                'type' => 'hidden',
                'name' => 'id_expertsender_field',
                'form_group_class' => 'es-api-mapping-hidden-id',
            ],
        ];

        $warning = '';
        $warning .= ($this->prestashopFieldDoesNotExist ? $this->module->l('Mapped Prestashop field no longer exists.', self::FILE_NAME) : '');
        $warning .= ($this->expertSenderFieldDoesNotExist ? $this->module->l('Mapped ECDP consent no longer exists.', self::FILE_NAME) : '');

        $this->fields_form['form_mapping'] = [
            'form' => [
                'id_form' => Config::PRODUCTS_ASSOCIATION_CONFIG_FORM_NAME,
                'legend' => [
                    'title' => $this->module->l('Product field association', self::FILE_NAME),
                    'icon' => 'icon-link',
                ],
                'warning' => $warning,
                'input' => $formFields,
                'submit' => [
                    'title' => $this->module->l('Save', self::FILE_NAME),
                    'name' => 'submitAdd' . $this->table,
                    'class' => 'btn btn-default pull-right',
                    'icon' => 'process-icon-save',
                ],
                'buttons' => [
                    'save-and-stay' => [
                        'title' => $this->module->l('Save and stay', self::FILE_NAME),
                        'name' => 'submitAdd' . $this->table . 'AndStay',
                        'type' => 'submit',
                        'class' => 'btn btn-default pull-right',
                        'icon' => 'process-icon-save',
                    ],
                ],
            ],
        ];

        $fields = $this->object::$definition['fields'];
        foreach ($fields as $fieldName => $null) {
            $values[$fieldName] = \Tools::getValue($fieldName, $this->object->{$fieldName});
        }

        $values['input_id_expertsender_field'] = \Tools::getValue('input_id_expertsender_field', \Tools::getValue('id_expertsender_field', $this->object->{'id_expertsender_field'}));
        $values['select_expertsender_field_name'] = \Tools::getValue('select_expertsender_field_name', \Tools::getValue('id_expertsender_field', $this->object->{'id_expertsender_field'}));
        $values['id_mapping_field'] = \Tools::getValue('id_mapping_field', $mappingFieldId);
        $values['id_mapping_field_value'] = \Tools::getValue('id_mapping_field_value', $mappingFieldValueId);

        $this->tpl_form_vars = array_merge($this->tpl_form_vars, $values);

        return parent::renderForm();
    }

    protected function _childValidation()
    {
        if (!\Tools::getValue('submitAdd' . $this->table) && !\Tools::isSubmit('submitAdd' . $this->table . 'AndStay')) {
            return;
        }

        $expertsenderFieldId = (int) \Tools::getValue('id_expertsender_field');
        $fieldId = (int) \Tools::getValue('id_field');

        if ($expertsenderFieldId === SelectValue::CHOOSE) {
            $this->errors[] = $this->module->l('You have to choose one of the ECDP association.', self::FILE_NAME);
        } else {
            $this->validateExpertSenderMappingField();
        }

        if ($fieldId === SelectValue::CHOOSE) {
            $this->errors[] = $this->module->l('You have to choose one of the Prestashop association.', self::FILE_NAME);
        } else {
            $this->validatePrestashopMappingField();
        }
    }

    private function validateExpertSenderMappingField()
    {
        $objectId = (int) \Tools::getValue('id_product_mapping');
        $expertsenderFieldId = (int) \Tools::getValue('id_expertsender_field');

        try {
            $availableMappings = $this->mappingApiRepository->getAttributes()->jsonSerialize();
        } catch (ExpertSenderException $exception) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the customer consents', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
            $this->errors[] = $this->module->l('Failed to retrive the customer consents');
            $this->errors[] = $exception->getMessage();

            return false;
        }

        $validMapping = false;
        foreach ($availableMappings as $mapping) {
            if ($mapping['id'] === $expertsenderFieldId) {
                $validMapping = true;
                break;
            }
        }

        if (!$validMapping) {
            $this->errors[] = $this->module->l('Incorrect ECDP association ID specified.');

            return false;
        }

        /** @var ExpertSenderProductMappingRepositoryInterface $prestaMappingRepository */
        $prestaMappingRepository = $this->module->getService(ExpertSenderProductMappingRepositoryInterface::class);

        /** @var \ExpertSenderProductMapping $existingMapping */
        $existingMapping = $prestaMappingRepository->findOneBy([
            'id_expertsender_field' => $expertsenderFieldId,
        ]);

        if (
            $existingMapping !== null
            && (int) $existingMapping->id_product_mapping !== (int) $objectId
        ) {
            $this->errors[] = $this->module->l('The assosiation for the selected field already exists.', self::FILE_NAME);

            return false;
        }

        return true;
    }

    private function validatePrestashopMappingField(): bool
    {
        $objectId = (int) \Tools::getValue('id_product_mapping');
        $fieldId = (int) \Tools::getValue('id_field');
        $mappingFieldId = (int) \Tools::getValue('id_mapping_field');
        $mappingFieldValueId = (int) \Tools::getValue('id_mapping_field_value');

        try {
            $prestaMappings = $this->mappingFactory->getAttributeDefinition()->jsonSerialize();
            $prestaMappingFields = $this->mappingFactory->getAttribute($fieldId)->fetchDefinition()->jsonSerialize();
            $prestaMappingFieldValues = $this->mappingFactory->getAttribute($fieldId)->fetchValues(['fieldId' => $mappingFieldId])->jsonSerialize();
        } catch (ExpertSenderException $exception) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the prestashop consents', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
            $this->errors[] = $this->module->l('Failed to retrive the prestashop consents');
            $this->errors[] = $exception->getMessage();

            return false;
        }

        $validMappingId = false;
        foreach ($prestaMappings as $consent) {
            if ($consent['id'] === $fieldId) {
                $validMappingId = true;
                break;
            }
        }

        $validMappingFieldId = empty($prestaMappingFields);
        foreach ($prestaMappingFields as $mappingField) {
            if ($mappingField['id'] === $mappingFieldId && $mappingField['id'] !== SelectValue::CHOOSE) {
                $validMappingFieldId = true;
                break;
            }
        }

        $validMappingFieldValueId = empty($prestaMappingFieldValues);
        foreach ($prestaMappingFieldValues as $mappingFieldValue) {
            if ($mappingFieldValue['id'] === $mappingFieldValueId && $mappingFieldValue['id'] !== SelectValue::CHOOSE) {
                $validMappingFieldValueId = true;
                break;
            }
        }

        if (!$validMappingId || !$validMappingFieldId || !$validMappingFieldValueId) {
            $this->errors[] = $this->module->l('Incorrect Prestashop association specified.');

            return false;
        }

        return true;
    }

    public function ajaxProcessStatusexpertsenderProductMapping()
    {
        if (!$id_object = (int) \Tools::getValue($this->identifier)) {
            exit(json_encode(['success' => false, 'error' => true, 'text' => $this->module->l('Failed to update the status', self::FILE_NAME)]));
        } else {
            $object = new $this->className((int) $id_object);
            if (\Validate::isLoadedObject($object)) {
                $object->active = 1 == $object->active ? 0 : 1;
                if ($object->save()) {
                    exit(json_encode(['success' => true, 'text' => $this->module->l('The status has been updated successfully', self::FILE_NAME)]));
                } else {
                    exit(json_encode(['success' => false, 'error' => true, 'text' => $this->module->l('Failed to update the status', self::FILE_NAME)]));
                }
            }
        }
    }

    public function ajaxProcessGetMappings()
    {
        $mappings = $apiMappings = [];
        $mappingId = (int) \Tools::getValue('id_mapping');
        $isMappingIdPresent = false;

        try {
            $apiMappings = $this->mappingApiRepository->getAttributes()->jsonSerialize();
        } catch (ExpertSenderApiException $exception) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the product attributes', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
        } catch (ExpertSenderException $th) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the product attributes', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($th),
                ],
            ]);
        }

        foreach ($apiMappings as $mapping) {
            if ($mapping['id'] === $mappingId) {
                $isMappingIdPresent = true;
                break;
            }
        }

        if ($isMappingIdPresent) {
            array_unshift($apiMappings, AttributesLine::create(
                SelectValue::CHOOSE,
                $this->module->l('--- Choose ---', self::FILE_NAME),
                ''
            )->jsonSerialize());
            $mappings = $apiMappings;
        }

        exit(json_encode([
            'mappings' => $mappings,
        ]));
    }

    public function ajaxProcessGetMappingFields()
    {
        $fields = [];
        $typeId = (int) \Tools::getValue('id_type');

        try {
            $fields = $this->mappingFactory->getAttribute($typeId)->fetchDefinition()->jsonSerialize();
        } catch (ExpertSenderException $exception) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the prestashop mapping field names', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
            $this->errors[] = $this->module->l('Failed to retrive the prestashop mapping fields');
            $this->errors[] = $exception->getMessage();

            return false;
        }

        exit(json_encode([
            'fields' => $fields,
        ]));
    }

    public function ajaxProcessGetMappingFieldValues()
    {
        $fieldValues = [];
        $typeId = (int) \Tools::getValue('id_type');
        $fieldId = (int) \Tools::getValue('id_field');

        try {
            $fieldValues = $this->mappingFactory->getAttribute($typeId)->fetchValues(['fieldId' => $fieldId])->jsonSerialize();
        } catch (ExpertSenderException $exception) {
            $this->module->getLogger()->error($this->module->l('Failed to retrive the prestashop mapping field names', self::FILE_NAME), [
                'context' => [
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ],
            ]);
            $this->errors[] = $this->module->l('Failed to retrive the prestashop mapping fields');
            $this->errors[] = $exception->getMessage();

            return false;
        }

        exit(json_encode([
            'fieldValues' => $fieldValues,
        ]));
    }
}
