<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Customer\Processor\SyncCustomersProcessor;
use JustIdea\ExpertSender\Core\Enum\ProcessState;
use JustIdea\ExpertSender\Core\Order\Processor\SyncOrdersProcessor;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class CronController extends AbstractAdminController
{
    public const FILE_NAME = 'CronController';

    /**
     * @var string
     */
    public $ajaxMethod;

    public function __construct()
    {
        parent::__construct();

        $this->ajaxMethod = \Tools::getValue('method');

        $this->handleCronRequest();
    }

    private function sendCronCallback($msg)
    {
        ignore_user_abort(true);
        set_time_limit(0);

        ob_start();
        echo json_encode($msg);
        header('Connection: close');
        header('Content-Length: ' . ob_get_length());
        ob_end_flush();
        ob_flush();
        flush();

        if (function_exists('fastcgi_finish_request')) {
            fastcgi_finish_request();
        }
    }

    private function handleCronRequest()
    {
        $secureKey = \Tools::getValue('secure_key');

        if ($secureKey !== \Configuration::getGlobalValue(Config::CRON_TOKEN)) {
            exit($this->module->l('Invalid secure_key.', self::FILE_NAME));
        }

        $this->executeCronTask();
    }

    private function executeCronTask()
    {
        if (method_exists($this, $this->ajaxMethod . 'CronAction')) {
            call_user_func([$this, $this->ajaxMethod . 'CronAction']);
        } else {
            exit($this->module->l('Cron task not found.', self::FILE_NAME));
        }
    }

    private function syncDataThroughApiCronAction()
    {
        if (method_exists($this, $this->ajaxMethod . 'CronCallbackAction')) {
            $this->sendCronCallback(call_user_func([$this, $this->ajaxMethod . 'CronCallbackAction']));
        } else {
            exit($this->module->l('Cron task not found.', self::FILE_NAME));
        }
    }

    private function syncDataThroughApiCronCallbackAction()
    {
        $return = '';

        $this->module->getReportLogger()->appendLogFileWithLineBreak();
        $this->module->getReportLogger()->info($this->module->l('Cron task has been started...', self::FILE_NAME));

        if (Config::OPTION_ON === \Tools::getValue(Config::SYNC_ORDERS)) {
            try {
                /** @var SyncOrdersProcessor $syncOrdersProcessor */
                $syncOrdersProcessor = $this->module->getService(SyncOrdersProcessor::class);

                $result = $syncOrdersProcessor->processAction();

                switch ($result) {
                    case ProcessState::SUCCESS:
                        break;

                    case ProcessState::NO_AFFECT:
                        // $this->module->getReportLogger()->info($this->module->l('No orders to be synced.', self::FILE_NAME));
                        break;

                    default:
                        break;
                }
            } catch (\Throwable $exception) {
                $this->module->getReportLogger()->error($this->module->l('Failed to sync the orders. View error logs for more information.', self::FILE_NAME));
                $this->module->getLogger()->error('Failed to sync the orders.', [
                    'context' => [],
                    'request' => ExceptionUtility::getLastRequest($exception),
                    'response' => ExceptionUtility::getLastResponse($exception),
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ]);
            }
        } else {
            $this->module->getReportLogger()->info($this->module->l('Due to the cron setting skipping order sync process.', self::FILE_NAME));
        }

        if (Config::OPTION_ON === \Tools::getValue(Config::SYNC_CUSTOMERS)) {
            try {
                /** @var SyncCustomersProcessor $syncCustomersProcessor */
                $syncCustomersProcessor = $this->module->getService(SyncCustomersProcessor::class);

                $result = $syncCustomersProcessor->processAction();

                switch ($result) {
                    case ProcessState::SUCCESS:
                        break;

                    case ProcessState::NO_AFFECT:
                        // $this->module->getReportLogger()->info($this->module->l('No customers to be synced.', self::FILE_NAME));
                        break;

                    default:
                        break;
                }
            } catch (\Throwable $exception) {
                $this->module->getReportLogger()->error($this->module->l('Failed to sync the customers. View error logs for more information.', self::FILE_NAME));
                $this->module->getLogger()->error('Failed to sync the orders.', [
                    'context' => [],
                    'request' => ExceptionUtility::getLastRequest($exception),
                    'response' => ExceptionUtility::getLastResponse($exception),
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ]);
            }

            $this->module->getReportLogger()->info($this->module->l('Cron task has been finished!', self::FILE_NAME));
        } else {
            $this->module->getReportLogger()->info($this->module->l('Due to the cron setting skipping customer sync process.', self::FILE_NAME));
        }

        $logContents = $this->module->getReportLogger()->getSessionLogFileContentsAsArray($this->module->l('Cron sync results:', self::FILE_NAME), false);

        foreach ($logContents as $value) {
            $return .= $value . '<br>';
        }

        return $return;
    }
}
