<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Order\Action\ImportOrderAction;
use JustIdea\ExpertSender\Core\Order\DTO\ImportOrderRequestData;
use JustIdea\ExpertSender\Infrastructure\Adapter\Validate;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ImportOrderController extends AbstractAdminController
{
    public const FILE_NAME = 'ImportOrderController';

    protected $importParametres = [];

    public function __construct()
    {
        parent::__construct();

        $this->override_folder = '_override/es_import_orders/';
        $this->tpl_folder = '_override/es_import_orders/';

        $this->importParametres = [
            Config::IMPORT_ORDER_DATE_FROM,
            Config::IMPORT_ORDER_DATE_TO,
        ];
    }

    public function initContent()
    {
        parent::initContent();

        $this->clearFieldsForm();
        $this->initImportForm();
        $this->context->smarty->assign('formImportOrders', $this->renderForm());

        $this->content = $this->context->smarty->fetch($this->getTemplatePath() . 'tabs/tab-import-orders.tpl');
        $this->context->smarty->assign('content', $this->content);
    }

    public function initProcess(): void
    {
        if (\Tools::isSubmit(Config::IMPORT_ORDER_FORM_SUBMIT)) {
            if (!$this->validateImportForm()) {
                return;
            }

            if ($this->processImportForm()) {
                $this->saveConfirmationToCookie($this->module->l('Orders exported successfully.', self::FILE_NAME));
            } else {
                $this->saveErrorToCookie($this->module->l('Failed to export the orders. View error logs for more information.', self::FILE_NAME));
            }
            \Tools::redirectAdmin($this->context->link->getAdminLink(Config::SYNC_CONTROLLER));
        }

        parent::initProcess();
    }

    public function processImportForm(): bool
    {
        $dateFrom = \Tools::getValue(Config::IMPORT_ORDER_DATE_FROM);
        $dateTo = \Tools::getValue(Config::IMPORT_ORDER_DATE_TO);

        /** @var ImportOrderAction $importOrderAction */
        $importOrderAction = $this->module->getService(ImportOrderAction::class);

        $importOrderAction->run(ImportOrderRequestData::create(
            $dateFrom,
            $dateTo
        ));

        return true;
    }

    public function validateImportForm(): bool
    {
        $result = true;

        $dateFrom = \Tools::getValue(Config::IMPORT_ORDER_DATE_FROM);
        $dateTo = \Tools::getValue(Config::IMPORT_ORDER_DATE_TO);

        if (Validate::isEmpty($dateFrom)) {
            $this->errors[] = sprintf($this->module->l('Date from can not be empty. It should be: %s.', self::FILE_NAME), $this->module->l('YYYY-MM-DD HH-mm-ss', self::FILE_NAME));
            $result &= false;
        } elseif (!Validate::isDateTimeFormat($dateTo)) {
            $this->errors[] = sprintf($this->module->l('Date to format (%s) is not valid. It should be: %s.', self::FILE_NAME), $dateTo, $this->module->l('YYYY-MM-DD HH-mm-ss', self::FILE_NAME));
            $result &= false;
        }

        if (Validate::isEmpty($dateFrom)) {
            $this->errors[] = sprintf($this->module->l('Date from can not be empty. It should be: %s.', self::FILE_NAME), $this->module->l('YYYY-MM-DD HH-mm-ss', self::FILE_NAME));
            $result &= false;
        } elseif (!Validate::isDateTimeFormat($dateFrom)) {
            $this->errors[] = sprintf($this->module->l('Date from format (%s) is not valid. It should be: %s.', self::FILE_NAME), $dateFrom, $this->module->l('YYYY-MM-DD HH-mm-ss', self::FILE_NAME));
            $result &= false;
        } elseif (new \DateTime($dateTo) <= new \DateTime($dateFrom)) {
            $this->errors[] = sprintf($this->module->l('Date from (%s) cannot be greater than date to (%s).', self::FILE_NAME), $dateFrom, $dateTo);

            $result &= false;
        }

        return (bool) $result;
    }

    private function initImportForm()
    {
        $formFields = [
            [
                'type' => 'desc_full_width',
                'name' => Config::IMPORT_ORDER_DATE_FROM . Config::CONST_STRING_DESCRPTION,
                'template' => Config::IMPORT_ORDER_DATE_FROM . Config::CONST_STRING_DESCRPTION,
                'class' => '',
            ],
            [
                'type' => 'desc',
                'name' => Config::IMPORT_ORDER_DATE_TO . Config::CONST_STRING_DESCRPTION,
                'template' => Config::IMPORT_ORDER_DATE_TO . Config::CONST_STRING_DESCRPTION,
                'class' => '',
            ],
            [
                'type' => 'datetime',
                'label' => $this->module->l('From:', self::FILE_NAME),
                'name' => Config::IMPORT_ORDER_DATE_FROM,
                'required' => true,
                'class' => 'es-datepicker',
                'desc' => $this->module->l('Export orders placed no earlier than the date selected above.', self::FILE_NAME),
            ],
            [
                'type' => 'datetime',
                'label' => $this->module->l('To:', self::FILE_NAME),
                'name' => Config::IMPORT_ORDER_DATE_TO,
                'required' => true,
                'class' => 'es-datepicker',
                'desc' => $this->module->l('Export orders placed no later than the date selected above.', self::FILE_NAME),
            ],
        ];

        $this->fields_form['form'] = [
            'form' => [
                'id_form' => Config::IMPORT_ORDER_FORM_NAME,
                'legend' => [
                    'title' => $this->module->l('Export Settings', self::FILE_NAME),
                    'icon' => 'icon-download-alt',
                ],
                'input' => $formFields,
                'submit' => [
                    'title' => $this->module->l('Export', self::FILE_NAME),
                    'name' => Config::IMPORT_ORDER_FORM_SUBMIT,
                    'id' => Config::IMPORT_ORDER_FORM_SUBMIT . 'SubmitBtn',
                    'class' => 'btn btn-default pull-right',
                    'icon' => 'process-icon-save',
                ],
            ],
        ];

        $values = [];
        foreach ($this->importParametres as $parameter) {
            $values[$parameter] = \Tools::getValue($parameter);
        }
        $this->tpl_form_vars = array_merge($this->tpl_form_vars, $values);

        $this->show_form_cancel_button = true;
        $this->back_url = $this->context->link->getAdminLink(Config::SYNC_CONTROLLER);
    }
}
