<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Enum\ProcessState;
use JustIdea\ExpertSender\Infrastructure\Enum\PermissionType;
use JustIdea\ExpertSender\Infrastructure\Logger\Export\CsvExportServiceInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\Export\LogZipExportServiceInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\Logger;
use JustIdea\ExpertSender\Infrastructure\Processor\RemoveLogFilesProcessorInterface;
use JustIdea\ExpertSender\Infrastructure\Processor\RemoveLogsProcessorInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LogController extends AbstractAdminController
{
    public const FILE_NAME = 'LogController';

    public function __construct()
    {
        $this->table = \ExpertSenderLog::$definition['table'];
        $this->identifier = \ExpertSenderLog::$definition['primary'];
        $this->className = \ExpertSenderLog::class;
        parent::__construct();

        $this->parametres = [
            Config::LOGS_DEBUG_MODE,
            Config::LOGS_REPORT_STORE_ENABLED,
        ];
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);

        \Media::addJsDef([
            'expertsenderlogger' => [
                'logsUrl' => $this->context->link->getAdminLink(Config::LOGGER_AJAX_CONTROLLER),
            ],
        ]);

        $this->addJS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/js/logs-ajax.js');
        $this->addCSS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/css/logs.css');
    }

    public function initContent()
    {
        parent::initContent();

        $this->clearFieldsForm();
        $this->renderLogForm();
        $this->context->smarty->assign('logsForm', $this->renderForm());

        $this->clearFieldsList();
        $this->renderLogList();
        $this->context->smarty->assign('logsList', $this->renderList());

        $this->content = $this->context->smarty->fetch($this->getTemplatePath() . 'tabs/tab-logs.tpl');
        $this->context->smarty->assign('content', $this->content);
    }

    public function initProcess(): void
    {
        if (\Tools::isSubmit(Config::LOG_PANEL_EXPORT_LOGS_TO_CSV_SUBMIT_NAME)) {
            if (!$this->ensureHasPermissions([PermissionType::EDIT, PermissionType::VIEW], true)) {
                $this->errors[] = $this->module->l('You do not have permission to do this.', self::FILE_NAME);
            } else {
                $result = $this->processExportAllColumns();

                if (!$result) {
                    $this->warnings[] = $this->module->l('No logs to export.', self::FILE_NAME);
                }
            }
        }

        if (\Tools::isSubmit(Config::LOG_PANEL_EXPORT_LOGS_TO_ZIP_FILE_SUBMIT_NAME)) {
            if (!$this->ensureHasPermissions([PermissionType::EDIT, PermissionType::VIEW], true)) {
                $this->errors[] = $this->module->l('You do not have permission to do this.', self::FILE_NAME);
            } else {
                $result = $this->processExportAllLogsFiles();

                if (!$result) {
                    $this->warnings[] = $this->module->l('No logs to export.', self::FILE_NAME);
                }
            }
        }

        if (\Tools::isSubmit(Config::LOG_PANEL_DELETE_LOGS_SUBMIT_NAME)) {
            if (!$this->ensureHasPermissions([PermissionType::DELETE, PermissionType::EDIT, PermissionType::VIEW], true)) {
                $this->errors[] = $this->module->l('You do not have permission to do this.', self::FILE_NAME);
            } else {
                try {
                    /** @var RemoveLogsProcessorInterface $removeLogsProcessor */
                    $removeLogsProcessor = $this->module->getService(RemoveLogsProcessorInterface::class);

                    $result = $removeLogsProcessor->processAction();

                    switch ($result) {
                        case ProcessState::SUCCESS:
                            $this->saveConfirmationToCookie($this->module->l('Logs deleted successfully.', self::FILE_NAME));
                            break;

                        case ProcessState::NO_AFFECT:
                            $this->saveWarningToCookie($this->module->l('No logs to be deleted.', self::FILE_NAME));
                            break;

                        default:
                            break;
                    }
                } catch (\Throwable $exception) {
                    $this->saveErrorToCookie($this->module->l('Failed to delete the logs. View error logs for more information.', self::FILE_NAME));

                    $this->module->getLogger()->error('Failed to delete the logs.', [
                        'context' => [],
                        'exceptions' => ExceptionUtility::getExceptions($exception),
                    ]);
                }

                \Tools::redirectAdmin($this->context->link->getAdminLink(Config::LOG_CONTROLLER));
            }
        }

        if (\Tools::isSubmit(Config::LOG_PANEL_DELETE_LOGS_FILES_SUBMIT_NAME)) {
            if (!$this->ensureHasPermissions([PermissionType::DELETE, PermissionType::EDIT, PermissionType::VIEW], true)) {
                $this->errors[] = $this->module->l('You do not have permission to do this.', self::FILE_NAME);
            } else {
                try {
                    /** @var RemoveLogFilesProcessorInterface $removeLogFilesProcessor */
                    $removeLogFilesProcessor = $this->module->getService(RemoveLogFilesProcessorInterface::class);

                    $result = $removeLogFilesProcessor->processAction();

                    switch ($result) {
                        case ProcessState::SUCCESS:
                            $this->saveConfirmationToCookie($this->module->l('Log files deleted successfully.', self::FILE_NAME));
                            break;

                        case ProcessState::NO_AFFECT:
                            $this->saveWarningToCookie($this->module->l('No log files to be deleted.', self::FILE_NAME));
                            break;

                        default:
                            break;
                    }
                } catch (\Throwable $exception) {
                    $this->saveErrorToCookie($this->module->l('Failed to delete the log files. View error logs for more information.', self::FILE_NAME));

                    $this->module->getLogger()->error('Failed to delete the log files.', [
                        'context' => [],
                        'exceptions' => ExceptionUtility::getExceptions($exception),
                    ]);
                }

                \Tools::redirectAdmin($this->context->link->getAdminLink(Config::LOG_CONTROLLER));
            }
        }

        if (\Tools::isSubmit(Config::LOG_PANEL_FORM_SUBMIT_NAME)) {
            if ($this->saveForm()) {
                $this->confirmations[] = $this->module->l('Successful update.', self::FILE_NAME);
            }
        }

        parent::initProcess();
    }

    public function initPageHeaderToolbar()
    {
        if ($this->ensureHasPermissions([PermissionType::EDIT, PermissionType::VIEW], true)) {
            if (empty($this->display)) {
                $this->page_header_toolbar_btn['expertsender-logs-export'] = [
                    'href' => self::$currentIndex . '&token=' . $this->token . '&' . Config::LOG_PANEL_EXPORT_LOGS_TO_CSV_SUBMIT_NAME,
                    'desc' => $this->module->l('Export logs', self::FILE_NAME),
                    'icon' => 'process-icon-download',
                ];

                $this->page_header_toolbar_btn['expertsender-file-logs-export'] = [
                    'href' => self::$currentIndex . '&token=' . $this->token . '&' . Config::LOG_PANEL_EXPORT_LOGS_TO_ZIP_FILE_SUBMIT_NAME,
                    'desc' => $this->module->l('Export history logs', self::FILE_NAME),
                    'icon' => 'process-icon-download',
                ];

                $this->page_header_toolbar_btn['delete-db-logs'] = [
                    'href' => '#',
                    'desc' => $this->module->l('Delete logs', self::FILE_NAME),
                    'icon' => 'process-icon-delete',
                    'js' => "redirectAfterConfirmation('" . self::$currentIndex . '&token=' . $this->token . '&' . Config::LOG_PANEL_DELETE_LOGS_SUBMIT_NAME . "', '" . $this->module->l('Are you sure you want to delete all logs stored in the database?', self::FILE_NAME) . "')",
                ];

                // Dodaj przycisk usuwania logów z folderu
                $this->page_header_toolbar_btn['delete-file-logs'] = [
                    'href' => '#',
                    'desc' => $this->module->l('Delete history logs', self::FILE_NAME),
                    'icon' => 'process-icon-delete',
                    'js' => "redirectAfterConfirmation('" . self::$currentIndex . '&token=' . $this->token . '&' . Config::LOG_PANEL_DELETE_LOGS_FILES_SUBMIT_NAME . "', '" . $this->module->l('Are you sure you want to delete all history logs stored in the files?', self::FILE_NAME) . "')",
                ];
            }
        }

        parent::initPageHeaderToolbar();
    }

    protected function renderLogForm()
    {
        $formFields = [
            [
                'type' => 'switch',
                'label' => $this->module->l('Store history logs', self::FILE_NAME),
                'name' => Config::LOGS_REPORT_STORE_ENABLED,
                'is_bool' => true,
                'desc' => $this->module->l('Check this option to enable log storage after synchronisation in log files. This option allows you to check the progress of the synchronisation process (manual or cron). Keep in mind that errors are always logged and available below.', self::FILE_NAME),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
            [
                'type' => 'switch',
                'label' => $this->module->l('Debug mode', self::FILE_NAME),
                'name' => Config::LOGS_DEBUG_MODE,
                'is_bool' => true,
                'desc' => $this->module->l('Check this option to enable debug mode for the logger in order to log more events.', self::FILE_NAME),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
        ];

        $this->fields_form['form'] = [
            'form' => [
                'id_form' => Config::LOG_CONFIG_FORM_NAME,
                'legend' => [
                    'title' => $this->module->l('Logger settings', self::FILE_NAME),
                    'icon' => 'icon-cogs',
                ],
                'input' => $formFields,
                'submit' => [
                    'title' => $this->module->l('Save', self::FILE_NAME),
                    'name' => Config::LOG_PANEL_FORM_SUBMIT_NAME,
                    'id' => Config::LOG_PANEL_FORM_SUBMIT_NAME . 'SubmitBtn',
                    'class' => 'btn btn-default pull-right',
                    'icon' => 'process-icon-save',
                ],
            ],
        ];

        $values = [];

        foreach ($this->parametres as $parameter) {
            $values[$parameter] = \Configuration::get(\Tools::strtoupper($parameter));
        }

        $this->tpl_form_vars = array_merge($this->tpl_form_vars, $values);
    }

    public function renderLogList()
    {
        $this->toolbar_title = $this->module->l('Expert Sender Logs', self::FILE_NAME);
        $this->table = \ExpertSenderLog::$definition['table'];
        $this->list_id = \ExpertSenderLog::$definition['table'];
        $this->identifier = \ExpertSenderLog::$definition['primary'];
        $this->className = \ExpertSenderLog::class;
        $this->multiple_fieldsets = true;
        $this->show_sql_query = true;
        $this->actions_available = [''];
        $this->list_no_link = true;
        $this->_use_found_rows = true;
        $this->display = 'list';

        $this->_defaultOrderBy = 'id_log';
        $this->_defaultOrderWay = 'DESC';

        $this->_select = 'pl.date_add, pl.message, pl.severity, a.id_log as details';
        $this->_join = 'LEFT JOIN `' . _DB_PREFIX_ . 'log` pl ON (pl.id_log = a.id_log)';
        $this->_where = 'AND pl.object_type = "expertsenderlog"';

        unset($this->toolbar_btn['new']);

        $this->toolbar_btn[Config::LOG_PANEL_EXPORT_LOGS_TO_CSV_SUBMIT_NAME] = [
            'href' => self::$currentIndex . '&token=' . $this->token . '&' . Config::LOG_PANEL_EXPORT_LOGS_TO_CSV_SUBMIT_NAME,
            'desc' => $this->module->l('Export to CSV', self::FILE_NAME),
            'icon' => 'process-icon-export',
        ];

        $fields_list = [
            'id_log' => [
                'title' => $this->module->l('ID', self::FILE_NAME),
                'align' => 'text-center',
                'filter_key' => 'a!id_log',
            ],
            'severity' => [
                'title' => $this->module->l('Severity', self::FILE_NAME),
                'align' => 'text-center',
                'callback' => 'displaySeverityColor',
            ],
            'message' => [
                'title' => $this->module->l('Message', self::FILE_NAME),
                'align' => 'left',
            ],
            'request' => [
                'title' => $this->module->l('Request', self::FILE_NAME),
                'align' => 'text-center',
                'callback' => 'displayRequestButton',
                'orderby' => false,
                'search' => false,
                'remove_onclick' => true,
            ],
            'response' => [
                'title' => $this->module->l('Response', self::FILE_NAME),
                'align' => 'text-center',
                'callback' => 'displayResponseButton',
                'orderby' => false,
                'search' => false,
                'remove_onclick' => true,
            ],
            'context' => [
                'title' => $this->module->l('Context', self::FILE_NAME),
                'align' => 'text-center',
                'callback' => 'displayContextButton',
                'orderby' => false,
                'search' => false,
                'remove_onclick' => true,
            ],
            'date_add' => [
                'title' => $this->module->l('Date Add', self::FILE_NAME),
                'align' => 'text-right',
                'type' => 'datetime',
            ],
        ];

        $this->fields_list = $fields_list;

        if (\Tools::isSubmit('submitFilter' . $this->table) || false !== $this->context->cookie->{'submitFilter' . $this->table}) {
            $this->filter = true;
        }
        $this->action = (\Tools::getIsset('submitReset' . $this->table) ? 'reset_filters' : null);

        $this->postProcess();

        $this->initToolbar();
    }

    public function displaySeverityColor($value, $row)
    {
        switch ($value) {
            case Logger::SEVERITY_INFO:
                return '<span class="badge badge-pill badge-success">' . sprintf($this->module->l('Informative only (%s)', self::FILE_NAME), Logger::SEVERITY_INFO) . '</span>';
            case Logger::SEVERITY_WARNING:
                return '<span class="badge badge-pill badge-warning">' . sprintf($this->module->l('Warning (%s)', self::FILE_NAME), Logger::SEVERITY_WARNING) . '</span>';
            case Logger::SEVERITY_ERROR:
                return '<span class="badge badge-pill badge-danger">' . sprintf($this->module->l('Error (%s)', self::FILE_NAME), Logger::SEVERITY_ERROR) . '</span>';
            case Logger::SEVERITY_CRITICAL:
                return '<span class="badge badge-pill badge-dark">' . sprintf($this->module->l('Major issue (crash)! (%s)', self::FILE_NAME), Logger::SEVERITY_CRITICAL) . '</span>';
            default:
                return $value;
        }
    }

    protected function processExportAllColumns(): bool
    {
        $sql = new \DbQuery();
        $sql->select('a.*, pl.date_add, pl.message, pl.severity');
        $sql->from(\ExpertSenderLog::$definition['table'], 'a');
        $sql->leftJoin('log', 'pl', 'pl.id_log = a.id_log');
        $sql->where('pl.object_type = "expertsenderlog"');
        $sql->orderBy('a.id_log DESC');

        $data = \Db::getInstance()->executeS($sql);

        if (empty($data)) {
            return false;
        }

        $headers = array_keys(reset($data));

        $csvExportService = $this->module->getService(CsvExportServiceInterface::class);
        $csvExportService->exportDataToCsv('expertsenderlogs_export', $headers, $data);

        return true;
    }

    protected function processExportAllLogsFiles(): bool
    {
        $filename = 'logs_es_' . date('Y-m-d_H-i-s') . '.zip';

        $zipExportService = $this->module->getService(LogZipExportServiceInterface::class);

        return $zipExportService->exportToZip($filename);
    }

    /**
     * @return false|string
     *
     * @throws SmartyException
     */
    public function displayDetailsButton(int $logId, string $data, string $logInformationType)
    {
        $unserializedData = json_decode($data);

        if (empty($unserializedData)) {
            return '--';
        }

        $this->context->smarty->assign([
            'log_id' => $logId,
            'log_information_type' => $logInformationType,
        ]);

        return $this->context->smarty->fetch($this->getTemplatePath() . 'logs/modal-log-details.tpl');
    }

    /**
     * @return false|string
     *
     * @throws SmartyException
     */
    public function displayRequestButton(string $request, array $data)
    {
        return $this->displayDetailsButton((int) $data['id_log'], $request, Config::LOG_TYPE_REQUEST);
    }

    /**
     * @return false|string
     *
     * @throws SmartyException
     */
    public function displayResponseButton(string $response, array $data)
    {
        return $this->displayDetailsButton((int) $data['id_log'], $response, Config::LOG_TYPE_RESPONSE);
    }

    /**
     * @return false|string
     *
     * @throws SmartyException
     */
    public function displayContextButton(string $context, array $data)
    {
        return $this->displayDetailsButton((int) $data['id_log'], $context, Config::LOG_TYPE_CONTEXT);
    }
}
