<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Context\GlobalShopContextInterface;
use JustIdea\ExpertSender\Infrastructure\Enum\PermissionType;
use JustIdea\ExpertSender\Infrastructure\Logger\Repository\LogRepositoryInterface;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\ToolsUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LoggerAjaxController extends AbstractAdminController
{
    public const FILE_NAME = 'LoggerAjaxController';

    /**
     * @var GlobalShopContextInterface
     */
    public $globalShopContext;

    /**
     * @var LogRepositoryInterface
     */
    public $logRepository;

    /**
     * @var ToolsUtility
     */
    public $tools;

    public function __construct()
    {
        parent::__construct();

        $this->logRepository = $this->module->getService(LogRepositoryInterface::class);
        $this->tools = $this->module->getService(ToolsUtility::class);
        $this->globalShopContext = $this->module->getService(GlobalShopContextInterface::class);
    }

    public function postProcess()
    {
        $actionMethod = \Tools::getValue('action') . 'Action';

        if (method_exists($this, $actionMethod)) {
            call_user_func([$this, $actionMethod]);
            exit;
        }

        $this->ajaxRender(
            json_encode([
                'success' => false,
                'message' => $this->module->l('Unknown action', self::FILE_NAME),
            ])
        );
        exit;
    }

    private function getLogDetailsAction()
    {
        if (!$this->ensureHasPermissions([PermissionType::EDIT, PermissionType::VIEW], true)) {
            return;
        }

        $logId = $this->tools->getValueAsInteger('id_log');

        try {
            /** @var ExpertSenderLog|null $log */
            $log = $this->logRepository->findOneBy([
                'id_log' => $logId,
                'id_shop' => $this->globalShopContext->getShopId(),
            ]);
        } catch (\Exception $exception) {
            $this->module->getLogger()->error('Failed to find log', [
                'context' => [
                    'id_log' => $logId,
                    'id_shop' => $this->globalShopContext->getShopId(),
                ],
                'exceptions' => ExceptionUtility::getExceptions($exception),
            ]);

            $this->ajaxResponse(json_encode([
                'error' => true,
                'message' => $this->module->l('Failed to find log.', self::FILE_NAME),
            ]));
        }

        if (!$log) {
            $this->module->getLogger()->error('No log information found.', [
                'context' => [
                    'id_log' => $logId,
                    'id_shop' => $this->globalShopContext->getShopId(),
                ],
                'exceptions' => [],
            ]);

            $this->ajaxResponse(json_encode([
                'error' => true,
                'message' => $this->module->l('No log information found.', self::FILE_NAME),
            ]));
        }

        $this->ajaxResponse(json_encode([
            'error' => false,
            'log' => [
                Config::LOG_TYPE_REQUEST => $log->request,
                Config::LOG_TYPE_RESPONSE => $log->response,
                Config::LOG_TYPE_CONTEXT => $log->context,
            ],
        ]));
    }
}
