<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Controller\Admin;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Core\Customer\Processor\SyncCustomersProcessor;
use JustIdea\ExpertSender\Core\Enum\ProcessState;
use JustIdea\ExpertSender\Core\Order\Exception\MissingWebsiteIdException;
use JustIdea\ExpertSender\Core\Order\Processor\SyncOrdersProcessor;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderCustomerRepositoryInterface;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderOrderRepository;
use JustIdea\ExpertSender\Core\Repository\ExpertSenderOrderRepositoryInterface;
use JustIdea\ExpertSender\Infrastructure\Adapter\Validate;
use JustIdea\ExpertSender\Infrastructure\Enum\PermissionType;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;
use JustIdea\ExpertSender\Infrastructure\Utility\TimeZoneUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SyncController extends AbstractAdminController
{
    public const FILE_NAME = 'SyncController';

    /**
     * @var ExpertSenderOrderRepository
     */
    public $orderRepository;

    /**
     * @var ExpertSenderCustomerRepositoryInterface
     */
    public $customerRepository;

    /**
     * @var array
     */
    protected $syncParametres = [];

    protected $statuses_array = [];

    public function __construct()
    {
        parent::__construct();

        $this->override_folder = '_override/es_sync/';
        $this->tpl_folder = '_override/es_sync/';

        $this->syncParametres = [
            Config::SYNC_BATCH_SIZE,
            Config::SYNC_INCLUDE_PHONE,
        ];

        $this->orderRepository = $this->module->getService(ExpertSenderOrderRepositoryInterface::class);
        $this->customerRepository = $this->module->getService(ExpertSenderCustomerRepositoryInterface::class);
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);

        \Media::addJsDef([
            'expertsendersyncontroller' => [
                'formId' => Config::SYNC_PROCESS_CONFIG_FORM_NAME,
                'baseUrl' => $this->buildCronTaskUrl(false),
            ],
        ]);

        $this->addJS(__PS_BASE_URI__ . 'modules/' . $this->module->name . '/views/js/sync-tab.js');
    }

    public function initProcess(): void
    {
        // if (\Tools::isSubmit(Config::OPTIONS_SYNC_SUBMIT)) {
        //     $this->handleSyncSubmit();
        // }

        if (\Tools::isSubmit(Config::SYNC_OPTION_SETTINGS_SUBMIT)) {
            if ($this->saveSyncSettingsForm()) {
                $this->confirmations[] = $this->module->l('Successful update.', self::FILE_NAME);
            }
        }

        parent::initProcess();
    }

    public function saveSyncSettingsForm()
    {
        $result = true;

        foreach (\Tools::getAllValues() as $fieldName => $fieldValue) {
            if (in_array($fieldName, $this->syncParametres) && Config::SYNC_BATCH_SIZE === $fieldName) {
                $fieldValueToSave = Validate::isValidBatchSize((int) $fieldValue) ? (int) $fieldValue : 20;
                $result &= \Configuration::updateValue(Config::SYNC_BATCH_SIZE, $fieldValueToSave);
            } elseif (in_array($fieldName, $this->syncParametres)) {
                $result &= \Configuration::updateValue(\Tools::strtoupper($fieldName), pSQL($fieldValue));
            }
        }

        return $result;
    }

    public function postProcess()
    {
        if (\Tools::isSubmit('submitBulksync' . $this->table)) {
            // Delete multiple CMS content
            if ($this->ensureHasPermissions([PermissionType::EDIT], false, true)) {
                $this->action = 'bulksync';
                $this->boxes = \Tools::getValue($this->table . 'Box');
                if (is_array($this->boxes) && array_key_exists(0, $this->boxes)) {
                    if (!$res = parent::postProcess(true)) {
                        return $res;
                    }
                    \Tools::redirectAdmin(self::$currentIndex . '&conf=4&token=' . \Tools::getAdminTokenLite(Config::SYNC_CONTROLLER));
                }
            } else {
                $this->errors[] = $this->module->l('You do not have permission to delete this.', self::FILE_NAME);
            }
        }

        parent::postProcess();
    }

    public function processBulkSync()
    {
        $identifierList = \Tools::getValue($this->list_id . 'Box', []);

        if (empty($identifierList)) {
            $this->errors[] = $this->module->l('No selection to sync.', self::FILE_NAME);

            return;
        }

        foreach ($identifierList as $id) {
            if ($this->table === \ExpertSenderOrder::$definition['table']) {
                $order = new \ExpertSenderOrder((int) $id);
                $order->synced = 0;
                $order->date_sync = null;
                $order->update();
            } elseif ($this->table === \ExpertSenderCustomer::$definition['table']) {
                $customer = new \ExpertSenderCustomer((int) $id);
                $customer->synced = 0;
                $customer->date_sync = null;
                $customer->update();
            }
        }

        return true;
    }

    protected function processSyncOrder()
    {
        $idOrder = \Tools::getValue(\ExpertSenderOrder::$definition['primary'], null);
        $order = new \ExpertSenderOrder($idOrder);

        if ($order->id_order === null) {
            $this->saveErrorToCookie(sprintf($this->module->l('Order with ID %s cannot be marked for synchronization', self::FILE_NAME), $idOrder));

            goto redirect;
        }

        $order->synced = 0;
        $order->date_sync = null;
        $order->update();

        $this->saveConfirmationToCookie(sprintf($this->module->l('Order with ID %s manually marked for synchronization.', self::FILE_NAME), $idOrder));

        redirect:
        \Tools::redirectAdmin($this->context->link->getAdminLink(Config::SYNC_CONTROLLER));
    }

    protected function processSyncCustomer()
    {
        $idCustomer = \Tools::getValue(\ExpertSenderCustomer::$definition['primary'], null);
        $customer = new \ExpertSenderCustomer($idCustomer);

        if ($customer->id_customer === null) {
            $this->saveErrorToCookie(sprintf($this->module->l('Customer with ID %s cannot be marked for synchronization.', self::FILE_NAME), $idCustomer));

            goto redirect;
        }

        $customer->synced = 0;
        $customer->date_sync = null;
        $customer->update();

        $this->saveConfirmationToCookie(sprintf($this->module->l('Customer with ID %s manually marked for synchronization.', self::FILE_NAME), $idCustomer));

        redirect:
        \Tools::redirectAdmin($this->context->link->getAdminLink(Config::SYNC_CONTROLLER));
    }

    public function initContent(): void
    {
        parent::initContent();

        $this->clearFieldsList();
        $this->renderOrdersList();
        $this->context->smarty->assign('ordersList', $this->renderList());

        $this->clearFieldsList();
        $this->renderCustomersList();
        $this->context->smarty->assign('customerList', $this->renderList());

        $this->clearFieldsForm();
        $this->initSyncSettingsForm();
        $this->context->smarty->assign('syncSettingsForm', $this->renderForm());

        $this->clearFieldsForm();
        $this->initProcessSyncForm();
        $this->context->smarty->assign('processSyncForm', $this->renderForm());

        $this->context->smarty->assign('ordersRemainingInQueue', $this->orderRepository->getTotalRemainingOrdersInQueue());
        $this->context->smarty->assign('customersRemainingInQueue', $this->customerRepository->getTotalRemainingCustomersInQueue());

        $this->content = $this->context->smarty->fetch($this->getTemplatePath() . 'tabs/tab-sync.tpl');
        $this->context->smarty->assign('content', $this->content);
    }

    public function initPageHeaderToolbar()
    {
        if (empty($this->display)) {
            $this->page_header_toolbar_btn['expertsender-import-orders'] = [
                'href' => $this->context->link->getAdminLink(Config::IMPORT_ORDER_CONTROLLER),
                'desc' => $this->module->l('Export Orders', self::FILE_NAME),
                'icon' => 'process-icon-cart-plus icon-cart-plus',
            ];

            $this->page_header_toolbar_btn['expertsender-import-customers'] = [
                'href' => $this->context->link->getAdminLink(Config::IMPORT_CUSTOMER_CONTROLLER),
                'desc' => $this->module->l('Export Customers', self::FILE_NAME),
                'icon' => 'process-icon-user-plus icon-user-plus',
            ];
        }

        parent::initPageHeaderToolbar();
    }

    public function renderOrdersList()
    {
        $this->toolbar_title = $this->module->l('Orders to sync', self::FILE_NAME);
        $this->table = \ExpertSenderOrder::$definition['table'];
        $this->list_id = \ExpertSenderOrder::$definition['table'];
        $this->identifier = \ExpertSenderOrder::$definition['primary'];
        $this->className = \ExpertSenderOrder::class;
        $this->multiple_fieldsets = true;
        $this->show_sql_query = true;
        $this->actions_available = [''];
        $this->list_no_link = true;
        $this->_use_found_rows = true;

        $this->addRowAction('vieworders');
        $this->addRowAction('syncorders');

        $this->bulk_actions = [
            'sync' => [
                'text' => $this->module->l('Resynchronize selected', self::FILE_NAME),
                'confirm' => $this->module->l('Mark selected items as unsynchronized?', self::FILE_NAME),
                'icon' => 'icon-refresh',
            ],
        ];

        if (\Tools::isSubmit('submitFilter' . $this->table) || false !== $this->context->cookie->{'submitFilter' . $this->table}) {
            $this->filter = true;
        }
        $this->action = (\Tools::getIsset('submitReset' . $this->table) ? 'reset_filters' : '');

        $this->_defaultOrderBy = 'id_order';
        $this->_defaultOrderWay = 'DESC';

        $this->_select .= '
        o.date_add AS order_date_add,
		CONCAT(LEFT(c.`firstname`, 1), \'. \', c.`lastname`) AS `customer`,
        o.reference,
        o.total_paid_tax_incl,
        o.payment,
		os.color,
        osl.name AS osname';

        $this->_join .= ' LEFT JOIN `' . _DB_PREFIX_ . 'orders` o ON (o.`id_order` = a.`id_order`)
                        LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON (c.`id_customer` = o.`id_customer`)
                        LEFT JOIN `' . _DB_PREFIX_ . 'order_state` os ON (os.`id_order_state` = o.`current_state`)
                        LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` osl ON (os.`id_order_state` = osl.`id_order_state` AND osl.`id_lang` = ' . (int) $this->context->language->id . ')';

        unset($this->toolbar_btn['new']);

        $statuses = \OrderState::getOrderStates((int) $this->context->language->id);
        foreach ($statuses as $status) {
            $this->statuses_array[$status['id_order_state']] = $status['name'];
        }

        $fields_list = [
            'id_order' => [
                'title' => $this->module->l('ID', self::FILE_NAME),
                'align' => 'center',
                'filter_key' => 'a!id_order',
            ],
            'reference' => [
                'title' => $this->module->l('Reference', self::FILE_NAME),
                'align' => 'text-left',
            ],
            'customer' => [
                'title' => $this->module->l('Customer', self::FILE_NAME),
                'align' => 'text-left',
                'havingFilter' => true,
            ],
            'total_paid_tax_incl' => [
                'title' => $this->module->l('Total Paid', self::FILE_NAME),
                'align' => 'text-right',
                'type' => 'price',
                'currency' => true,
            ],
            'payment' => [
                'title' => $this->module->l('Payment Method', self::FILE_NAME),
                'align' => 'text-left',
            ],
            'osname' => [
                'title' => $this->module->l('Status', self::FILE_NAME),
                'type' => 'select',
                'color' => 'color',
                'list' => $this->statuses_array,
                'filter_key' => 'os!id_order_state',
                'filter_type' => 'int',
                'order_key' => 'osname',
            ],
            'synced' => [
                'title' => $this->module->l('Synced', self::FILE_NAME),
                'type' => 'bool',
                'align' => 'center',
                'orderby' => false,
                'badge_success' => true,
                'badge_danger' => true,
            ],
            'order_date_add' => [
                'title' => $this->module->l('Order Date', self::FILE_NAME),
                'align' => 'text-right',
                'type' => 'datetime',
            ],
            'date_sync' => [
                'title' => $this->module->l('Sync Date', self::FILE_NAME),
                'align' => 'text-right',
                'type' => 'datetime',
            ],
        ];

        $this->fields_list = $fields_list;

        $this->postProcess();

        $this->initToolbar();
    }

    public function renderCustomersList()
    {
        $this->toolbar_title = $this->module->l('Customers to sync', self::FILE_NAME);
        $this->table = \ExpertSenderCustomer::$definition['table'];
        $this->list_id = \ExpertSenderCustomer::$definition['table'];
        $this->identifier = \ExpertSenderCustomer::$definition['primary'];
        $this->className = \ExpertSenderCustomer::class;
        $this->multiple_fieldsets = true;
        $this->show_sql_query = true;
        $this->actions_available = [''];
        $this->list_no_link = true;
        $this->_use_found_rows = true;

        $this->addRowAction('viewcustomers');
        $this->addRowAction('synccustomers');

        $this->bulk_actions = [
            'sync' => [
                'text' => $this->module->l('Resynchronize selected', self::FILE_NAME),
                'confirm' => $this->module->l('Mark selected items as unsynchronized?', self::FILE_NAME),
                'icon' => 'icon-refresh',
            ],
        ];

        if (\Tools::isSubmit('submitFilter' . $this->table) || false !== $this->context->cookie->{'submitFilter' . $this->table}) {
            $this->filter = true;
        }
        $this->action = (\Tools::getIsset('submitReset' . $this->table) ? 'reset_filters' : '');

        $this->_defaultOrderBy = 'id_customer';
        $this->_defaultOrderWay = 'DESC';

        $this->_select .= '
        c.`firstname`,
        c.`lastname`,
        c.`email`,
        (
            SELECT SUM(total_paid_real / conversion_rate)
            FROM ' . _DB_PREFIX_ . 'orders o
            WHERE o.id_customer = a.id_customer
            ' . \Shop::addSqlRestriction(\Shop::SHARE_ORDER, 'o') . '
            AND o.valid = 1
        ) as total_spent,
        c.`active`,
        c.`newsletter`,
        c.`optin`,
        c.`date_add` AS registration_date';

        $this->_join .= '
            LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON (c.`id_customer` = a.`id_customer`)';

        unset($this->toolbar_btn['new']);

        $statuses = \OrderState::getOrderStates((int) $this->context->language->id);
        foreach ($statuses as $status) {
            $this->statuses_array[$status['id_order_state']] = $status['name'];
        }

        $fields_list = [
            'id_customer' => [
                'title' => $this->module->l('ID', self::FILE_NAME),
                'align' => 'center',
                'filter_key' => 'a!id_customer',
            ],
            'firstname' => [
                'title' => $this->module->l('First name', self::FILE_NAME),
                'align' => 'text-left',
                'maxlength' => 30,
                'havingFilter' => true,
            ],
            'lastname' => [
                'title' => $this->module->l('Last name', self::FILE_NAME),
                'align' => 'text-left',
                'maxlength' => 30,
                'havingFilter' => true,
            ],
            'total_spent' => [
                'title' => $this->module->l('Sales', self::FILE_NAME),
                'type' => 'price',
                'havingFilter' => true,
                'align' => 'text-right',
            ],
            'email' => [
                'title' => $this->module->l('Email address', self::FILE_NAME),
                'align' => 'text-left',
                'maxlength' => 50,
            ],
            'active' => [
                'title' => $this->module->l('Enabled', self::FILE_NAME),
                'align' => 'text-center',
                'type' => 'bool',
                'orderby' => false,
                'filter_key' => 'c!active',
            ],
            'newsletter' => [
                'title' => $this->module->l('Newsletter', self::FILE_NAME),
                'align' => 'center',
                'type' => 'bool',
            ],
            'optin' => [
                'title' => $this->module->l('Partner offers', self::FILE_NAME),
                'align' => 'center',
                'type' => 'bool',
            ],
            'registration_date' => [
                'title' => $this->module->l('Registration', self::FILE_NAME),
                'align' => 'text-right',
                'type' => 'datetime',
            ],
            'synced' => [
                'title' => $this->module->l('Synced', self::FILE_NAME),
                'type' => 'bool',
                'align' => 'center',
                'orderby' => false,
                'badge_success' => true,
                'badge_danger' => true,
            ],
            'date_sync' => [
                'title' => $this->module->l('Sync Date', self::FILE_NAME),
                'align' => 'text-right',
                'type' => 'datetime',
            ],
        ];

        $this->fields_list = $fields_list;

        $this->postProcess();

        $this->initToolbar();
    }

    public function getList($id_lang, $orderBy = null, $orderWay = null, $start = 0, $limit = null, $id_lang_shop = null)
    {
        parent::getList($id_lang, $orderBy, $orderWay, $start, $limit, $id_lang_shop);

        if ($this->table === \ExpertSenderOrder::$definition['table'] && $this->_list) {
            foreach ($this->_list as &$row) {
                $row['badge_success'] = $row['synced'];
                $row['badge_danger'] = !$row['synced'];
                $row['date_sync'] = TimeZoneUtility::isEmptyDate($row['date_sync']) ? null : $row['date_sync'];
            }
        } elseif ($this->table === \ExpertSenderCustomer::$definition['table'] && $this->_list) {
            foreach ($this->_list as &$row) {
                $row['badge_success'] = $row['synced'];
                $row['badge_danger'] = !$row['synced'];
                $row['date_sync'] = TimeZoneUtility::isEmptyDate($row['date_sync']) ? null : $row['date_sync'];
            }
        }
    }

    /**
     * Display vieworders action link.
     *
     * @param string $token the token to add to the link
     * @param int $id the identifier to add to the link
     *
     * @return string
     */
    public function displayViewordersLink($token, $id)
    {
        if ($this->ensureHasPermissions([PermissionType::EDIT], false, true)) {
            if (!array_key_exists('vieworders', self::$cache_lang)) {
                self::$cache_lang['vieworders'] = $this->module->l('View', self::FILE_NAME);
            }

            $href = self::$currentIndex . '&token=' . (null != $token ? $token : $this->token);

            $this->context->smarty->assign([
                'href' => $this->context->link->getAdminLink('AdminOrders') .
                '&id_order=' . (int) $id . '&vieworder&back=' . urlencode($href),
                'action' => self::$cache_lang['vieworders'],
            ]);

            return $this->context->smarty->fetch('helpers/list/list_action_view.tpl');
        } else {
            return;
        }
    }

    /**
     * Display viewcustomers action link.
     *
     * @param string $token the token to add to the link
     * @param int $id the identifier to add to the link
     *
     * @return string
     */
    public function displayViewcustomersLink($token, $id)
    {
        if ($this->ensureHasPermissions([PermissionType::EDIT], false, true)) {
            if (!array_key_exists('viewcustomers', self::$cache_lang)) {
                self::$cache_lang['viewcustomers'] = $this->module->l('View', self::FILE_NAME);
            }

            $href = self::$currentIndex . '&token=' . (null != $token ? $token : $this->token);

            $this->context->smarty->assign([
                'href' => $this->context->link->getAdminLink('AdminCustomers') .
                '&id_customer=' . (int) $id . '&viewcustomer&back=' . urlencode($href),
                'action' => self::$cache_lang['viewcustomers'],
            ]);

            return $this->context->smarty->fetch('helpers/list/list_action_view.tpl');
        }
    }

    /**
     * Display editcustomers action link.
     *
     * @param string $token the token to add to the link
     * @param int $id the identifier to add to the link
     *
     * @return string
     */
    public function displayEditcustomersLink($token, $id)
    {
        if ($this->ensureHasPermissions([PermissionType::EDIT], false, true)) {
            if (!array_key_exists('editcustomers', self::$cache_lang)) {
                self::$cache_lang['editcustomers'] = $this->module->l('Edit', self::FILE_NAME);
            }

            $href = self::$currentIndex . '&token=' . (null != $token ? $token : $this->token);

            $this->context->smarty->assign([
                'href' => $this->context->link->getAdminLink('AdminCustomers') .
                '&id_customer=' . (int) $id . '&updatecustomer&back=' . urlencode($href),
                'action' => self::$cache_lang['editcustomers'],
            ]);

            return $this->context->smarty->fetch('helpers/list/list_action_edit.tpl');
        }
    }

    public function displaySyncordersLink($token, $id)
    {
        if ($this->ensureHasPermissions([PermissionType::EDIT], false, true)) {
            if (!array_key_exists('syncorders', self::$cache_lang)) {
                self::$cache_lang['syncorders'] = $this->module->l('Mark as unsynced', self::FILE_NAME);
            }

            $href = self::$currentIndex . '&' . \ExpertSenderOrder::$definition['primary'] . '=' . $id . '&action=syncOrder&token=' . (null != $token ? $token : $this->token);

            $this->context->smarty->assign([
                'href' => $href,
                'action' => self::$cache_lang['syncorders'],
            ]);

            return $this->context->smarty->fetch($this->getTemplatePath() . $this->override_folder . 'helpers/list/list_action_sync.tpl');
        }
    }

    public function displaySynccustomersLink($token, $id)
    {
        if ($this->ensureHasPermissions([PermissionType::EDIT], false, true)) {
            if (!array_key_exists('synccustomers', self::$cache_lang)) {
                self::$cache_lang['synccustomers'] = $this->module->l('Mark as unsynced', self::FILE_NAME);
            }

            $href = self::$currentIndex . '&' . \ExpertSenderCustomer::$definition['primary'] . '=' . $id . '&action=syncCustomer&token=' . (null != $token ? $token : $this->token);

            $this->context->smarty->assign([
                'href' => $href,
                'action' => self::$cache_lang['synccustomers'],
            ]);

            return $this->context->smarty->fetch($this->getTemplatePath() . $this->override_folder . 'helpers/list/list_action_sync.tpl');
        }
    }

    private function initSyncSettingsForm()
    {
        $formFields = [
            [
                'type' => 'switch',
                'name' => Config::SYNC_INCLUDE_PHONE,
                'label' => $this->module->l('Include phone', self::FILE_NAME),
                'is_bool' => true,
                'desc' => $this->module->l('Check this option if you want to pass the phone number during the customer synchronization process.', self::FILE_NAME),
                'class' => 'es-cron-param',
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
            [
                'type' => 'desc',
                'name' => Config::SYNC_INCLUDE_PHONE . Config::CONST_STRING_DESCRPTION,
                'template' => Config::SYNC_INCLUDE_PHONE . Config::CONST_STRING_DESCRPTION,
                'class' => '',
            ],
            [
                'type' => 'select',
                'name' => Config::SYNC_BATCH_SIZE,
                'label' => $this->module->l('Batch size', self::FILE_NAME),
                'desc' => $this->module->l('Select how many customers and orders should be synchronized for a single API request.', self::FILE_NAME),
                'class' => 'fixed-width-xxl es-chosen es-internal-mapping-field-value',
                'options' => [
                    'query' => Config::getBatchSizeValuesArray(),
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'type' => 'desc',
                'name' => Config::SYNC_BATCH_SIZE . Config::CONST_STRING_DESCRPTION,
                'template' => Config::SYNC_BATCH_SIZE . Config::CONST_STRING_DESCRPTION,
                'class' => '',
            ],
        ];

        $this->fields_form['form'] = [
            'form' => [
                'id_form' => Config::SYNC_SETTINGS_CONFIG_FORM_NAME,
                'legend' => [
                    'title' => $this->module->l('Sync Settings', self::FILE_NAME),
                    'icon' => 'icon-cogs',
                ],
                'input' => $formFields,
                'submit' => [
                    'title' => $this->module->l('Save', self::FILE_NAME),
                    'name' => Config::SYNC_OPTION_SETTINGS_SUBMIT,
                    'id' => Config::SYNC_OPTION_SETTINGS_SUBMIT . 'SubmitBtn',
                    'class' => 'btn btn-default pull-right',
                    'icon' => 'process-icon-save',
                ],
            ],
        ];

        $values = [];
        foreach ($this->syncParametres as $parameter) {
            $values[$parameter] = \Configuration::get(\Tools::strtoupper($parameter));
        }
        $this->tpl_form_vars = array_merge($this->tpl_form_vars, $values);
    }

    public function ajaxProcessInitSyncSubmit()
    {
        $syncOrders = Config::OPTION_ON === \Tools::getValue(Config::SYNC_ORDERS);
        $syncCustomers = Config::OPTION_ON === \Tools::getValue(Config::SYNC_CUSTOMERS);
        $syncReset = Config::OPTION_ON === \Tools::getValue(Config::SYNC_RESET);

        $this->module->getReportLogger()->appendSessionLogFileWithLineBreak();
        $this->module->getReportLogger()->info(sprintf($this->module->l('Started the sync process: orders (%s), customers (%s).', self::FILE_NAME), $syncOrders ? $this->module->l('yes', self::FILE_NAME) : $this->module->l('no', self::FILE_NAME), $syncCustomers ? $this->module->l('yes', self::FILE_NAME) : $this->module->l('no', self::FILE_NAME)));

        if ($syncReset && $syncOrders) {
            \Configuration::updateValue(Config::LAST_SYNCED_ORDER_ID, 0);
        }

        if ($syncReset && $syncCustomers) {
            \Configuration::updateValue(Config::LAST_SYNCED_CUSTOMER_ID, 0);
        }

        \Configuration::updateValue(Config::SYNC_PROCESS_ORDER_IN_PROGRESS, false);
        \Configuration::updateValue(Config::SYNC_PROCESS_ORDER_FINISHED, false);
        \Configuration::updateValue(Config::SYNC_PROCESS_CUSTOMER_IN_PROGRESS, false);
        \Configuration::updateValue(Config::SYNC_PROCESS_CUSTOMER_FINISHED, false);

        exit(json_encode([
            'logs' => $this->module->getReportLogger()->getSessionLogFileContentsAsString(),
        ]));
    }

    public function ajaxProcessHandleSyncSubmit()
    {
        $continue = $this->handleSyncSubmit();

        $logs = $this->module->getReportLogger()->getSessionLogFileContentsAsString();
        // $this->module->getReportLogger()->archiveSessionLogs();

        exit(json_encode([
            'logs' => $logs,
            'continue' => $continue,
            'order_sync_runtime' => (int) \Configuration::get(Config::LASTTIME_SYNC_ORDER_DATE) ? \Configuration::get(Config::LASTTIME_SYNC_ORDER_DATE) : $this->module->l('never', self::FILE_NAME),
            'order_sync_object_id' => (int) \Configuration::get(Config::LAST_SYNCED_ORDER_ID) ? \Configuration::get(Config::LAST_SYNCED_ORDER_ID) : $this->module->l('none', self::FILE_NAME),
            'customer_sync_runtime' => (int) \Configuration::get(Config::LASTTIME_SYNC_CUSTOMER_DATE) ? \Configuration::get(Config::LASTTIME_SYNC_CUSTOMER_DATE) : $this->module->l('never', self::FILE_NAME),
            'customer_sync_object_id' => (int) \Configuration::get(Config::LAST_SYNCED_CUSTOMER_ID) ? \Configuration::get(Config::LAST_SYNCED_CUSTOMER_ID) : $this->module->l('none', self::FILE_NAME),
        ]));
    }

    private function handleSyncSubmit(): bool
    {
        $continue = false;

        if (Config::OPTION_ON === \Tools::getValue(Config::SYNC_ORDERS)) {
            try {
                $result = $this->syncOrders();

                switch ($result) {
                    case ProcessState::SUCCESS:
                        $continue |= true;
                        // $this->confirmations[] = $this->module->l('Orders synchronized successfully.', self::FILE_NAME) . nl2br("\r\n");
                        break;

                    case ProcessState::NO_AFFECT:
                        // $this->module->getReportLogger()->info($this->module->l('No orders to be synced.', self::FILE_NAME));
                        // $this->warnings[] = $this->module->l('No orders to be synced.', self::FILE_NAME) . nl2br("\r\n");
                        break;

                    default:
                        // code...
                        break;
                }
            } catch (\Throwable $exception) {
                if ($exception instanceof MissingWebsiteIdException) {
                    $this->module->getReportLogger()->error($this->module->l('The site ID has not been specified or is incorrect. Go to the Settings tab and enter the site ID.', self::FILE_NAME));
                } else {
                    $this->module->getReportLogger()->error($this->module->l('Failed to sync the orders. View error logs for more information.', self::FILE_NAME));
                }

                $this->module->getLogger()->error('Failed to sync the orders.', [
                    'context' => [],
                    'request' => ExceptionUtility::getLastRequest($exception),
                    'response' => ExceptionUtility::getLastResponse($exception),
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ]);
            }
        }

        if (Config::OPTION_ON === \Tools::getValue(Config::SYNC_CUSTOMERS)) {
            try {
                $result = $this->syncCustomers();

                switch ($result) {
                    case ProcessState::SUCCESS:
                        $continue |= true;
                        // $this->confirmations[] = $this->module->l('Customers synchronized successfully.', self::FILE_NAME) . nl2br("\r\n");
                        break;

                    case ProcessState::NO_AFFECT:
                        // $this->module->getReportLogger()->info($this->module->l('No customers to be synced.', self::FILE_NAME));
                        // $this->warnings[] = $this->module->l('No customers to be synced.', self::FILE_NAME) . nl2br("\r\n");
                        break;

                    default:
                        // code...
                        break;
                }
            } catch (\Throwable $exception) {
                $this->module->getReportLogger()->error($this->module->l('Failed to sync the customers. View error logs for more information.', self::FILE_NAME));

                $this->module->getLogger()->error('Failed to sync the customers.', [
                    'context' => [],
                    'request' => ExceptionUtility::getLastRequest($exception),
                    'response' => ExceptionUtility::getLastResponse($exception),
                    'exceptions' => ExceptionUtility::getExceptions($exception),
                ]);
            }
        }

        return (bool) $continue;
    }

    private function syncOrders(): string
    {
        /** @var SyncOrdersProcessor $syncOrdersProcessor */
        $syncOrdersProcessor = $this->module->getService(SyncOrdersProcessor::class);

        return $syncOrdersProcessor->processAction();
    }

    private function syncCustomers(): string
    {
        /** @var SyncCustomersProcessor $syncCustomersProcessor */
        $syncCustomersProcessor = $this->module->getService(SyncCustomersProcessor::class);

        return $syncCustomersProcessor->processAction();
    }

    private function initProcessSyncForm()
    {
        $formFields = [
            [
                'type' => 'desc_full_width',
                'name' => Config::OPTIONS_SYNC_SUBMIT . Config::CONST_STRING_CRON_DESCRPTION,
                'template' => Config::OPTIONS_SYNC_SUBMIT . Config::CONST_STRING_CRON_DESCRPTION,
                'class' => '',
            ],
            [
                'type' => 'desc_full_width',
                'name' => Config::OPTIONS_SYNC_SUBMIT . Config::CONST_STRING_DESCRPTION,
                'template' => Config::OPTIONS_SYNC_SUBMIT . Config::CONST_STRING_DESCRPTION,
                'class' => '',
            ],
            [
                'type' => 'switch',
                'name' => Config::SYNC_ORDERS,
                'label' => $this->module->l('Orders', self::FILE_NAME),
                'is_bool' => true,
                'desc' => $this->module->l('Check this option if you want to include order synchronization.', self::FILE_NAME),
                'class' => 'es-cron-param',
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
            [
                'type' => 'switch',
                'name' => Config::SYNC_CUSTOMERS,
                'label' => $this->module->l('Customers', self::FILE_NAME),
                'is_bool' => true,
                'desc' => $this->module->l('Check this option if you want to include customer synchronization.', self::FILE_NAME),
                'class' => 'es-cron-param',
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
            [
                'type' => 'html',
                'name' => 'form_hr_1',
                'html_content' => '<hr>',
            ],
            [
                'type' => 'switch',
                'name' => Config::SYNC_RESET,
                'label' => $this->module->l('Start from beginning', self::FILE_NAME),
                'is_bool' => true,
                'desc' => $this->module->l('Check this option if you want to start the full synchronization from the beginning (including previous errors).', self::FILE_NAME),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => 1,
                        'label' => $this->module->l('Enabled', self::FILE_NAME),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => 0,
                        'label' => $this->module->l('Disabled', self::FILE_NAME),
                    ],
                ],
            ],
            [
                'type' => 'logs_textarea',
                'label' => $this->module->l('Sync logs', self::FILE_NAME),
                'name' => Config::LOGS_TEXTAREA,
                'desc' => $this->module->l('To enable or disable the storage of logs after synchronization or to clear logs from the synchronisation process, go to the Logs tab.', self::FILE_NAME),
                'id' => Config::LOGS_TEXTAREA,
                'value' => $this->module->getReportLogger()->getLogFileContentsAsString(),
            ],
            [
                'type' => 'progress_bar',
                // 'label' => $this->module->l('Progress', self::FILE_NAME),
                'name' => Config::LOGS_PROGRESS_BAR,
                'id' => Config::LOGS_PROGRESS_BAR,
                'bar_type' => 'progress-bar-info',
                'bar_value' => '100',
            ],
            [
                'type' => 'desc_full_width',
                'name' => Config::OPTIONS_SYNC_SUBMIT . Config::CONST_STRING_DETAILS_DESCRPTION,
                'template' => Config::OPTIONS_SYNC_SUBMIT . Config::CONST_STRING_DETAILS_DESCRPTION,
                'class' => '',
            ],
        ];

        $this->fields_form['form'] = [
            'form' => [
                'id_form' => Config::SYNC_PROCESS_CONFIG_FORM_NAME,
                'legend' => [
                    'title' => $this->module->l('Sync', self::FILE_NAME),
                    'icon' => 'icon-refresh',
                ],
                'input' => $formFields,
                'buttons' => [
                    'startSyncButton' => [
                        'title' => $this->module->l('Synchronize', self::FILE_NAME),
                        'name' => Config::OPTIONS_START_SYNC_SUBMIT,
                        'id' => Config::OPTIONS_START_SYNC_SUBMIT,
                        'class' => 'btn btn-success pull-right',
                        'icon' => 'process-icon-cogs',
                        'js' => 'ExpertSenderOfficial.startSyncJob()',
                    ],
                    'stopSyncButton' => [
                        'title' => $this->module->l('Stop', self::FILE_NAME),
                        'name' => Config::OPTIONS_STOP_SYNC_SUBMIT,
                        'id' => Config::OPTIONS_STOP_SYNC_SUBMIT,
                        'class' => 'btn btn-danger pull-right mr-0',
                        'icon' => 'process-icon-cogs',
                        'js' => 'ExpertSenderOfficial.stopSyncJob()',
                    ],
                ],
            ],
        ];

        $tpl_vars = [
            'cron_task_order_sync_runtime' => (int) \Configuration::get(Config::LASTTIME_SYNC_ORDER_DATE) ? \Configuration::get(Config::LASTTIME_SYNC_ORDER_DATE) : $this->module->l('never', self::FILE_NAME),
            'cron_task_order_sync_object_id' => (int) \Configuration::get(Config::LAST_SYNCED_ORDER_ID) ? \Configuration::get(Config::LAST_SYNCED_ORDER_ID) : $this->module->l('none', self::FILE_NAME),
            'cron_task_customer_sync_runtime' => (int) \Configuration::get(Config::LASTTIME_SYNC_CUSTOMER_DATE) ? \Configuration::get(Config::LASTTIME_SYNC_CUSTOMER_DATE) : $this->module->l('never', self::FILE_NAME),
            'cron_task_customer_sync_object_id' => (int) \Configuration::get(Config::LAST_SYNCED_CUSTOMER_ID) ? \Configuration::get(Config::LAST_SYNCED_CUSTOMER_ID) : $this->module->l('none', self::FILE_NAME),
            'cron_task_sync_url' => $this->buildCronTaskUrl(),
            'cron_task_sync_only_orders_url' => $this->buildCronTaskUrl(true, 1, 0),
            'cron_task_sync_only_customers_url' => $this->buildCronTaskUrl(true, 0, 1),
            'cron_manual_pdf_url' => $this->buildCronManualPdfUrl(),
        ];

        $this->context->smarty->assign($tpl_vars);

        $tpl_form_vars = [
            Config::SYNC_ORDERS => 1,
            Config::SYNC_CUSTOMERS => 1,
            Config::SYNC_RESET => 0,
        ];

        $this->tpl_form_vars = array_merge($this->tpl_form_vars, $tpl_form_vars);
    }

    private function buildCronTaskUrl(bool $full = true, int $orders = 1, int $customers = 1)
    {
        $token = \Configuration::getGlobalValue(Config::CRON_TOKEN);
        $cronTaskUrl = $this->context->link->getAdminLink(Config::CRON_CONTROLLER, false);
        $cronTaskUrl .= '&secure_key=' . $token;
        $cronTaskUrl .= '&method=syncDataThroughApi';

        if ($full) {
            $cronTaskUrl .= '&orders=' . $orders;
            $cronTaskUrl .= '&customers=' . $customers;
        }

        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $adminFolder = basename(_PS_ADMIN_DIR_);
            $path = \Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . $adminFolder . '/';
            $cronTaskUrl = $path . $cronTaskUrl;
        }

        return $cronTaskUrl;
    }

    private function buildCronManualPdfUrl()
    {
        $token = \Configuration::getGlobalValue(Config::AJAX_TOKEN);
        $taskUrl = $this->context->link->getAdminLink(Config::AJAX_CONTROLLER, true);
        $taskUrl .= '&secure_key=' . $token;
        $taskUrl .= '&method=downloadCronjobManual';

        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $adminFolder = basename(_PS_ADMIN_DIR_);
            $path = \Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . $adminFolder . '/';
            $taskUrl = $path . $taskUrl;
        }

        return $taskUrl;
    }
}
