<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Database\Install;

use JustIdea\ExpertSender\Infrastructure\Adapter\Language;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Adapter\Tab;
use JustIdea\ExpertSender\Infrastructure\Database\Exception\CouldNotInstallModule;
use JustIdea\ExpertSender\Infrastructure\Database\ModuleTabs;
use JustIdea\ExpertSender\Infrastructure\EntityManager\EntityManagerInterface;
use JustIdea\ExpertSender\Infrastructure\EntityManager\ObjectModelUnitOfWork;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ModuleTabInstaller implements InstallerInterface
{
    /**
     * @var Tab
     */
    private $tab;

    /**
     * @var Language
     */
    private $language;

    /**
     * @var ModuleTabs
     */
    private $moduleTabs;

    /**
     * @var EntityManagerInterface
     */
    private $entityManager;

    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    public function __construct(
        ModuleFactory $moduleFactory,
        Tab $tab,
        Language $language,
        ModuleTabs $moduleTabs,
        EntityManagerInterface $entityManager
    ) {
        $this->tab = $tab;
        $this->language = $language;
        $this->moduleTabs = $moduleTabs;
        $this->entityManager = $entityManager;
        $this->module = $moduleFactory->getModule();
    }

    /**
     * @throws CouldNotInstallModule
     */
    public function init(): void
    {
        $tabs = $this->moduleTabs->getTabs();

        foreach ($tabs as $tab) {
            if ($this->tab->getIdFromClassName($tab['class_name'])) {
                continue;
            }

            $this->installTab(
                $tab['class_name'],
                $tab['parent_class_name'],
                $tab['name'],
                isset($tab['icon']) ? $tab['icon'] : null
            );
        }
    }

    /**
     * @throws CouldNotInstallModule
     */
    private function installTab(string $className, ?string $parent, array $name, ?string $icon): void
    {
        $idParent = null !== $parent ? $this->tab->getIdFromClassName($parent . '_MTR') : -1;

        if (!$idParent && null !== $parent) {
            $idParent = $this->tab->getIdFromClassName($parent);
        }

        $moduleTab = $this->tab->initTab();
        $moduleTab->class_name = $className;
        $moduleTab->id_parent = $idParent;
        $moduleTab->module = $this->module->name;
        if (property_exists($moduleTab, 'icon') && $icon) {
            $moduleTab->icon = $icon;
        }

        $languages = $this->language->getAllLanguages();
        foreach ($languages as $language) {
            $moduleTab->name[$language['id_lang']] = isset($name[$language['iso_code']]) ? pSQL($name[$language['iso_code']]) : pSQL($name['en']);
        }

        try {
            $this->entityManager->persist($moduleTab, ObjectModelUnitOfWork::UNIT_OF_WORK_SAVE);
            $this->entityManager->flush();
        } catch (\Exception $exception) {
            throw CouldNotInstallModule::failedToInstallModuleTab($className, $exception);
        }
    }
}
