<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Database;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Translator\AdminModuleTabTranslatorInterface;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ModuleTabs
{
    /**
     * @var \ModuleCore&\Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * @var AdminModuleTabTranslatorInterface
     */
    private $adminModuleTabTranslator;

    public function __construct(
        ModuleFactory $moduleFactory,
        AdminModuleTabTranslatorInterface $adminModuleTabTranslator
    ) {
        $this->module = $moduleFactory->getModule();
        $this->adminModuleTabTranslator = $adminModuleTabTranslator;
    }

    public function getTabs(): array
    {
        return [
            [
                'name' => $this->getTabName(Config::SIDEBAR_CONTROLLER),
                'parent_class_name' => Config::LEGACY_PARENT_CONTROLLER,
                'class_name' => Config::SIDEBAR_CONTROLLER,
                'icon' => 'notifications_active',
                'module_tab' => true,
                'visible' => true,
            ],
            [
                'name' => $this->getTabName(Config::TAB_CONTROLLER),
                'parent_class_name' => Config::SIDEBAR_CONTROLLER,
                'class_name' => Config::TAB_CONTROLLER,
                'module_tab' => true,
                'visible' => true,
            ],
            [
                'name' => $this->getTabName(Config::SETTINGS_CONTROLLER),
                'parent_class_name' => Config::TAB_CONTROLLER,
                'class_name' => Config::SETTINGS_CONTROLLER,
                'module_tab' => true,
                'visible' => true,
            ],
            [
                'name' => $this->getTabName(Config::ASSOCIATION_CONTROLLER),
                'parent_class_name' => Config::TAB_CONTROLLER,
                'class_name' => Config::ASSOCIATION_CONTROLLER,
                'module_tab' => true,
                'visible' => true,
            ],
            [
                'name' => $this->getTabName(Config::SYNC_CONTROLLER),
                'parent_class_name' => Config::TAB_CONTROLLER,
                'class_name' => Config::SYNC_CONTROLLER,
                'module_tab' => true,
                'visible' => true,
            ],
            [
                'name' => $this->getTabName(Config::LOG_CONTROLLER),
                'parent_class_name' => Config::TAB_CONTROLLER,
                'class_name' => Config::LOG_CONTROLLER,
                'module_tab' => true,
                'visible' => true,
            ],
            [
                'name' => $this->getTabName(Config::ASSOCIATION_CONSENTS_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::ASSOCIATION_CONSENTS_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::ASSOCIATION_CUSTOMERS_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::ASSOCIATION_CUSTOMERS_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::ASSOCIATION_ORDERS_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::ASSOCIATION_ORDERS_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::ASSOCIATION_ORDER_STATES_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::ASSOCIATION_ORDER_STATES_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::ASSOCIATION_PRODUCTS_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::ASSOCIATION_PRODUCTS_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::IMPORT_ORDER_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::IMPORT_ORDER_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::IMPORT_CUSTOMER_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::IMPORT_CUSTOMER_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::LOGGER_AJAX_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::LOGGER_AJAX_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::AJAX_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::AJAX_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
            [
                'name' => $this->getTabName(Config::CRON_CONTROLLER),
                'parent_class_name' => null,
                'class_name' => Config::CRON_CONTROLLER,
                'module_tab' => false,
                'visible' => false,
            ],
        ];
    }

    private function getTabName($controllerName): array
    {
        $tabName = $this->adminModuleTabTranslator->translate($controllerName);

        $tabName['en'] = isset($tabName['en']) ? $tabName['en'] : 'Missing';
        $tabName['en-US'] = isset($tabName['en-US']) ? $tabName['en-US'] : 'Missing';

        return $tabName;
    }

    public function getTabsClassNames(): array
    {
        $filtered = [];
        $tabs = $this->getTabs();

        foreach ($tabs as $tab) {
            if (isset($tab['visible']) && $tab['visible']) {
                $filtered[] = $tab['class_name'];
            }
        }

        return $filtered;
    }
}
