<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Database\Uninstall;

use JustIdea\ExpertSender\Infrastructure\Database\Exception\CouldNotUninstallModule;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderConsentMappingTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderCustomerConsentTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderCustomerMappingTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderCustomersTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderLogsTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderOrderMappingTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderOrdersTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderOrderStateMappingRelationTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderOrderStateMappingTableUninstallCommand;
use JustIdea\ExpertSender\Infrastructure\Database\Uninstall\Command\ExpertSenderProductMappingTableUninstallCommand;

if (!defined('_PS_VERSION_')) {
    exit;
}

class DatabaseTableUninstaller implements UninstallerInterface
{
    /**
     * @var ExpertSenderLogsTableUninstallCommand
     */
    private $expertSenderLogsTableUninstallCommand;

    /**
     * @var ExpertSenderOrdersTableUninstallCommand
     */
    private $expertSenderOrdersTableUninstallCommand;

    /**
     * @var ExpertSenderCustomersTableUninstallCommand
     */
    private $expertSenderCustomersTableUninstallCommand;

    /**
     * @var ExpertSenderCustomerConsentTableUninstallCommand
     */
    private $expertSenderCustomerConsentTableUninstallCommand;

    /**
     * @var ExpertSenderConsentMappingTableUninstallCommand
     */
    private $expertSenderConsentMappingTableUninstallCommand;

    /**
     * @var ExpertSenderCustomerMappingTableUninstallCommand
     */
    private $expertSenderCustomerMappingTableUninstallCommand;

    /**
     * @var ExpertSenderOrderMappingTableUninstallCommand
     */
    private $expertSenderOrderMappingTableUninstallCommand;

    /**
     * @var ExpertSenderOrderStateMappingTableUninstallCommand
     */
    private $expertSenderOrderStateMappingTableUninstallCommand;

    /**
     * @var ExpertSenderOrderStateMappingRelationTableUninstallCommand
     */
    private $expertSenderOrderStateMappingRelationTableUninstallCommand;

    /**
     * @var ExpertSenderProductMappingTableUninstallCommand
     */
    private $expertSenderProductMappingTableUninstallCommand;

    public function __construct(
        ExpertSenderLogsTableUninstallCommand $expertSenderLogsTableUninstallCommand,
        ExpertSenderOrdersTableUninstallCommand $expertSenderOrdersTableUninstallCommand,
        ExpertSenderCustomersTableUninstallCommand $expertSenderCustomersTableUninstallCommand,
        ExpertSenderCustomerConsentTableUninstallCommand $expertSenderCustomerConsentTableUninstallCommand,
        ExpertSenderConsentMappingTableUninstallCommand $expertSenderConsentMappingTableUninstallCommand,
        ExpertSenderCustomerMappingTableUninstallCommand $expertSenderCustomerMappingTableUninstallCommand,
        ExpertSenderOrderMappingTableUninstallCommand $expertSenderOrderMappingTableUninstallCommand,
        ExpertSenderOrderStateMappingTableUninstallCommand $expertSenderOrderStateMappingTableUninstallCommand,
        ExpertSenderOrderStateMappingRelationTableUninstallCommand $expertSenderOrderStateMappingRelationTableUninstallCommand,
        ExpertSenderProductMappingTableUninstallCommand $expertSenderProductMappingTableUninstallCommand
    ) {
        $this->expertSenderLogsTableUninstallCommand = $expertSenderLogsTableUninstallCommand;
        $this->expertSenderOrdersTableUninstallCommand = $expertSenderOrdersTableUninstallCommand;
        $this->expertSenderCustomersTableUninstallCommand = $expertSenderCustomersTableUninstallCommand;
        $this->expertSenderCustomerConsentTableUninstallCommand = $expertSenderCustomerConsentTableUninstallCommand;
        $this->expertSenderConsentMappingTableUninstallCommand = $expertSenderConsentMappingTableUninstallCommand;
        $this->expertSenderCustomerMappingTableUninstallCommand = $expertSenderCustomerMappingTableUninstallCommand;
        $this->expertSenderOrderMappingTableUninstallCommand = $expertSenderOrderMappingTableUninstallCommand;
        $this->expertSenderOrderStateMappingTableUninstallCommand = $expertSenderOrderStateMappingTableUninstallCommand;
        $this->expertSenderOrderStateMappingRelationTableUninstallCommand = $expertSenderOrderStateMappingRelationTableUninstallCommand;
        $this->expertSenderProductMappingTableUninstallCommand = $expertSenderProductMappingTableUninstallCommand;
    }

    /** {@inheritDoc} */
    public function init(): void
    {
        $commands = $this->getCommands();

        foreach ($commands as $command) {
            if (false == \Db::getInstance()->execute($command->getCommand())) {
                throw CouldNotUninstallModule::failedToUninstallDatabaseTable($command->getName());
            }
        }
    }

    /**
     * @return UninstallCommandInterface[]
     */
    private function getCommands(): array
    {
        return [
            $this->expertSenderLogsTableUninstallCommand,
            $this->expertSenderOrdersTableUninstallCommand,
            $this->expertSenderCustomersTableUninstallCommand,
            $this->expertSenderCustomerConsentTableUninstallCommand,
            $this->expertSenderConsentMappingTableUninstallCommand,
            $this->expertSenderCustomerMappingTableUninstallCommand,
            $this->expertSenderOrderMappingTableUninstallCommand,
            $this->expertSenderOrderStateMappingRelationTableUninstallCommand,
            $this->expertSenderOrderStateMappingTableUninstallCommand,
            $this->expertSenderProductMappingTableUninstallCommand,
        ];
    }
}
