<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Exception;

use JustIdea\ExpertSender\Infrastructure\Enum\ExceptionCode;
use JustIdea\ExpertSender\Infrastructure\Utility\ExceptionUtility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ExpertSenderException extends \Exception
{
    private $context;

    public function __construct(
        string $internalMessage,
        int $code,
        ?\Throwable $previous = null,
        array $context = []
    ) {
        parent::__construct($internalMessage, $code, $previous);
        $this->context = $context;
    }

    public function getContext(): array
    {
        return $this->context;
    }

    public function getExceptions(): array
    {
        $exceptions = [];
        $exception = $this;
        while ($exception) {
            $exceptions[] = ExceptionUtility::toArray($exception);

            $exception = $exception->getPrevious();
        }

        return $exceptions;
    }

    public static function failedToGetSuccessfulApiResponse(?\Throwable $exception = null, $context = []): self
    {
        return new static(
            'Failed to get successful api response. Please check system logs or contact Expert Sender support.',
            ExceptionCode::API_FAILED_TO_GET_SUCCESSFUL_RESPONSE,
            $exception,
            $context
        );
    }

    public static function failedDueToBadRequest(?\Throwable $exception = null, $context = []): self
    {
        return new static(
            'A Bad Request error occurred. Please check system logs or contact Expert Sender support.',
            ExceptionCode::API_FAILED_TO_PROCESS_REQUEST,
            $exception,
            $context
        );
    }

    public static function failedDueToUnauthorized(?\Throwable $exception = null, $context = []): self
    {
        return new static(
            'Unauthorized access detected. Please verify your authentication token. In case of problem contact Expert Sender support.',
            ExceptionCode::API_FAILED_TO_AUTHORIZE,
            $exception,
            $context
        );
    }

    public static function invalidArgument(string $given, string $expected = ''): self
    {
        if ($expected) {
            return new static(sprintf(
                'Failed due to invalid argument. Item must be an instance of %s and was given %s.',
                $expected,
                $given
            ),
                ExceptionCode::INFRASTRUCTURE_FAILED_DUE_TO_INVALID_ARGUMENT
            );
        } else {
            return new static(
                sprintf('Failed due to invalid argument. Given argument (%s) is invalid', $given),
                ExceptionCode::INFRASTRUCTURE_FAILED_DUE_TO_INVALID_ARGUMENT
            );
        }
    }

    public static function notImplemented(?\Throwable $exception = null, $context = []): self
    {
        return new static(
            'This method or handle has not been implemented. Please contact Expert Sender support.',
            ExceptionCode::NOT_IMPLEMENTED,
            $exception,
            $context
        );
    }

    public static function mappingNotFound(?\Throwable $exception = null, $context = []): self
    {
        return new static(
            'Incorrect ECDP association ID specified.',
            ExceptionCode::MAPPING_NOT_FOUND,
            $exception,
            $context
        );
    }

    public static function fileCouldNotBeDeleted(?\Throwable $exception = null, $context = []): self
    {
        return new static(
            'Failed to remove a file.',
            ExceptionCode::FAILED_TO_REMOVE_FILE,
            $exception,
            $context
        );
    }

    public static function unknownError(\Throwable $exception, $context = []): self
    {
        return new static(
            'An unknown error occurred. Please check system logs or contact Expert Sender support.',
            ExceptionCode::UNKNOWN_ERROR,
            $exception,
            $context
        );
    }
}
