<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Logger;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Adapter\Configuration;
use JustIdea\ExpertSender\Infrastructure\Exception\Handler\ExceptionHandlerInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\Writer\ExtendedContextLogWriter;
use JustIdea\ExpertSender\Infrastructure\Logger\Writer\PrestaShopLogWriterInterface;

if (!defined('_PS_VERSION_')) {
    exit;
}

class Logger implements LoggerInterface
{
    public const FILE_NAME = 'Logger';
    public const LOG_OBJECT_TYPE = 'expertsenderlog';

    public const SEVERITY_INFO = 1;
    public const SEVERITY_WARNING = 2;
    public const SEVERITY_ERROR = 3;
    public const SEVERITY_CRITICAL = 4;

    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @var ExceptionHandlerInterface
     */
    private $exceptionHandler;

    /**
     * @var ExtendedContextLogWriter
     */
    private $extendedLogWriter;

    /**
     * @var PrestaShopLogWriterInterface
     */
    private $logWriter;

    /**
     * @var int
     */
    private $shopId;

    public function __construct(
        Configuration $configuration,
        ExceptionHandlerInterface $exceptionHandler,
        ExtendedContextLogWriter $extendedLogWriter,
        PrestaShopLogWriterInterface $logWriter,
        int $shopId
    ) {
        $this->configuration = $configuration;
        $this->exceptionHandler = $exceptionHandler;
        $this->extendedLogWriter = $extendedLogWriter;
        $this->logWriter = $logWriter;
        $this->shopId = $shopId;
    }

    /**
     * System is unusable.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function emergency($message, array $context = [])
    {
        $this->log(
            $this->configuration->getAsInteger(
                'PS_LOGS_BY_EMAIL',
                $this->shopId
            ),
            $message,
            $context
        );
    }

    /**
     * Action must be taken immediately.
     *
     * Example: Entire website down, database unavailable, etc. This should
     * trigger the SMS alerts and wake you up.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function alert($message, array $context = [])
    {
        $this->log(self::SEVERITY_WARNING, $message, $context);
    }

    /**
     * Critical conditions.
     *
     * Example: Application component unavailable, unexpected exception.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function critical($message, array $context = [])
    {
        $this->log(
            $this->configuration->getAsInteger(
                'PS_LOGS_BY_EMAIL',
                $this->shopId
            ),
            $message,
            $context
        );
    }

    /**
     * Runtime errors that do not require immediate action but should typically
     * be logged and monitored.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function error($message, array $context = [])
    {
        $this->log(self::SEVERITY_ERROR, $message, $context);
    }

    /**
     * Exceptional occurrences that are not errors.
     *
     * Example: Use of deprecated APIs, poor use of an API, undesirable things
     * that are not necessarily wrong.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function warning($message, array $context = [])
    {
        $this->log(self::SEVERITY_WARNING, $message, $context);
    }

    /**
     * Normal but significant events.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function notice($message, array $context = [])
    {
        $this->log(self::SEVERITY_INFO, $message, $context);
    }

    /**
     * Interesting events.
     *
     * Example: User logs in, SQL logs.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function info($message, array $context = [])
    {
        $this->log(self::SEVERITY_INFO, $message, $context);
    }

    /**
     * Detailed debug information.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function debug($message, array $context = [])
    {
        if (!$this->configuration->get(Config::LOGS_DEBUG_MODE)) {
            return;
        }

        $this->log(self::SEVERITY_INFO, $message, $context);
    }

    /**
     * Logs with an arbitrary level.
     *
     * @param int $level
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     *
     * @throws \Psr\Log\InvalidArgumentException
     */
    public function log($level, $message, array $context = [])
    {
        try {
            $logId = $this->logWriter->addLog($level, $message);

            if (!$logId) {
                return;
            }
            $this->extendedLogWriter->logContext((int) $logId, $context);
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }
    }

    /**
     * Handles exceptions by delegating to the exception handler.
     *
     * @param \Throwable $exception the exception to handle
     *
     * @return void
     */
    public function sendToExceptionHandler(\Throwable $exception): void
    {
        $this->exceptionHandler->handle($exception);
    }
}
