<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Logger\Reader;

use JustIdea\ExpertSender\Infrastructure\Logger\Formatter\ReportLogFormatterInterface;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ReportLogReader implements ReportLogReaderInterface
{
    /**
     * @var ReportLogFormatterInterface
     */
    private $logFormatter;

    public function __construct(
        ReportLogFormatterInterface $logFormatter
    ) {
        $this->logFormatter = $logFormatter;
    }

    public const LOG_OBJECT_TYPE = 'expertsenderlog';

    /**
     * Read the file content as a string.
     *
     * @param string $filename the path to the log file
     * @param string $title title to prepend to the content
     * @param bool $format whether to format the content
     *
     * @return string the formatted file content as a string
     */
    public function readFileAsString(string $filename, string $title, bool $format): string
    {
        $content = $this->readFileAsArray($filename, $title, $format);

        return !empty($content) ? implode("\n", $content) : '';
    }

    /**
     * Read the file content as an array.
     *
     * @param string $filename the path to the log file
     * @param string $title title to prepend to the content
     * @param bool $format whether to format the content
     *
     * @return array<string> the file content as an array of strings
     */
    public function readFileAsArray(string $filename, string $title, bool $format): array
    {
        if (!file_exists($filename)) {
            return [];
        }

        $content = file($filename, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

        if (empty($content)) {
            return [];
        }

        foreach ($content as &$value) {
            $value = $format ? $this->logFormatter->readMessage($value) : $value;
        }

        if ($title) {
            array_unshift($content, $format ? "<h4>{$title}</h4>" : $title);
        }

        return $content;
    }
}
