<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Logger;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Adapter\Configuration;
use JustIdea\ExpertSender\Infrastructure\Exception\Handler\ReportExceptionHandlerInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\Export\FileWritingException;
use JustIdea\ExpertSender\Infrastructure\Logger\Reader\ReportLogReaderInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\Writer\ReportLogWriterInterface;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ReportLogger implements ReportLoggerInterface
{
    public const SEVERITY_SUCCESS = 0;
    public const SEVERITY_INFO = 1;
    public const SEVERITY_WARNING = 2;
    public const SEVERITY_ERROR = 3;
    public const SEVERITY_CRITICAL = 4;

    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @var ReportExceptionHandlerInterface
     */
    private $exceptionHandler;

    /**
     * @var ReportLogWriterInterface
     */
    private $logWriter;

    /**
     * @var ReportLogReaderInterface
     */
    private $logReader;

    /**
     * @var int
     */
    private $shopId;

    /**
     * @var string
     */
    private $sessionLogsFilename;

    /**
     * @var string
     */
    private $dailyLogsFilename;

    /**
     * @var string
     */
    private $logFolder;

    public function __construct(
        Configuration $configuration,
        ReportExceptionHandlerInterface $exceptionHandler,
        ReportLogWriterInterface $logWriter,
        ReportLogReaderInterface $logReader,
        int $shopId
    ) {
        $this->configuration = $configuration;
        $this->exceptionHandler = $exceptionHandler;
        $this->logWriter = $logWriter;
        $this->logReader = $logReader;
        $this->shopId = $shopId;

        $this->logFolder = _PS_MODULE_DIR_ . 'expertsenderofficial/logs/';
        $this->initializeFilename();
    }

    public function __destruct()
    {
        $this->archiveSessionLogs();
    }

    /**
     * Initializes filenames for logging.
     *
     * @return void
     */
    private function initializeFilename(): void
    {
        $sessionLogsFilename = date('Y-m-d_His') . '_es.log';
        $logsFilename = 'logs_es.log';

        $this->setSessionLogsFilename($sessionLogsFilename);
        $this->setLogsFilename($logsFilename);
    }

    /**
     * System is unusable.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function emergency($message, array $context = [])
    {
        $this->log(
            $this->configuration->getAsInteger(
                'PS_LOGS_BY_EMAIL',
                $this->shopId
            ),
            $message,
            $context
        );
    }

    /**
     * Action must be taken immediately.
     *
     * Example: Entire website down, database unavailable, etc. This should
     * trigger the SMS alerts and wake you up.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function alert($message, array $context = [])
    {
        $this->log(self::SEVERITY_WARNING, $message, $context);
    }

    /**
     * Critical conditions.
     *
     * Example: Application component unavailable, unexpected exception.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function critical($message, array $context = [])
    {
        $this->log(
            $this->configuration->getAsInteger(
                'PS_LOGS_BY_EMAIL',
                $this->shopId
            ),
            $message,
            $context
        );
    }

    /**
     * Runtime errors that do not require immediate action but should typically
     * be logged and monitored.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function error($message, array $context = [])
    {
        $this->log(self::SEVERITY_ERROR, $message, $context);
    }

    /**
     * Exceptional occurrences that are not errors.
     *
     * Example: Use of deprecated APIs, poor use of an API, undesirable things
     * that are not necessarily wrong.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function warning($message, array $context = [])
    {
        $this->log(self::SEVERITY_WARNING, $message, $context);
    }

    /**
     * Normal but significant events.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function notice($message, array $context = [])
    {
        $this->log(self::SEVERITY_INFO, $message, $context);
    }

    /**
     * Log a success event.
     *
     * @param string $message the message to log
     * @param mixed[] $context additional context for the log
     *
     * @return void
     */
    public function success($message, array $context = [])
    {
        $this->log(self::SEVERITY_SUCCESS, $message, $context);
    }

    /**
     * Interesting events.
     *
     * Example: User logs in, SQL logs.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function info($message, array $context = [])
    {
        $this->log(self::SEVERITY_INFO, $message, $context);
    }

    /**
     * Detailed debug information.
     *
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     */
    public function debug($message, array $context = [])
    {
        if (!$this->configuration->get(Config::LOGS_DEBUG_MODE)) {
            return;
        }

        $this->log(self::SEVERITY_INFO, $message, $context);
    }

    /**
     * Logs with an arbitrary level.
     *
     * @param int $level
     * @param string $message
     * @param mixed[] $context
     *
     * @return void
     *
     * @throws \Psr\Log\InvalidArgumentException
     */
    public function log($level, $message, array $context = [])
    {
        try {
            $this->logWriter->addLog($this->getSessionLogsFilename(), $level, $message);
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }
    }

    /**
     * Handles exceptions by delegating to the exception handler.
     *
     * @param \Throwable $exception the exception to handle
     *
     * @return void
     */
    public function sendToExceptionHandler(\Throwable $exception): void
    {
        $this->exceptionHandler->handle($exception);
    }

    /**
     * Sets the filename for session logs.
     *
     * @param string $filename the filename to set
     *
     * @return void
     */
    private function setSessionLogsFilename($filename): void
    {
        $sessionLogsFilename = $this->logFolder . $filename;

        if (is_writable(dirname($sessionLogsFilename))) {
            $this->sessionLogsFilename = $sessionLogsFilename;
        } else {
            $this->sendToExceptionHandler(new FileWritingException(sprintf('Driectory (%s) is not writable.', dirname($sessionLogsFilename)), FileWritingException::CANNOT_OPEN_FILE_FOR_WRITING));
        }
    }

    /**
     * Sets the filename for daily logs.
     *
     * @param string $filename the filename to set
     *
     * @return void
     */
    private function setLogsFilename($filename): void
    {
        $dailyLogsFilename = $this->logFolder . $filename;

        if (is_writable(dirname($dailyLogsFilename))) {
            $this->dailyLogsFilename = $dailyLogsFilename;
        } else {
            $this->sendToExceptionHandler(new FileWritingException(sprintf('Driectory (%s) is not writable.', dirname($dailyLogsFilename)), FileWritingException::CANNOT_OPEN_FILE_FOR_WRITING));
        }
    }

    /**
     * Log the message.
     *
     * @return string $sessionLogsFilename
     */
    private function getSessionLogsFilename(): string
    {
        if (empty($this->sessionLogsFilename)) {
            throw new FileWritingException('Session Logs Filename is empty', FileWritingException::CANNOT_OPEN_FILE_FOR_WRITING);
        }

        return $this->sessionLogsFilename;
    }

    /**
     * Log the message.
     *
     * @return string $dailyLogsFilename
     */
    private function getLogsFilename(): string
    {
        if (empty($this->dailyLogsFilename)) {
            throw new FileWritingException('Logs Filename is empty', FileWritingException::CANNOT_OPEN_FILE_FOR_WRITING);
        }

        return $this->dailyLogsFilename;
    }

    /**
     * Retrieve session log file contents as an array.
     *
     * @param string $title optional title to filter logs
     * @param bool $format whether to format the output
     *
     * @return array<string> the contents of the log file as an array
     */
    public function getSessionLogFileContentsAsArray(string $title = '', bool $format = true): array
    {
        try {
            $content = $this->logReader->readFileAsArray($this->getSessionLogsFilename(), $title, $format);

            return $content;
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return [];
    }

    /**
     * Retrieve session log file contents as a string.
     *
     * @param string $title optional title to filter logs
     * @param bool $format whether to format the output
     *
     * @return string the contents of the log file as a string
     */
    public function getSessionLogFileContentsAsString(string $title = '', bool $format = true): string
    {
        try {
            $content = $this->logReader->readFileAsString($this->getSessionLogsFilename(), $title, $format);

            return $content;
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return '';
    }

    /**
     * Retrieve log file contents as an array.
     *
     * @param string $title optional title to filter logs
     * @param bool $format whether to format the output
     *
     * @return array<string> the contents of the log file as an array
     */
    public function getLogFileContentsAsArray(string $title = '', bool $format = true): array
    {
        try {
            $content = $this->logReader->readFileAsArray($this->getLogsFilename(), $title, $format);

            return $content;
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return [];
    }

    /**
     * Retrieve log file contents as a string.
     *
     * @param string $title optional title to filter logs
     * @param bool $format whether to format the output
     *
     * @return string the contents of the log file as a string
     */
    public function getLogFileContentsAsString(string $title = '', bool $format = true): string
    {
        try {
            $content = $this->logReader->readFileAsString($this->getLogsFilename(), $title, $format);

            return $content;
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return '';
    }

    /**
     * Archive session logs to a daily log file.
     *
     * @return bool true if the operation was successful, false otherwise
     */
    public function archiveSessionLogs(): bool
    {
        try {
            return $this->logWriter->archiveSessionLogsToDailyLogs($this->getSessionLogsFilename(), $this->getLogsFileName());
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return false;
    }

    /**
     * Append a line break to the session log file.
     *
     * @return bool true if successful, false otherwise
     */
    public function appendSessionLogFileWithLineBreak(): bool
    {
        try {
            return $this->logWriter->addLineBreak($this->getSessionLogsFilename());
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return false;
    }

    /**
     * Append a line break to the log file.
     *
     * @return bool true if successful, false otherwise
     */
    public function appendLogFileWithLineBreak(): bool
    {
        try {
            return $this->logWriter->addLineBreak($this->getLogsFilename());
        } catch (\Throwable $exception) {
            $this->sendToExceptionHandler($exception);
        }

        return false;
    }
}
