<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Logger\Writer;

use JustIdea\ExpertSender\Infrastructure\Adapter\Context;
use JustIdea\ExpertSender\Infrastructure\Context\GlobalShopContextInterface;
use JustIdea\ExpertSender\Infrastructure\EntityManager\EntityManagerInterface;
use JustIdea\ExpertSender\Infrastructure\EntityManager\ObjectModelUnitOfWork;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ExtendedContextLogWriter
{
    /**
     * @var EntityManagerInterface The custom entity manager
     */
    private $entityManager;

    /**
     * @var GlobalShopContextInterface The module global shop context class
     */
    private $globalShopContext;

    /**
     * @var Context The module context class
     */
    private $context;

    /**
     * Constructs a new ExtendedContextLogWriter instance.
     *
     * @param EntityManagerInterface $entityManager The entity manager
     * @param GlobalShopContextInterface $globalShopContext The global shop context
     * @param Context $context The context
     */
    public function __construct(
        EntityManagerInterface $entityManager,
        GlobalShopContextInterface $globalShopContext,
        Context $context
    ) {
        $this->entityManager = $entityManager;
        $this->globalShopContext = $globalShopContext;
        $this->context = $context;
    }

    /**
     * Logs the context information.
     *
     * @param int $logId The ID of the log
     * @param array $context The context information
     */
    public function logContext(int $logId, array $context): void
    {
        $request = '';
        $response = '';

        if (isset($context['request'])) {
            $request = $context['request'];
            unset($context['request']);
        }

        if (isset($context['response'])) {
            $response = $context['response'];
            unset($context['response']);
        }

        $expertSenderLog = new \ExpertSenderLog();
        $expertSenderLog->id_log = $logId;
        $expertSenderLog->id_shop = $this->globalShopContext->getShopId();
        $expertSenderLog->context = json_encode($this->getFilledContextWithShopData($context));
        $expertSenderLog->request = json_encode($request);
        $expertSenderLog->response = json_encode($response);

        $this->entityManager->persist($expertSenderLog, ObjectModelUnitOfWork::UNIT_OF_WORK_SAVE);
        $this->entityManager->flush();
    }

    /**
     * Fills the context with shop data.
     *
     * @param array $context The context information
     *
     * @return array The filled context
     */
    private function getFilledContextWithShopData(array $context = []): array
    {
        $context['context_id_customer'] = $this->context->getCustomerId();
        $context['id_shop'] = $this->globalShopContext->getShopId();
        $context['currency'] = $this->globalShopContext->getCurrencyIso();
        $context['id_language'] = $this->globalShopContext->getLanguageId();

        return $context;
    }
}
