<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Logger\Writer;

use JustIdea\ExpertSender\Infrastructure\Logger\Formatter\LogFormatterInterface;
use JustIdea\ExpertSender\Infrastructure\Logger\Repository\PrestashopLoggerRepositoryInterface;
use JustIdea\ExpertSender\Infrastructure\Provider\IdempotencyProviderInterface;

if (!defined('_PS_VERSION_')) {
    exit;
}

class PrestaShopLogWriter implements PrestaShopLogWriterInterface
{
    public const LOG_OBJECT_TYPE = 'expertsenderlog';

    /**
     * @var LogFormatterInterface The formatter for log messages
     */
    private $logFormatter;

    /**
     * @var IdempotencyProviderInterface The idempotency provider for generating keys
     */
    private $idempotencyProvider;

    /**
     * @var PrestashopLoggerRepositoryInterface The repository for PrestaShop logger
     */
    private $prestashopLoggerRepository;

    /**
     * @var int The ID of the shop
     */
    private $shopId;

    /**
     * Constructs a new PrestaShopLogWriter instance.
     *
     * @param LogFormatterInterface $logFormatter The formatter for log messages
     * @param IdempotencyProviderInterface $idempotencyProvider The idempotency provider for generating keys
     * @param PrestashopLoggerRepositoryInterface $prestashopLoggerRepository The repository for PrestaShop logger
     * @param int $shopId The ID of the shop
     */
    public function __construct(
        LogFormatterInterface $logFormatter,
        IdempotencyProviderInterface $idempotencyProvider,
        PrestashopLoggerRepositoryInterface $prestashopLoggerRepository,
        int $shopId
    ) {
        $this->logFormatter = $logFormatter;
        $this->idempotencyProvider = $idempotencyProvider;
        $this->prestashopLoggerRepository = $prestashopLoggerRepository;
        $this->shopId = $shopId;
    }

    /**
     * Adds a log entry to the PrestaShop logger.
     *
     * @param int $level The level of the log entry
     * @param string $message The message to be logged
     *
     * @return int|null The ID of the log entry, or null if not found
     */
    public function addLog(int $level, string $message): ?int
    {
        $idempotencyKey = (int) $this->idempotencyProvider->getIdempotencyKey();

        \PrestaShopLogger::addLog(
            $this->logFormatter->getMessage($message),
            $level,
            null,
            self::LOG_OBJECT_TYPE,
            $idempotencyKey
        );

        $logId = $this->prestashopLoggerRepository->getLogIdByObjectId(
            $idempotencyKey,
            $this->shopId
        );

        return $logId;
    }
}
