<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Repository;

use JustIdea\ExpertSender\Core\Config\Config;
use JustIdea\ExpertSender\Infrastructure\Adapter\Validate;

if (!defined('_PS_VERSION_')) {
    exit;
}

class CollectionRepository implements ReadOnlyCollectionRepositoryInterface
{
    /**
     * @var int
     */
    protected $batchSize;

    /**
     * @var string
     */
    private $fullyClassifiedClassName;

    public function __construct(string $fullyClassifiedClassName)
    {
        $this->fullyClassifiedClassName = $fullyClassifiedClassName;

        $batchSize = (int) \Configuration::get(Config::SYNC_BATCH_SIZE);

        $this->batchSize = Validate::isValidBatchSize($batchSize) ? $batchSize : 20;
    }

    /**
     * Retrieves all objects in the collection.
     *
     * @param int|null $langId The language ID (optional)
     *
     * @return \PrestaShopCollection The collection of objects
     *
     * @throws \PrestaShopException
     */
    public function findAllInCollection($langId = null): \PrestaShopCollection
    {
        return new \PrestaShopCollection($this->fullyClassifiedClassName, $langId);
    }

    /**
     * Finds a single object by the specified criteria.
     *
     * @param array<string, mixed> $keyValueCriteria The criteria to search by (e.g., ['id_log' => 1])
     * @param int|null $langId The language ID (optional)
     *
     * @return \ObjectModel|null The found object, or null if not found
     *
     * @throws \PrestaShopException
     */
    public function findOneBy(array $keyValueCriteria, $langId = null): ?\ObjectModel
    {
        $psCollection = new \PrestaShopCollection($this->fullyClassifiedClassName, $langId);

        foreach ($keyValueCriteria as $field => $value) {
            $psCollection = $psCollection->where($field, '=', $value);
        }

        $first = $psCollection->getFirst();

        return false === $first ? null : $first;
    }

    /**
     * Finds objects by the specified criteria.
     *
     * @param array<string, mixed> $keyValueCriteria The criteria to search by (e.g., ['id_log' => 1])
     * @param int|null $langId The language ID (optional)
     *
     * @return \PrestaShopCollection The collection of found objects
     *
     * @throws \PrestaShopException
     */
    public function findBy(array $keyValueCriteria, $langId = null): \PrestaShopCollection
    {
        $psCollection = new \PrestaShopCollection($this->fullyClassifiedClassName, $langId);

        foreach ($keyValueCriteria as $field => $value) {
            $psCollection = $psCollection->where($field, '=', $value);
        }

        return $psCollection;
    }
}
