<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Translator;

use JustIdea\ExpertSender\Infrastructure\Adapter\ModuleFactory;
use JustIdea\ExpertSender\Infrastructure\Database\Exception\CouldNotInstallModule;
use JustIdea\ExpertSender\Infrastructure\Database\Exception\CouldNotUninstallModule;
use JustIdea\ExpertSender\Infrastructure\Enum\ExceptionCode;
use JustIdea\ExpertSender\Infrastructure\Exception\ExpertSenderException;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ExceptionLegacyTranslator implements ExceptionTranslatorInterface
{
    /**
     * File name for the exception translator.
     */
    public const FILE_NAME = 'ExceptionLegacyTranslator';

    /**
     * The ExpertSender module to which the exception translator applies.
     *
     * @var \Module&\ExpertSenderOfficial
     */
    private $module;

    /**
     * Constructor of the ExceptionLegacyTranslator class.
     *
     * @param ModuleFactory $moduleFactory Module factory
     */
    public function __construct(ModuleFactory $moduleFactory)
    {
        $this->module = $moduleFactory->getModule();
    }

    /**
     * Translates an ExpertSenderException into a user-friendly message.
     *
     * @param ExpertSenderException $exception The exception to translate
     *
     * @return string Translated message
     */
    public function translate(ExpertSenderException $exception): string
    {
        $customMessages = $this->getCustomMessages();
        $genericMessages = $this->getGenericMessages();

        $exceptionType = get_class($exception);
        $exceptionCode = $exception->getCode();

        if (isset($customMessages[$exceptionType], $customMessages[$exceptionType][$exceptionCode])) {
            return $this->getFormattedMessage($customMessages[$exceptionType][$exceptionCode], $exception->getContext());
        }

        if (isset($genericMessages[$exceptionCode])) {
            return $this->getFormattedMessage($genericMessages[$exceptionCode], $exception->getContext());
        }

        return $this->getFormattedMessage($exception->getMessage(), $exception->getContext());
    }

    /**
     * Translation collection of custom messages.
     *
     * @return array<string, array<int, string>> Array with translated messages
     */
    private function getCustomMessages(): array
    {
        return [
            CouldNotInstallModule::class => [
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_INSTALL_DATABASE_TABLE => $this->module->l('Failed to install database table (%s).', self::FILE_NAME),
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_INSTALL_MODULE_TAB => $this->module->l('Failed to install module tab (%s).', self::FILE_NAME),
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_INSTALL_HOOK => $this->module->l('Failed to install hook (%s).', self::FILE_NAME),
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_INSTALL_CONFIGURATION => $this->module->l('Failed to install configuration ("%1$s" => "%2$s").', self::FILE_NAME),
            ],
            CouldNotUninstallModule::class => [
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_UNINSTALL_DATABASE_TABLE => $this->module->l('Failed to uninstall database table (%s).', self::FILE_NAME),
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_UNINSTALL_MODULE_TAB => $this->module->l('Failed to uninstall module tab (%s).', self::FILE_NAME),
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_UNINSTALL_HOOK => $this->module->l('Failed to uninstall hook (%s).', self::FILE_NAME),
                ExceptionCode::INFRASTRUCTURE_FAILED_TO_UNINSTALL_CONFIGURATION => $this->module->l('Failed to uninstall configuration (%s).', self::FILE_NAME),
            ],
        ];
    }

    /**
     * Translation collection of generic messages.
     *
     * @return array<int, string> Array with translated messages
     */
    private function getGenericMessages(): array
    {
        return [
            ExceptionCode::MISSING_WEBSITE_ID => $this->module->l('The site ID has not been specified or is incorrect. Go to the Settings tab and enter the site ID.', self::FILE_NAME),
            ExceptionCode::MAPPING_NOT_FOUND => $this->module->l('Incorrect ECDP association ID specified.', self::FILE_NAME),
            ExceptionCode::UNKNOWN_ERROR => $this->module->l('An unknown error error occurred. Please check system logs or contact Expert Sender support.', self::FILE_NAME),
            ExceptionCode::MISSING_API_KEY => $this->module->l('Api token is missing for api client.', self::FILE_NAME),
            ExceptionCode::MISSING_API_TYPE => $this->module->l('The API type (%s) is invalid.', self::FILE_NAME),
        ];
    }

    /**
     * Formats a message using vsprintf if placeholders are found in the message.
     *
     * If the message contains placeholders (%), it is formatted using vsprintf
     * with the provided context array. Otherwise, the original message is returned.
     *
     * @param string $message The message to format
     * @param array $context The context to use for formatting
     *
     * @return string Formatted message
     */
    private function getFormattedMessage(string $message, array $context): string
    {
        if (false !== strpos($message, '%')) {
            return vsprintf($message, $context);
        }

        return $message;
    }
}
