<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Utility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ExceptionUtility
{
    /**
     * getExceptions returns exceptions if available, otherwise converts the throwable to an array
     *
     * @param \Throwable $exception
     *
     * @return array<mixed, mixed>
     */
    public static function getExceptions(\Throwable $exception): array
    {
        if (method_exists($exception, 'getExceptions')) {
            return $exception->getExceptions();
        }

        return [self::toArray($exception)];
    }

    /**
     * getLastRequest retrieves the last request associated with the throwable, if available
     *
     * @param \Throwable $exception
     *
     * @return array<mixed, mixed>|null
     */
    public static function getLastRequest(\Throwable $exception): ?array
    {
        while ($exception) {
            $context = method_exists($exception, 'getContext') ? $exception->getContext() : [];
            if (isset($context['request'])) {
                return (array) $context['request'];
            }
            $exception = $exception->getPrevious();
        }

        return null;
    }

    /**
     * getLastResponse retrieves the last response associated with the throwable, if available
     *
     * @param \Throwable $exception
     *
     * @return array<mixed, mixed>|null
     */
    public static function getLastResponse(\Throwable $exception): ?array
    {
        while ($exception) {
            $context = method_exists($exception, 'getContext') ? $exception->getContext() : [];
            if (isset($context['response'])) {
                return $context['response'];
            }
            $exception = $exception->getPrevious();
        }

        return null;
    }

    /**
     * toArray converts the throwable to an array
     *
     * @param \Throwable $exception
     *
     * @return array<string, mixed>
     */
    public static function toArray(\Throwable $exception): array
    {
        $context = method_exists($exception, 'getContext') ? $exception->getContext() : [];

        $exceptionArray = [
            'message' => (string) $exception->getMessage(),
            'code' => (int) $exception->getCode(),
            'file' => (string) $exception->getFile(),
            'line' => (int) $exception->getLine(),
            'context' => $context,
        ];

        return $exceptionArray;
    }
}
