<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Utility;

if (!defined('_PS_VERSION_')) {
    exit;
}

class TimeZoneUtility
{
    public const SYNC_DATE_TIME_FORMAT = 'Y-m-d H:i:s';
    public const ISO_8601_DATETIME_FORMAT = 'Y-m-d\TH:i:s.v\Z';
    public const ISO_8601_DATE_FORMAT = 'Y-m-d';
    public const DEFAULT_TIME_ZONE = 'Europe/Warsaw';

    /**
     * getDateTimeFormatted returns the datetime formatted according to the selected format
     *
     * @param string $format datetime format
     *
     * @return string datetime formatted
     */
    public static function getDateTimeFormatted($format = self::SYNC_DATE_TIME_FORMAT): string
    {
        $timeZoneObj = new \DateTimeZone(\Configuration::get('PS_TIMEZONE', self::DEFAULT_TIME_ZONE));
        $today = new \DateTime('now', $timeZoneObj);

        return $today->format($format);
    }

    /**
     * getDateTime returns the DataTime object with current date and time
     *
     * @return \DateTimeInterface datetime object
     */
    public static function getDateTime(): \DateTimeInterface
    {
        $timeZoneObj = new \DateTimeZone(\Configuration::get('PS_TIMEZONE', self::DEFAULT_TIME_ZONE));

        return new \DateTime('now', $timeZoneObj);
    }

    /**
     * convertDateTimeToISO8601Format returns the datetime formatted according to the ISO8601 format
     *
     * @param string $date as string
     *
     * @return string|null datetime in ISO8601 format, null for empty dates
     */
    public static function convertDateTimeToISO8601Format(string $date): ?string
    {
        if (self::isEmptyDate($date)) {
            return null;
        }

        $timeZoneObj = new \DateTimeZone(\Configuration::get('PS_TIMEZONE', self::DEFAULT_TIME_ZONE));
        $dateTime = new \DateTime($date, $timeZoneObj);

        return $dateTime->format(self::ISO_8601_DATETIME_FORMAT);
    }

    /**
     * convertDateTimeToISO8601Format returns the date formatted according to the ISO8601 format
     *
     * @param string $date as string
     *
     * @return string|null date in ISO8601 format, null for empty dates
     */
    public static function convertDateToISO8601Format(string $date): ?string
    {
        if (self::isEmptyDate($date)) {
            return null;
        }

        $timeZoneObj = new \DateTimeZone(\Configuration::get('PS_TIMEZONE', self::DEFAULT_TIME_ZONE));
        $dateTime = new \DateTime($date, $timeZoneObj);

        return $dateTime->format(self::ISO_8601_DATE_FORMAT);
    }

    /**
     * isEmptyDate returns the result of check if the date is empty
     *
     * @param string|null $date
     *
     * @return bool result of the check
     */
    public static function isEmptyDate(?string $date): bool
    {
        return null === $date || '0000-00-00' === $date || '0000-00-00 00:00:00' === $date;
    }
}
