<?php
/**
 * Copyright since 2016 JustIdea Agency.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author    JustIdea Agency <contact@justidea.agency>
 * @copyright 2016-2024 JustIdea Agency
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

declare(strict_types=1);

namespace JustIdea\ExpertSender\Infrastructure\Utility;

use Context as PrestashopContext;
use Tools as PrestashopTools;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ToolsUtility
{
    public function hash(string $string): string
    {
        return PrestashopTools::hash($string);
    }

    public function linkRewrite(string $str): string
    {
        return PrestashopTools::str2url($str);
    }

    public function redirectAdmin(string $controller): void
    {
        PrestashopTools::redirectAdmin($controller);
    }

    public function redirect(string $url): void
    {
        PrestashopTools::redirect($url);
    }

    public function isSubmit(string $form): bool
    {
        return PrestashopTools::isSubmit($form);
    }

    public function strtoupper(string $string): string
    {
        return PrestashopTools::strtoupper($string);
    }

    public function strtolower(string $string): string
    {
        return PrestashopTools::strtolower($string);
    }

    /**
     * Random password generator.
     *
     * @param int $length Desired length (optional)
     * @param string $flag Output type (NUMERIC, ALPHANUMERIC, NO_NUMERIC, RANDOM)
     *
     * @return bool|string Password
     */
    public function passwdGen(int $length = 8, string $flag = 'ALPHANUMERIC')
    {
        return PrestashopTools::passwdGen($length, $flag);
    }

    /**
     * This method allows to get the content from either a URL or a local file.
     *
     * @param string $url the url to get the content from
     * @param bool $useIncludePath second parameter of http://php.net/manual/en/function.file-get-contents.php
     * @param resource $steamContext third parameter of http://php.net/manual/en/function.file-get-contents.php
     * @param int $curlTimeout
     * @param bool $fallback whether or not to use the fallback if the main solution fails
     *
     * @return bool|string false or the string content
     */
    public function fileGetContents(
        string $url,
        bool $useIncludePath = false,
        $steamContext = null,
        int $curlTimeout = 5,
        bool $fallback = false
    ) {
        return PrestashopTools::file_get_contents($url, $useIncludePath, $steamContext, $curlTimeout, $fallback);
    }

    /**
     * Replace all accented chars by their equivalent non accented chars.
     *
     * @param string $string
     *
     * @return string
     */
    public static function replaceAccentedChars(string $string): string
    {
        return PrestashopTools::replaceAccentedChars($string);
    }

    /**
     * Checks if a key exists either in $_POST or $_GET.
     *
     * @param string $key
     *
     * @return bool
     */
    public function getIsset($key): bool
    {
        $issetToolsKey = PrestashopTools::getIsset($key);

        return $issetToolsKey;
    }

    /**
     * @param string $value
     * @param string|false $defaultValue
     *
     * @return mixed Value
     */
    public function getValue($value, $defaultValue = false)
    {
        $toolsValue = PrestashopTools::getValue($value, $defaultValue);

        return is_null($toolsValue) || '' === $toolsValue || 'null' === $toolsValue ? null : $toolsValue;
    }

    /**
     * @param string $value
     * @param string|false $defaultValue
     *
     * @return bool
     */
    public function getValueAsBoolean($value, $defaultValue = false): bool
    {
        $result = $this->getValue($value, $defaultValue);

        if (in_array($result, ['false', '0', null, false, 0], true)) {
            return false;
        }

        return (bool) $result;
    }

    /**
     * @param string $value
     * @param string|false $defaultValue
     *
     * @return int
     */
    public function getValueAsInteger($value, $defaultValue = false): int
    {
        $result = $this->getValue($value, $defaultValue);

        if (in_array($result, ['false', '0', null, false, 0], true)) {
            return 0;
        }

        return (int) $result;
    }

    /**
     * Get all values from $_POST/$_GET.
     *
     * @return mixed
     */
    public function getAllValues()
    {
        return PrestashopTools::getAllValues();
    }

    /**
     * getShopDomain returns domain name according to configuration and ignoring ssl.
     *
     * @param bool $http if true, return domain name with protocol
     * @param bool $entities if true, convert special chars to HTML entities
     *
     * @return string domain
     */
    public function getShopDomain($http = false, $entities = false): string
    {
        return PrestashopTools::getShopDomain($http, $entities);
    }

    /**
     * Return price with currency sign for a given product.
     *
     * @param float $price Product price
     * @param object|array|null $currency Current currency (object, id_currency, NULL => context currency)
     * @param bool $no_utf8
     * @param PrestashopContext|null $context
     *
     * @return string Price correctly formated (sign, decimal separator...)
     *                if you modify this function, don't forget to modify the Javascript function formatCurrency (in tools.js)
     */
    public function displayPrice(float $price, $currency = null, $no_utf8 = false, ?PrestashopContext $context = null): string
    {
        return PrestashopTools::displayPrice($price, $currency, $no_utf8, $context);
    }

    /**
     * returns the rounded value of $value to specified precision, according to your configuration;.
     *
     * @note : PHP 5.3.0 introduce a 3rd parameter mode in round function
     *
     * @param float $value
     * @param int $precision
     * @param int|null $round_mode defined as consts
     *
     * @return float
     */
    public function ps_round(float $value, int $precision = 0, ?int $round_mode = null): float
    {
        return PrestashopTools::ps_round($value, $precision, $round_mode);
    }

    /**
     * Get token to prevent CSRF.
     *
     * @param bool $page adjust page name to token hash
     * @param PrestashopContext|null $context adjust page name to token hash
     *
     * @return string Hashed token
     */
    public function getToken($page = true, ?PrestashopContext $context = null): string
    {
        return PrestashopTools::getToken($page, $context);
    }
}
